package com.sos.jobnet.classes;

/**
 * Created with IntelliJ IDEA.
 *
 * @uthor schaedi
 * at 09.09.13 17:16
 */
public enum ConditionState {
    CREATED       (0),          // created but not active
    MISSING       (1),          // active, but not fulfilled
    FULFILLED     (2),          // fulfilled
    IGNORED       (3),          // manually set to ignored (another kind of fulfilled)
    SKIPPED       (4);          // skipped by program (WaiterSkipped)

    private int statusValue;
    private ConditionStateText text;

    private ConditionState(int status) {
        this.statusValue = status;
        this.text = new ConditionStateText(this);
    }

    public Integer getIndex() {
        return statusValue;
    }

    public String getMsg() {
        return text.getMsg();
    }

    public static ConditionState valueOf(int index) {
        for(ConditionState s : ConditionState.values()) {
            if(s.getIndex() == index)
                return s;
        }
        throw new JobNetDAOException("The index " + index + " does not point to a valid NodeStatus.");
    }

    public boolean isFinished() {
        return(isFulfilled() || isIgnored() || isSkipped());
    }

    public boolean isFulfilled() {
        return(this == ConditionState.FULFILLED);
    }

    public boolean isIgnored() {
        return(this == ConditionState.IGNORED);
    }

    public boolean isSkipped() {
        return(this == ConditionState.SKIPPED);
    }

    public boolean isMissing() {
        return(this == ConditionState.MISSING);
    }

}
