package com.sos.jobnet.classes;

import com.sos.i18n.Logger;
import com.sos.i18n.LoggerFactory;

/**
 * List of global jobnet parameters.
 *
 * All parameters in this list will be provided to each node of the jobnet. It is possible to override the values at node level.
 */
public enum JobNetGlobalParams {

    /** the unique id of the jobnet instance - a combination of JobChainName.JobNetId */
    uuid_jobnet_identifier,

    /** the name of the parameter that contains the JobNetId additionally */
    JobnetIdParameterName,

    /** the id of the jobnet, normally a time key such yyyymmdd */
    JobNetId,

    /** the qualifier of the jobnet, normally _indx */
    JobNetIdQualifier,

    /** full name of the jobnet (liva_folder_path/JobChainName) */
    JobNet,

    /** name of the genric jobchain of the jobnet (without path) */
    JobChainName,

    /** base folder for the scripts to execute (a global parameter of scheduler.xml) */
    ScriptRoot,

    /** the option of JobNet Start */
    StartOption;

    private final Logger logger = LoggerFactory.getLogger(JobNetGlobalParams.class);

	private JobNetGlobalParams() {
	}

    /**
     * Detects if a given name correspondents to a global parameter name (cases are not matched)
     * @param name
     * @return
     */
    public static boolean isItem(String name) {
		boolean result = false;
		for (JobNetGlobalParams p : JobNetGlobalParams.values()) {
			if(p.name().equals(name)) {
				result = true;
				break;
			}
		}
		return result;
	}

    /**
     * Replaces all occurrences of the placeholders for the given parameter with a real value.
     *
     * @param replaceWith
     */
    public String replaceInString(String replaceIn, String replaceWith) {
        logger.trace("Replace {} in {} with {}.",getPlaceholder(),replaceIn,replaceWith);
        return replaceIn.replaceAll(getPlaceholder(),replaceWith);
    }

    public String getPlaceholder() {
        return "%" + name() + "%";
    }
}
