package com.sos.jobnet.classes;

/**
 * An enumeration that represents different states of a jobnet instance.
 * <ul>
 *     <li>NOT_PROCESSED: The jobnet instance is created, but not started yet.</li>
 *     <li>RUNNING: The jobnet instance is currently running.</li>
 *     <li>FINISHED: The jobnet instance is successfully finished.</li>
 *     <li>ERROR: The jobnet instance has an error.</li>
 * </ul>
 * @author ss
 * @version 02.09.13 - 15:00
 */
public enum JobNetStatus {
    NOT_PROCESSED(0),
    RUNNING      (1),
    FINISHED     (2),
    ERROR        (3);

    private final int statusValue;
    private JobNetStatusText text;

    private JobNetStatus(int status) {
        this.statusValue = status;
        this.text = new JobNetStatusText(this);
    }

    public Integer getIndex() {
        return statusValue;
    }

    /**
     * Gets a JobNetStatus object via its index.
     *
     * @param index
     * @throws JobNetDAOException: if the index is not present in the Enum.
     * @return JobNetStatus
     */
    public static JobNetStatus valueOf(int index) {
        for(JobNetStatus s : JobNetStatus.values()) {
            if(s.getIndex() == index)
                return s;
        }
        throw new JobNetDAOException("The index " + index + " does not point to a valid JobNetStatus.");
    }

    /**
     * The I18N text from the resource bundle of this project.
     * @return String
     */
    public String getMsg() {
        return text.getMsg();
    }

    public boolean isNotProcessed() {
        return(this == JobNetStatus.NOT_PROCESSED);
    }

    public boolean isRunning() {
        return(this == JobNetStatus.RUNNING);
    }

    public boolean isFinished() {
        return(this == JobNetStatus.FINISHED);
    }

    public boolean isError() {
        return(this == JobNetStatus.ERROR);
    }
}
