package com.sos.jobnet.creator;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnet.options.JobNetIdCalculatorOptions;
import com.sos.jobnet.utils.NamedDurationFieldType;
import com.sos.jobnet.utils.TimeKeyCalculator;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Arrays;
import java.util.List;

public class JobNetIdCalculator  {

	private final Logger logger = LoggerFactory.getLogger(JobNetIdCalculator.class);
	
    private final String loadId;
    private final String loadIdFormat;
    private final String loadIdType;
    private final String rule;
    private final String targetFormat;
    private final List<String> targetFields;

    public JobNetIdCalculator(JobNetIdCalculatorOptions options, boolean isOrderJob) {
    	
    	try {
			options.CheckMandatory();
		} catch (Exception e) {
			String msg = "Error checking options for " + JobNetIdCalculator.class.getName();
			logger.error(msg,e);
			throw new JobSchedulerException(msg,e);
		}

        this.loadIdType = options.getLoadIdDurationType().Value();
        this.rule = options.getCalculationRule().Value();
        this.targetFormat = options.getResultDateFormat().Value();
        this.loadIdFormat = options.getLoadIdDateFormat().Value();
        if (options.LoadId.Value().isEmpty()) {
            logger.info("Parameter LoadId is empty - using current time stamp with key " + loadIdFormat);
            DateTimeFormatter formatter = DateTimeFormat.forPattern(loadIdFormat);
            options.LoadId.Value(DateTime.now().toString(formatter));
        }
        this.loadId = getFormattedDate(options.getLoadId().Value());


        String targetFieldsString = options.getResultParameterName().Value();
        this.targetFields = Arrays.asList(targetFieldsString.split(","));

        logger.info("Processing time key ...: " + loadId);
        logger.info("Expected format .......: " + loadIdFormat);
        logger.info("Type of calculation ...: " + loadIdType);
        logger.info("Calculation rule ......: " + rule);
        logger.info("Target format .........: " + targetFormat);
        logger.info("Target fields..........: " + targetFieldsString + " (" + (isOrderJob ? "order" : "job") + ")");
		
	}
    
    private String getFormattedDate(String input) {
    	String result = input;
    	if(result.toLowerCase().equals("*now")) {
    		result = DateTimeFormat.forPattern(loadIdFormat).print(new DateTime());
    	}
    	return result;
    }

	public String execute() {
		String result = "";
		try {
	        TimeKeyCalculator calc = new TimeKeyCalculator(loadId, DateTimeFormat.forPattern(loadIdFormat), NamedDurationFieldType.of(loadIdType));
	        result = calc.calculate(rule);
	        if(!targetFormat.isEmpty()) {
                result = calc.convert( DateTimeFormat.forPattern(targetFormat) );
            }
	        logger.info("Result value ..........: " + result);
        } catch (Exception e) {
            throw new JobSchedulerException(e);
        }
		return result;
	}
	
	public List<String> getTargetField() {
		return targetFields;
	}

}
