package com.sos.jobnet.creator.jobs;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Logging.Log4JHelper;
import com.sos.jobnet.creator.JobNetProcessor;
import com.sos.jobnet.options.JobNetCreatorOptions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * \class 		JobNetCreatorMain - Main-Class for "creates the job net for a given bootstrap order"
 *
 * \brief MainClass to launch JobNetCreator as an executable command-line program
 *
 * This Class JobNetCreatorMain is the worker-class.
 *

 *
 * see \see C:\Users\schaedi\AppData\Local\Temp\scheduler_editor-3377211190692187569.html for (more) details.
 *
 * \verbatim ;
 * mechanicaly created by C:\ProgramData\sos-berlin.com\jobscheduler\scheduler-1.3.12.2032\config\JOETemplates\java\xsl\JSJobDoc2JSMainClass.xsl from http://www.sos-berlin.com at 20120228153423 
 * \endverbatim
 */
public class JobNetProcessorMain extends JSToolBox {
	private final static String					conClassName						= "JobNetProcessorMain"; //$NON-NLS-1$
	private static Logger		logger			= LoggerFactory.getLogger(JobNetProcessorMain.class);
	@SuppressWarnings("unused")	
	private static Log4JHelper	objLogger		= null;
	
	@SuppressWarnings("unused")
	private	final String		conSVNVersion			= "$Id: JobNetProcessorMain.java 16850 2012-03-21 12:14:23Z ss $";

	protected JobNetCreatorOptions	objOptions			= null;

	/**
	 * 
	 * \brief main
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param pstrArgs
	 * @throws Exception
	 */
	public final static void main(String[] pstrArgs) {

		final String conMethodName = conClassName + "::Main"; //$NON-NLS-1$

		logger.info("JobNetCreator - Main"); //$NON-NLS-1$

		try {
			JobNetCreatorOptions options = new JobNetCreatorOptions();
			options.CommandLineArgs(pstrArgs);
			options.CheckMandatory();
			JobNetProcessor processor = new JobNetProcessor(options);
			processor.run();
		}
		
		catch (Exception e) {
			System.err.println(conMethodName + ": " + "Error occured ..." + e.getMessage()); 
			e.printStackTrace(System.err);
			int intExitCode = 99;
			logger.error(String.format("JSJ-E-105: %1$s - terminated with exit-code %2$d", conMethodName, intExitCode), e);		
			System.exit(intExitCode);
		}
		
		logger.info(String.format("JSJ-I-106: %1$s - ended without errors", conMethodName));		
	}

}  // class JobNetCreatorMain
