package com.sos.jobnet.dataprovider;

import com.sos.hibernate.classes.DbItem;
import com.sos.hibernate.classes.SOSSearchFilterData;
import com.sos.hibernate.interfaces.ISOSDashboardDataProvider;
import com.sos.hibernate.interfaces.ISOSHibernateDataProvider;
import com.sos.jobnet.classes.SosJobNetPlanTableItem;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnet.db.JobNetPlanFilter;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Table;
import org.hibernate.Session;
import org.joda.time.DateTime;

import java.io.File;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.prefs.Preferences;


/**
 * \class JobNetDataProvider
 * 
 * \brief JobNetDataProvider -
 * 
 * \details
 * 
 * \section JobNetDataProvider.java_intro_sec Introduction
 * 
 * \section JobNetDataProvider.java_samples Some Samples
 * 
 * \code .... code goes here ... \endcode
 * 
 * <p style="text-align:center">
 * <br />
 * --------------------------------------------------------------------------- <br />
 * APL/Software GmbH - Berlin <br />
 * ##### generated by ClaviusXPress (http://www.sos-berlin.com) ######### <br />
 * ---------------------------------------------------------------------------
 * </p>
 * \author Uwe Risse \version 19.01.2012 \see reference
 * 
 * Created on 19.01.2012 09:31:01
 */

public class JobNetPlanDataProvider implements ISOSHibernateDataProvider, ISOSDashboardDataProvider {

    @SuppressWarnings("unused")
    private final String conClassName = "JobNetDataProvider";

    private List<JobNetPlanDBItem> listOfJobNetPlanDBItems = null;
    private JobNetPlanDBLayer jobNetPlanDBLayer = null;
    private static Logger logger = Logger.getLogger(JobNetPlanDataProvider.class);
    private Table tableJobNetPlan = null;

    public JobNetPlanDataProvider(File configurationFile, String schedulerId) {
        this.jobNetPlanDBLayer = new JobNetPlanDBLayer(configurationFile, schedulerId);
    }

    public JobNetPlanDataProvider(File configurationFile) {
        this.jobNetPlanDBLayer = new JobNetPlanDBLayer(configurationFile);
    }
    
    public JobNetPlanFilter getFilter() {
        return jobNetPlanDBLayer.getFilter();
    }
    
    public void  setFilter(JobNetPlanFilter f) {
        jobNetPlanDBLayer.setFilter(f);
    }

    public void resetFilter() {
        jobNetPlanDBLayer.setFilter(new JobNetPlanFilter());
    }

    public void getData(int limit) {
        listOfJobNetPlanDBItems = jobNetPlanDBLayer.getJobnetPlanList(limit);
    }

    public void fillSchedulerIds(CCombo cbSchedulerId) {
        if (tableJobNetPlan != null) {
            Iterator jobNetPlanEntries = listOfJobNetPlanDBItems.iterator();
            while (jobNetPlanEntries.hasNext()) {
                JobNetPlanDBItem h = (JobNetPlanDBItem) jobNetPlanEntries.next();
                if (cbSchedulerId.indexOf(h.getJobnetNodeDBItem().getSchedulerId()) < 0) {
                    logger.debug("... cbSchedulerId --> : " + h.getJobnetNodeDBItem().getSchedulerId());
                    cbSchedulerId.add(h.getJobnetNodeDBItem().getSchedulerId());
                }
            }
        }
    }

   

    public void fillTable(Table table) {
        this.tableJobNetPlan = table;
        Iterator<JobNetPlanDBItem> dailyScheduleEntries = listOfJobNetPlanDBItems.iterator();
        while (dailyScheduleEntries.hasNext()) {
            JobNetPlanDBItem h = dailyScheduleEntries.next();
            if (jobNetPlanDBLayer.getFilter().isFiltered(h)) {
            } else {

                final SosJobNetPlanTableItem newItemTableItem = new SosJobNetPlanTableItem(table, SWT.BORDER);
                newItemTableItem.setDBItem(h);
                newItemTableItem.setColor();
                newItemTableItem.setColumns();
            }
        }
    }

    public File getConfigurationFile() {
        return jobNetPlanDBLayer.getConfigurationFile();

    }

    @Override
    public void beginTransaction() {
        jobNetPlanDBLayer.beginTransaction();
    }

    public void save(JobNetPlanDBItem dbItem) {
        jobNetPlanDBLayer.beginTransaction();
        jobNetPlanDBLayer.saveOrUpdate(dbItem);
        jobNetPlanDBLayer.commit();
    }

    @Override
    public void update(DbItem dbItem) {
        jobNetPlanDBLayer.update(dbItem);
    }

    public Session getSession() {
        return jobNetPlanDBLayer.getSession();
    }

    @Override
    public void commit() {
        jobNetPlanDBLayer.commit();
    }

    @Override
    public void setSchedulerId(String schedulerId) {
        this.getFilter().setSchedulerId(schedulerId);
    }

    @Override
    public void setFrom(Date d) {
        this.getFilter().setPlannedStartTimeFrom( new DateTime(d) );
    }

    @Override
    public void setTo(Date d) {
        this.getFilter().setPlannedStartTimeTo( new DateTime(d) );
    }

    @Override
    public void setSearchField(SOSSearchFilterData s) {
        this.getFilter().setSearchFiled(s.getSearchfield());
    }

    @Override
    public void setShowJobs(boolean b) {
     }

    @Override
    public void setShowJobChains(boolean b) {
     }

    @Override
    public void setIgnoreList(Preferences prefs) {
     }

    @Override
    public void addToIgnorelist(Preferences prefs, DbItem h) {
     }

    @Override
    public void disableIgnoreList(Preferences prefs) {
     }

    @Override
    public void resetIgnoreList(Preferences prefs) {
     }

    @Override
    public void setLate(boolean b) {
     }

    @Override
    public void setStatus(String status) {
     }

    @Override
    public void setShowWithError(boolean b) {
        // TODO Auto-generated method stub
        
    }

	@Override
	public void setShowRunning(boolean b) {
		// TODO Auto-generated method stub
		
	}

}
