package com.sos.jobnet.db;

import com.sos.hibernate.layer.SOSHibernateDBLayer;
import com.sos.jobnet.classes.JobNetDAOConstants;
import com.sos.jobnet.classes.JobNetDAOException;
import org.hibernate.Query;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;

// import org.slf4j.Logger;

/**
 * 
 * \class JobNetCmdHistoryDBLayer \brief JobNetCmdHistoryDBLayer -
 * 
 * \details
 * 
 * \section JobNetCmdHistoryDBLayer.java_intro_sec Introduction
 * 
 * \section JobNetCmdHistoryDBLayer.java_samples Some Samples
 * 
 * \code .... code goes here ... \endcode
 * 
 * <p style="text-align:center">
 * <br />
 * --------------------------------------------------------------------------- <br />
 * APL/Software GmbH - Berlin <br />
 * ##### generated by ClaviusXPress (http://www.sos-berlin.com) ######### <br />
 * ---------------------------------------------------------------------------
 * </p>
 * \author Uwe Risse \version 13.08.2013 \see reference
 * 
 * Created on 13.08.2013 14:40:18
 */

public class JobNetCmdHistoryDBLayer extends SOSHibernateDBLayer {
	

 
    private static final String START_OPTION = "startOption";

    private static final String NODE_TYPE = "nodeType";
    private static final String UUID = "uuid";

    private static final String START_TIME = "START_TIME";

    private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    private static final String START_TIME_TO = "startTimeTo";
    private static final String START_TIME_FROM = "startTimeFrom";
    private static final String HISTORY_ID = "historyId";
    
    private final static Logger logger = LoggerFactory.getLogger(JobNetCmdHistoryDBLayer.class);

    @SuppressWarnings("unused")
    private final String conClassName = "JobNetCmdHistoryDBLayer";
    public final String conSVNVersion = "$Id: JobNetCmdHistoryDBLayer.java 20037 2013-05-03 04:55:36Z ss $";


 

    private JobNetCmdHistoryFilter filter = null;


    public JobNetCmdHistoryDBLayer(File configurationFile_, String schedulerId_) {
        super();
        this.filter = new JobNetCmdHistoryFilter();
        this.setConfigurationFile(configurationFile_);
        this.filter.setDateFormat(DATE_FORMAT);
        this.filter.setOrderCriteria(START_TIME);
     }

    public JobNetCmdHistoryDBLayer(File configurationFile_) {
        super();
        this.filter = new JobNetCmdHistoryFilter();

        this.setConfigurationFile(configurationFile_);
        this.filter.setDateFormat(DATE_FORMAT);
        this.filter.setOrderCriteria(START_TIME);
     }
    
    private Query setQueryParams(String hql) {
        Query query = session.createQuery(hql);

        if (filter.getStartTimeFrom() != null) {
            query.setTimestamp(START_TIME_FROM, filter.getStartTimeFrom().toDate());
        }

        if (filter.getStartTimeTo() != null) {
            query.setTimestamp(START_TIME_TO, filter.getStartTimeTo().toDate());
        }

        if (filter.getNodeType() != null && !filter.getNodeType().equals("")) {
            query.setParameter(NODE_TYPE, filter.getNodeType());
        }
        
        if (filter.getUuid() != null && !filter.getUuid().equals("")) {
            query.setParameter(UUID, filter.getUuid());
        }

        if (filter.getStartOption() != null && !filter.getStartOption().equals("")) {
            query.setParameter(START_OPTION, filter.getStartOption());
        }
 
        if (filter.getHistoryId() != null   ) {
            query.setParameter(HISTORY_ID, filter.getHistoryId());
        }

         
        return query;
    }

    public int delete() {

        String hql = "delete from JobNetCmdHistoryDBItem " + getWhere();

        Query query = setQueryParams(hql);
        int row = query.executeUpdate();

        return row;
    }

    private String getWhere() {
        String where = "";
        String and = "";

        if (filter.getStartTimeFrom() != null) {
            where += and + " startTime>= :startTimeFrom";
            and = " and ";
        }

        if (filter.getStartTimeTo() != null && !filter.getStartTimeTo().equals("")) {
            where += and + " startTime< :startTimeTo";
            and = " and ";
        }

        if (filter.getNodeType() != null && !filter.getNodeType().equals("")) {
            where += and + " nodeType = :nodeType";
            and = " and "; 
        }

        if (filter.getUuid() != null && !filter.getUuid().equals("")) {
            where += and + " jobnet.jobNetHistoryDBItem.uuid = :uuid";
            and = " and "; 
        }

        if (filter.getStartOption() != null && !filter.getStartOption().equals("")) {
            where += and + " startOption = :startOption";
            and = " and ";
        }
      
        if (filter.getHistoryId() != null) {
            where += and + " historyId = :historyId";
            and = " and ";
        }
 
        if (where.trim().equals("")) {

        } else {
            where = "where " + where;
        }
        return where;

    }

    public void save(JobNetCmdHistoryDBItem item) {
        initSession();
        super.save(item);
    }

    public List<JobNetCmdHistoryDBItem> getJobNetCmdHistoryList(int limit) {
        initSession();
     

        Query query = setQueryParams("from JobNetCmdHistoryDBItem jobnet " + getWhere() + this.filter.getOrderCriteria() + this.filter.getSortMode());

        if (limit > 0) {
            query.setMaxResults(limit);
        }

        @SuppressWarnings("unchecked")
        List<JobNetCmdHistoryDBItem> jobnetHistoryList = query.list();
        return jobnetHistoryList;

    }


    public List<JobNetCmdHistoryDBItem> getJobNetCmdHistoryList(JobNetHistoryDBItem jobNetHistoryDBItem, int limit) {
        return getJobNetCmdHistoryListByHistoryId(jobNetHistoryDBItem.getHistoryId(),limit);

    }

    public List<JobNetCmdHistoryDBItem> getJobNetCmdHistoryListByHistoryId(Long historyId, int limit) {
        initSession();

        resetFilter();
        filter.setHistoryId(historyId);

        Query query = setQueryParams("from JobNetCmdHistoryDBItem jobnet " + getWhere() + this.filter.getOrderCriteria() + this.filter.getSortMode());

        if (limit > 0) {
            query.setMaxResults(limit);
        }

        List<JobNetCmdHistoryDBItem> jobnetHistoryList = query.list();
        return jobnetHistoryList;

    }

    /**
     * Search the command entry with the latest StartTime. This is the command submitted at last.
     * @param historyId
     * @return
     */
    public JobNetCmdHistoryDBItem getLastJobNetCmdHistoryEntry(Long historyId) {
        List<JobNetCmdHistoryDBItem> records = getJobNetCmdHistoryListByHistoryId(historyId,0);
        JobNetCmdHistoryDBItem result = null;
        if(records.size() == 0) {
            logger.warn("No command history found for history id " + historyId);
        } else {
            DateTime maxDate = null;
            for(JobNetCmdHistoryDBItem item : records) {
                if(maxDate == null || item. getStartTime().isAfter(maxDate)) {
                    maxDate = item.getStartTime();
                    result = item;
                }
            }
            logger.info("The last command submitted is " + result.getStartOption());
        }
        return result;
    }

   
    
    public JobNetCmdHistoryDBItem createJobNetCmdHistory(Long nodeId, Long historyId, String startMode, String message) {
        JobNetCmdHistoryDBItem result = new JobNetCmdHistoryDBItem();
        try {
            this.beginTransaction();
            result.setHistoryId(historyId);
            result.setStartOption(startMode);
            result.setNodeId(nodeId);
            result.setAcknowledged(false);
            result.setMessage(message);
            result.setStartTime( new DateTime() );
            result.setCreated( new DateTime() );
            result.setCreatedBy(JobNetDAOConstants.CURRENT_USER);
            this.save(result);
            this.commit();
            logger.info("Command history record created for StartOption " + startMode + " referencing history record with id " + historyId);
        } catch (Exception e) {
            throw new JobNetDAOException("Error creating record in table SCHEDULER_JOB_NET_CMD_HISTORY.",e);
        }
        return result;
    }

 
  
    public JobNetCmdHistoryFilter getFilter() {
        return filter;
    }

    public void resetFilter() {
        filter = new JobNetCmdHistoryFilter();
    }

    public void setFilter(JobNetCmdHistoryFilter filter) {
        this.filter = filter;
    }
	
 

}
