package com.sos.jobnet.db;

import com.sos.hibernate.layer.SOSHibernateDBLayer;
import com.sos.jobnet.classes.InternalJobNetError;
import com.sos.jobnet.classes.JobNetDAOConstants;
import com.sos.jobnet.classes.JobNetDAOException;
import com.sos.jobnet.classes.JobNetStatus;
import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.Query;
import org.joda.time.DateTime;
import org.joda.time.Interval;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;

// import org.slf4j.Logger;

/**
 * 
 * \class SchedulerJobNetHistoryDBLayer \brief SchedulerJobNetHistoryDBLayer -
 * 
 * \details
 * 
 * \section SchedulerJobNetHistoryDBLayer.java_intro_sec Introduction
 * 
 * \section SchedulerJobNetHistoryDBLayer.java_samples Some Samples
 * 
 * \code .... code goes here ... \endcode
 * 
 * <p style="text-align:center">
 * <br />
 * --------------------------------------------------------------------------- <br />
 * APL/Software GmbH - Berlin <br />
 * ##### generated by ClaviusXPress (http://www.sos-berlin.com) ######### <br />
 * ---------------------------------------------------------------------------
 * </p>
 * \author Uwe Risse \version 13.08.2013 \see reference
 * 
 * Created on 13.08.2013 14:40:18
 */

public class JobNetHistoryDBLayer extends SOSHibernateDBLayer {
	

 
    private static final Boolean INCREASE_NUMBER_OF_FINSIHED_NODES = true;

    private static final String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";

    private static final String START_TIME_TO = "startTimeTo";
    private static final String START_TIME_FROM = "startTimeFrom";
    private static final String NET_ID = "netId";
    private static final String HISTORY_ID = "historyId";
    private static final String EXIT_CODE = "exitCode";
    private static final String JOBNET_ID = "jobnetId";
    private static final String STATUS = "status";
    private static final String STATUSFINISHED = "statusFinished";
    private static final String STATUSERROR = "statusError";
    private static final String JOBNET = "jobNet";
    private static final String UUID = "uuid";
    private static final String SCHEDULER_ID = "schedulerId";
    
    private final static Logger logger = LoggerFactory.getLogger(JobNetHistoryDBLayer.class);

    @SuppressWarnings("unused")
    private final String conClassName = "SchedulerJobNetHistoryDBLayer";
    public final String conSVNVersion = "$Id: SchedulerJobNetHistoryDBLayer.java 20037 2013-05-03 04:55:36Z ss $";


 

    private JobNetHistoryFilter filter = null;


    public JobNetHistoryDBLayer(File configurationFile_, String schedulerId_) {
        super();

        this.filter = new JobNetHistoryFilter();

        this.filter.setSchedulerId(schedulerId_);

        this.setConfigurationFile(configurationFile_);
        this.filter.setDateFormat(DATE_FORMAT);
        this.filter.setOrderCriteria(UUID);
        this.filter.setSchedulerId(schedulerId_);
        initSession();
    }

    public JobNetHistoryDBLayer(File configurationFile_) {
        super();
        this.filter = new JobNetHistoryFilter();

        this.setConfigurationFile(configurationFile_);
        this.filter.setDateFormat(DATE_FORMAT);
        this.filter.setOrderCriteria(UUID);
        initSession();
     }
    
    private Query setQueryParams(String hql) {
        Query query = session.createQuery(hql);

        if (filter.getStartTimeFrom() != null && !filter.getStartTimeFrom().equals("")) {
            query.setTimestamp(START_TIME_FROM, filter.getStartTimeFrom().toDate());
        }

        if (filter.getStartTimeTo() != null && !filter.getStartTimeTo().equals("")) {
            query.setTimestamp(START_TIME_TO, filter.getStartTimeTo().toDate());
        }

        if (filter.getSchedulerId() != null && !filter.getSchedulerId().equals("")) {
            query.setParameter(SCHEDULER_ID, filter.getSchedulerId());
        }

        if (filter.getUuid() != null && !filter.getUuid().equals("")) {
            query.setParameter(UUID, filter.getUuid());
        }

        if (filter.getFinishedStatus()) {
            query.setInteger(STATUSFINISHED, JobNetStatus.FINISHED.getIndex());
            query.setInteger(STATUSERROR, JobNetStatus.ERROR.getIndex());
        }
        
        if (filter.getStatus() != null && !filter.getStatus().equals("")) {
            query.setInteger(STATUS, filter.getStatus().getIndex());
        }
        
       
        
        if (filter.getHistoryId() != null   ) {
            query.setParameter(HISTORY_ID, filter.getHistoryId());
        }

        if (filter.getNetId() != null ) {
            query.setParameter(NET_ID, filter.getNetId());
        }

        if (filter.getExitCode() != null   ) {
            query.setParameter(EXIT_CODE, filter.getExitCode());
        }
        
        if (filter.getJobnetId() != null   ) {
            query.setParameter(JOBNET_ID, filter.getJobnetId());
        }

        return query;
    }

    public int delete() {

        String hql = "delete from JobNetHistoryDBItem " + getWhere();

        Query query = setQueryParams(hql);
        int row = query.executeUpdate();

        return row;
    }

    private String getWhere() {
        String where = "";
        String and = "";

        if (filter.getStartTimeFrom() != null && !filter.getStartTimeFrom().equals("")) {
            where += and + " startOfJobnet>= :startTimeFrom";
            and = " and ";
        }

        if (filter.getStartTimeTo() != null && !filter.getStartTimeTo().equals("")) {
            where += and + " startOfJobnet< :startTimeTo";
            and = " and ";
        }

        if (!filter.getSchedulerId().equals("")) {
            where += and + " schedulerId = :schedulerId";
            and = " and "; 
        }

        if (filter.getUuid() != null && !filter.getUuid().equals("")) {
            where += and + " uuid = :uuid";
            and = " and ";
        }
      
        if (filter.getNetId() != null) {
            where += and + " netId = :netId";
            and = " and ";
        }

        if (filter.getStatus() != null) {
            where += and + " status  = :status";
            and = " and ";
        }

        if (filter.getFinishedStatus()) {
            where += and + " status  = :statusFinished or status = :statusError";
            and = " and ";
        }

        if (filter.getExitCode() != null) {
            where += and + " exitCode  = :exitCode";
            and = " and ";
        }     
        
        if (filter.getHistoryId() != null) {
            where += and + " historyId  = :historyId";
            and = " and ";
        }     
                
        if (filter.getJobnetId() != null && !filter.getJobnetId().equals("")) {
            where += and + " jobnetId = :jobnetId";
            and = " and ";
        }

        if (where.trim().equals("")) {

        } else {
            where = "where " + where;
        }
        return where;

    }

    public List<JobNetHistoryDBItem> getSchedulerJobNetHistoryList(int limit) {
        initSession();
     

        Query query = setQueryParams("from JobNetHistoryDBItem jobnet " + getWhere() + this.filter.getOrderCriteria() + this.filter.getSortMode());

        if (limit > 0) {
            query.setMaxResults(limit);
        }

        @SuppressWarnings("unchecked")
        List<JobNetHistoryDBItem> jobnetHistoryList = query.list();
        return jobnetHistoryList;

    }

    public List<JobNetHistoryDBItem> getSchedulerJobNetHistoryListAndLock(int limit) {
        initSession();


        Query query = setQueryParams("from JobNetHistoryDBItem jobnet " + getWhere() + this.filter.getOrderCriteria() + this.filter.getSortMode());
        query.setLockOptions(new LockOptions(LockMode.PESSIMISTIC_WRITE));

        if (limit > 0) {
            query.setMaxResults(limit);
        }

        @SuppressWarnings("unchecked")
        List<JobNetHistoryDBItem> jobnetHistoryList = query.list();
        return jobnetHistoryList;

    }


    public List<JobNetHistoryDBItem> getSchedulerJobNetHistoryList(JobNetDBItem schedulerJobNetDBItem, int limit) {
        initSession();
     
       // resetFilter();
        filter.setNetId(schedulerJobNetDBItem.getNetId());

        Query query = setQueryParams("from JobNetHistoryDBItem jobnet " + getWhere() + this.filter.getOrderCriteria() + this.filter.getSortMode());

        if (limit > 0) {
            query.setMaxResults(limit);
        }

        @SuppressWarnings("unchecked")
        List<JobNetHistoryDBItem> jobnetHistoryList = query.list();
        return jobnetHistoryList;

    }

   
  
    public JobNetHistoryFilter getFilter() {
        return filter;
    }

    public void resetFilter() {
        filter = new JobNetHistoryFilter();
    }

    public void setFilter(JobNetHistoryFilter filter) {
        this.filter = filter;
    }
	
	public void deleteJobnetInstance(String uuid) {
		JobNetHistoryFilter filter = new JobNetHistoryFilter();
		filter.setUuid(uuid);
		setFilter(filter);
    	if (!getSession().isOpen()) initSession();
		Query query = setQueryParams("from JobNetHistoryDBItem plan " + getWhere());
		@SuppressWarnings("unchecked")
		List<JobNetHistoryDBItem> resultList = query.list();
		logger.info(resultList.size() + " records deleted for jobnet instance " + uuid + " deleted.");
		for(JobNetHistoryDBItem item : resultList) {
			delete(item);
		}
	}

    // update the history record for the jobnet
    public JobNetHistoryDBItem startHistory(String uuid) {
        JobNetHistoryDBItem historyRecord = getJobNetInstanceAndLock(uuid);
        DateTime d = historyRecord.getStartOfJobnet();
        if(d != null)
            throw new JobNetDAOException("The jobnet with uuid " + historyRecord.getUuid() + " is already started.");
        DateTime now = new DateTime();
        beginTransaction();
        historyRecord.setStatus(JobNetStatus.RUNNING.getIndex());
        historyRecord.setStartOfJobnet(now);
        historyRecord.setRunTime(now);
        historyRecord.setModified(now);
        historyRecord.setModifiedBy(JobNetDAOConstants.CURRENT_USER);
        update(historyRecord);
        commit();;
        return historyRecord;
    }

    // update the history record for the jobnet
    private Boolean updateHistory(String uuid, Boolean increaseNumberOfFinishedNodes) {
        JobNetHistoryDBItem historyRecord = getJobNetInstanceAndLock(uuid);
        Boolean jobNetEnded = false;
        DateTime now = new DateTime();
        beginTransaction();
        historyRecord.setRunTime(now);
        historyRecord.setModified(now);
        historyRecord.setModifiedBy(JobNetDAOConstants.CURRENT_USER);
        if(increaseNumberOfFinishedNodes) {
            Integer finishedNodes = historyRecord.getNumberOfFinishedNodes();
            finishedNodes++;
            historyRecord.setNumberOfFinishedNodes(finishedNodes);
            if(finishedNodes.equals(historyRecord.getNumberOfNodes())) {
                boolean hasError = (historyRecord.getExitCode()!=0);
                JobNetStatus endState = (hasError) ? JobNetStatus.ERROR : JobNetStatus.FINISHED;
                historyRecord.setStatus(endState.getIndex());
                historyRecord.setEndOfJobnet(now);
                jobNetEnded = true;
            }
        }
        update(historyRecord);
        commit();
        return jobNetEnded;
    }

    // update the history record for the jobnet
    public Boolean updateHistoryAndIncreaseFinishedOrders(String uuid) {
        return updateHistory(uuid,INCREASE_NUMBER_OF_FINSIHED_NODES);
    }

    // update the history record for the jobnet
    public void updateHistory(String uuid) {
        updateHistory(uuid,!INCREASE_NUMBER_OF_FINSIHED_NODES);
    }

    // update the history record for the jobnet
    public void updateHistoryWithError(String uuid, InternalJobNetError error, String... parameter) {
        JobNetHistoryDBItem historyRecord = getJobNetInstanceAndLock(uuid);
        DateTime now = new DateTime();
        beginTransaction();
        historyRecord.setEndOfJobnet(now);
        historyRecord.setRunTime(now);
        historyRecord.setModified(now);
        Integer currentExitCode = historyRecord.getExitCode();
        if (error.getExitCode() >= currentExitCode) {
            historyRecord.setExitCode(error.getExitCode());
            historyRecord.setExitMessage(error.getMsg());
        }
        historyRecord.setModifiedBy(JobNetDAOConstants.CURRENT_USER);
        update(historyRecord);
        commit();
    }

    public Interval getRunTime(JobNetHistoryDBItem historyRecord) {
        DateTime fromDate = new DateTime(historyRecord.getStartOfJobnet());
        DateTime toDate = new DateTime(historyRecord.getRunTime());
        return new Interval(fromDate, toDate);
    }

    public JobNetHistoryDBItem getJobNetInstance(String uuid) {
        logger.info("Search history record for uuid " + uuid);
        JobNetHistoryFilter filter = new JobNetHistoryFilter();
        filter.setUuid(uuid);
        setFilter(filter);
        List<JobNetHistoryDBItem> records = getSchedulerJobNetHistoryList(0);
        if (records.size() > 1)
            throw new JobNetDAOException("More than one instance detected for " + uuid + " in table SCHEDULER_JOB_NET_HISTORY");
        if (records.size() == 0)
            logger.info("No history record found for jobnet instance with uuid " + uuid + ".");
        return (records.size() == 1) ? records.get(0) : null;
    }

    public JobNetHistoryDBItem getJobNetInstanceAndLock(String uuid) {
        logger.info("Search history record for uuid " + uuid);
        JobNetHistoryFilter filter = new JobNetHistoryFilter();
        filter.setUuid(uuid);
        setFilter(filter);
        List<JobNetHistoryDBItem> records = getSchedulerJobNetHistoryListAndLock(0);
        if (records.size() > 1)
            throw new JobNetDAOException("More than one instance detected for " + uuid + " in table SCHEDULER_JOB_NET_HISTORY");
        if (records.size() == 0)
            logger.info("No history record found for jobnet instance with uuid " + uuid + ".");
        return (records.size() == 1) ? records.get(0) : null;
    }

    public boolean isJobNetInstance(String uuid) {
        JobNetHistoryDBItem record = getJobNetInstance(uuid);
        return (record != null);
    }

    public boolean isJobnetRunning(JobNetHistoryDBItem historyRecord) {
        return (historyRecord.getNumberOfFinishedNodes() != 0 && historyRecord.getEndOfJobnet() == null);
    }

    public boolean isJobnetEnded(JobNetHistoryDBItem historyRecord) {
        return (historyRecord.getEndOfJobnet() != null);
    }

}
