package com.sos.jobnet.db;

import com.sos.jobnetmodel.objects.NodeStatus;

import java.util.ArrayList;
import java.util.List;

/**
 * A representation of the jobnet as tree.
 *
 * @author ss
 * at 05.09.13 11:58
 */
public class JobNetPlanTree {

    private final List<JobNetPlanTree> successors;
    private final JobNetPlanDBItem node;
    private JobNetPlanTree parent = null;
    private NodeStatus statusForDisplay;

    public JobNetPlanTree(JobNetPlanDBItem node) {
        this.node = node;
        this.statusForDisplay = NodeStatus.valueOf(node.getStatusNode());
        this.successors = new ArrayList<JobNetPlanTree>();
    }

    /**
     * Add a successor node to the JobNet tree. The status of the node will be transferred to all predecessor nodes in the tree
     * while its status rank is higher than the rank of its predecessor node.
     *
     * @param node
     * @return
     */
    public JobNetPlanTree addSuccessor(JobNetPlanDBItem node) {
        JobNetPlanTree newTree = new JobNetPlanTree(node);
        newTree.parent = this;
        successors.add( newTree );
        updateStatusRank(newTree.parent,NodeStatus.valueOf(node.getStatusNode()));
        return newTree;
    }

    /**
     * Set the status of the parent nodes.
     *
     * @param currentTreeNode
     * @param statusToUpdate
     */
    private void updateStatusRank(JobNetPlanTree currentTreeNode, NodeStatus statusToUpdate) {
        NodeStatus currentStatus = currentTreeNode.statusForDisplay;
        if(statusToUpdate.getStatusRank() > currentStatus.getStatusRank()) {
            currentTreeNode.statusForDisplay = statusToUpdate;
            if(currentTreeNode.parent != null)          // null = top of the tree
                updateStatusRank(currentTreeNode.parent,statusToUpdate);
        }
    }

    /**
     * The database item of the node.
     *
     * @return
     */
    public JobNetPlanDBItem getJobNetPlanDBItem() {
        return node;
    }

    /**
     * Get the successor nodes of a tree node.
     * @return
     */
    public List<JobNetPlanTree> getSuccessors() {
        return successors;
    }

    public NodeStatus getStatusForDisplay() {
        return statusForDisplay;
    }

}
