package com.sos.jobnet.frontend;

import java.util.prefs.Preferences;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;

 import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
 
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;


import com.sos.jid.dialog.classes.SOSDashboardTableView;
import com.sos.dashboard.globals.SOSDashboardOptions;
import com.sos.jobnet.dialog.SchedulerJobNetDataProvider;
import com.sos.jobnet.dialog.classes.SOSTabGraphicalOverview;
import com.sos.jobnet.dialog.classes.SOSTabJobnetCmdHistory;
import com.sos.jobnet.dialog.classes.SOSTabJobnetMonitor;
import com.sos.jobnet.frontend.JobnetShowGraph;
 

@SuppressWarnings("deprecation")
public class JobnetMainWindow extends SOSDashboardTableView {

    private static final String     TABNAME_JOBNET      = "Jobnet";
    private static final String     TABNAME_MONITORING  = "Monitoring";
    private static final String     TABNAME_CMD_HISTORY = "Commands";

    private final Composite         parentComposite;
    private final GridLayout        layout;
    private boolean                 haveDb              = true;         // TODO false setzen, wenn kein DB connect

    private CTabFolder              mainTabFolder       = null;

    private SOSTabJobnetMonitor     tbtmJobnetMonitor;
    private SOSTabJobnetCmdHistory  tbtmJobnetCmdHistory;
    private SOSTabGraphicalOverview tbtmGraphicalOverview;

    private JobnetShowGraph         jobnetShowGraph;
    private SOSDashboardOptions     objOptions;

    public JobnetMainWindow(final Composite parentComposite, SOSDashboardOptions objOptions_) {
        super(parentComposite);
        objOptions = objOptions_;
        this.parentComposite = parentComposite;
        mainViewComposite = parentComposite;
        layout = new GridLayout(1, false);
        parentComposite.setLayout(layout);
        parentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        createMainWindow();
        
        parentComposite.layout();
    }
    

    private void createMainWindow() {

        
        prefs = Preferences.userNodeForPackage(this.getClass());

        GridLayout layout = new GridLayout(9, false);
        mainViewComposite.setLayout(layout);
       // sosDashboardHeader = new SosDashboardHeader(mainViewComposite, this);

        mainTabFolder = new CTabFolder(parentComposite, SWT.NONE);
        mainTabFolder.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(final SelectionEvent e) {
                if (mainTabFolder.getSelectionIndex() == 0) {
                    if (haveDb) {
                        tbtmJobnetCmdHistory.disableRefresh();
                        tbtmJobnetMonitor.disableRefresh();
                        tbtmGraphicalOverview.enableRefresh();
                        tbtmGraphicalOverview.actualize();
  
                    }
                }
                if (mainTabFolder.getSelectionIndex() == 1) {
                    if (haveDb) {
                        tbtmGraphicalOverview.disableRefresh();
                        tbtmJobnetCmdHistory.disableRefresh();
                        tbtmJobnetMonitor.enableRefresh();
                        tbtmJobnetMonitor.actualize();
                        
                     }
                }
                if (mainTabFolder.getSelectionIndex() == 2) {
                    if (haveDb) {
                        tbtmGraphicalOverview.disableRefresh();
                        tbtmJobnetMonitor.disableRefresh();
                        tbtmJobnetCmdHistory.enableRefresh();
                        tbtmJobnetCmdHistory.actualize();
                    }
                }

            }
        });
        
        
       /* parentComposite.getDisplay().addFilter(SWT.KeyDown, new Listener() {
            public void handleEvent(Event event) {
     
                if (event.keyCode == SWT.F5) {
                    if (mainTabFolder.getSelectionIndex() == 0) {
                        if (haveDb) {
                            tbtmGraphicalOverview.enableRefresh();
                            tbtmGraphicalOverview.resetRefresh();
                            tbtmGraphicalOverview.actualize();
                        }
                    }
                    if (mainTabFolder.getSelectionIndex() == 1) {
                        if (haveDb) {
                            tbtmJobnetMonitor.enableRefresh();
                            tbtmJobnetMonitor.resetRefresh();
                            tbtmJobnetMonitor.actualize();
                         }
                    }
                    if (mainTabFolder.getSelectionIndex() == 2) {
                        if (haveDb) {
                            tbtmJobnetCmdHistory.enableRefresh();
                            tbtmJobnetCmdHistory.actualize();
                         }
                    }                }
                event.doit = false;
            }
        });
        */

        mainTabFolder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 9, 1));
        mainTabFolder.setSelectionBackground(Display.getCurrent().getSystemColor(SWT.COLOR_TITLE_INACTIVE_BACKGROUND_GRADIENT));

        tbtmGraphicalOverview = new SOSTabGraphicalOverview(TABNAME_JOBNET, mainTabFolder, objOptions);
        tbtmJobnetMonitor = new SOSTabJobnetMonitor(TABNAME_MONITORING, mainTabFolder, objOptions);
        tbtmJobnetCmdHistory = new SOSTabJobnetCmdHistory(TABNAME_CMD_HISTORY, mainTabFolder, objOptions);

        tbtmJobnetCmdHistory.disableRefresh();
        tbtmJobnetMonitor.disableRefresh();
        tbtmGraphicalOverview.enableRefresh();

        mainTabFolder.setSelection(0);
    }
    
   

    public void refresh() {
        
        if (mainTabFolder.getSelectionIndex() == 0) {
            if (haveDb) {
                tbtmGraphicalOverview.enableRefresh();
                tbtmGraphicalOverview.resetRefresh();
                tbtmGraphicalOverview.actualize();
            }
        }
        if (mainTabFolder.getSelectionIndex() == 1) {
            if (haveDb) {
                tbtmJobnetMonitor.enableRefresh();
                tbtmJobnetMonitor.resetRefresh();
                tbtmJobnetMonitor.actualize();
             }
        }
        if (mainTabFolder.getSelectionIndex() == 2) {
            if (haveDb) {
                tbtmJobnetCmdHistory.enableRefresh();
                tbtmJobnetCmdHistory.actualize();
             }
        }
        
     }
    
    public Composite getParentComposite() {
        return parentComposite;
    }

    public void disableRefresh() {
        jobnetShowGraph.disableRefresh();
    }

    public void enableRefresh() {
        jobnetShowGraph.resetRefreshTimer();
    }

    public SOSTabGraphicalOverview getTbtmGraphicalOverview() {
        return tbtmGraphicalOverview;
    }

    public SOSTabJobnetMonitor getTbtmJobnetMonitor() {
        return tbtmJobnetMonitor;
    }

    public SOSTabJobnetCmdHistory getTbtmJobnetCmdHistory() {
        return tbtmJobnetCmdHistory;
    }
    
    public void setSchedulerJobNetDataProvider(SchedulerJobNetDataProvider schedulerJobNetDataProvider) {
        tbtmJobnetMonitor.setSchedulerJobNetDataProvider(schedulerJobNetDataProvider);
    }

    
    
}

