package com.sos.jobnet.graph;

import com.sos.hibernate.options.HibernateOptions;
import com.sos.jobnet.context.ContextBuilder;
import com.sos.jobnet.db.JobNetEdgesDBLayer;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnetmodel.objects.JobnetAdapter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;

/*
 * Adapter class to convert the database representation of the jobnet into the internal jobnet model.
 */
public class JobNetPlanAdapter extends JobnetAdapter {
	
	private static Logger logger = LoggerFactory.getLogger(JobNetPlanAdapter.class);
	
	private final File configurationFile;
	private final JobNetPlanDBLayer jobNetPlanDBLayer;
	private final JobNetEdgesDBLayer edgesDBReader;
	private final boolean jobnetFound;
    private final JobNetPlanDBItem startNode;

	public JobNetPlanAdapter(HibernateOptions options, String uuid, String orderWithoutPath) {

		this.configurationFile 	= new File(options.hibernate_connection_config_file.Value());
	    this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(configurationFile);
		this.edgesDBReader = new JobNetEdgesDBLayer(configurationFile);

        ContextBuilder contextBuilder = new ContextBuilder(uuid, orderWithoutPath, jobNetPlanDBLayer);
        this.startNode = contextBuilder.getStartNode();

	    List<JobNetPlanDBItem> dbRecords = contextBuilder.getContextList().getNodesInContext();
	    this.jobnetFound = (dbRecords.size() == 0) ? false : true;
	    logger.info(dbRecords.size() + " read from database for uuid " + uuid);
	    for(JobNetPlanDBItem record : dbRecords) {
            if(record.getPlanId() == startNode.getPlanId())
                record.setBootstrap(true);                      // the adapter needs one bootstarp node
            JobNetPlanRecordAdapter adaptedRecord = new JobNetPlanRecordAdapter(edgesDBReader,record);
            addNode(adaptedRecord);
	    }
	}

	public boolean isJobnetPresentInDB() {
		return jobnetFound;
	}

}
