package com.sos.jobnet.jobs;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionUUID;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetDAOConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.classes.JobNetNodeOrderStarter;
import com.sos.jobnet.creator.JobNetIdCalculator;
import com.sos.jobnet.creator.jobs.JobNetJobAdapterClass;
import com.sos.jobnet.db.EventsDBItem;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.interfaces.IJobNetCallback;
import com.sos.jobnet.interfaces.IMessageListener;
import com.sos.jobnet.jobs.dispatcher.JobNetDispatcher;
import com.sos.jobnet.jobs.waiter.JobNetWaiter;
import com.sos.jobnet.logging.JobNetProtocol;
import com.sos.jobnet.logging.Level;
import com.sos.jobnet.logging.StatusProtocol;
import com.sos.jobnet.options.JobNetIdCalculatorOptions;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.jobnetmodel.live.JobnetOrderOptions;
import com.sos.jobnetmodel.objects.NodeStatus;
import com.sos.scheduler.cmd.ICommandActionsListener;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.objects.JSObjOrder;
import com.sos.scheduler.model.objects.JSObjParams;
import com.sos.scheduler.model.objects.Param;
import com.sos.scheduler.model.objects.Spooler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sos.spooler.Job_chain_node;
import sos.spooler.Order;
import sos.spooler.Supervisor_client;
import sos.spooler.Variable_set;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The JobAdapter to process a jobnet instance.
 *
 * @author KB/SS/UR
 * @version $Id$
 *
 * Created on 28.02.2012 13:10:54
 */
public class JobNetWaitAndDispatch extends JobNetJobAdapterClass implements
		IJobNetCallback, IMessageListener, ICommandActionsListener {

	private final String conClassName = "JobNetWaitAndDispatch";
	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id$";

	private static final Logger logger = LoggerFactory.getLogger(JobNetWaitAndDispatch.class);
	private JobNetProtocol jobnetProtocol = null;
	private StatusProtocol statusProtocol = null;
	private boolean flgRunnerIsSkipped = false;
	private boolean flgWaiterOK = true;
	private boolean flgDispatcherDone = false;
	private final static int maxLengthOfStatusText = 100;
    private boolean appenderInitialized = false;
    private SchedulerObjectFactory objectFacory = null;

    private class JobnetIdCalculation {
		private boolean isPresent = false;
		private String paramName;
		private String paramValue;
		private String originalValue;
	}

    @Override
	public boolean spooler_process_before() {

        flgRunnerIsSkipped = false;
		flgWaiterOK = true;
		flgDispatcherDone = false;
        JobNetOptions options = null;

        try {
			initializeLog4JAppender();

			JobnetIdCalculation calculatedId = calculateJobNetId();
	        if(calculatedId.isPresent) {
		        spooler_log.info("JobNetId for this order was calculated: " + calculatedId.paramName + "=" + calculatedId.paramValue);
		        spooler_log.info("The original value was " + calculatedId.originalValue);
		        spooler_task.order().params().set_value(calculatedId.paramName, calculatedId.paramValue);
	        }

			JobNetWaiter jobNetWaiter = new JobNetWaiter( spooler_task.id(), spooler_job.name() );
			jobNetWaiter.setIJobNetCallback(this);

			options = jobNetWaiter.Options();
			jobNetWaiter.setOptionsUsingHashMap(getAllParametersAsProperties());

			String orderId = getOrderIdForDBNode();
            options.node.Value(orderId);
            options.scheduler_id.Value(spooler.id());
            replaceVars();
            logOrderParams();

            logger.info(String.format("Checking order %1s in job chain %2s ", orderId, options.jobnet.Value()));
			initializeProtocol(orderId, options.jobnet,options.uuid_jobnet_identifier);
			Level.log(jobnetProtocol, Level.INFO, String.format("%1$s is started.", orderId));

			setCheckMandatory(options);
			options.CheckMandatory();
			jobNetWaiter.doInit();
			flgWaiterOK = jobNetWaiter.run();

			if(calculatedId.isPresent) {
		        spooler_task.order().params().set_value(calculatedId.paramName, calculatedId.originalValue);
		        spooler_log.info("JobNetId for this order was reset to original value: " + calculatedId.paramName + "=" + calculatedId.originalValue);
	        }

		} catch (Exception e) {
			flgWaiterOK = false;
            String msg = "Error in spooler_process_before.";
            logger.error(msg,e);
            throw new JobNetException(msg,e);
        }

        boolean result = flgRunnerIsSkipped == false && flgWaiterOK;
        log(Level.INFO,"Waiter finished with: " + result);

		// run the dispatcher immediately and set the next state if runner is skipped
		// otherwise the order will be suspended if spooler_process_before ends with false 
		if (flgRunnerIsSkipped  && flgWaiterOK) {
            String msg = "Processing of the order is skipped.";
            logger.info(msg);
			if (jobnetProtocol != null) jobnetProtocol.info(msg);
			do_task_after();
			String nextState = getLastStateOfChain(spooler_task.order().job_chain_node());
			log(Level.INFO,"Order is set to state " + nextState);
			spooler_task.order().set_state(nextState);
		}
		return result;
	}

    /**
     * A maximum of five levels is supported to assign indirect variable values. This avoids errors if placeholder are defined which can not be replaced with a order param.
     */
    private void replaceVars() {
        String[] arr = spooler_task.order().params().names().split(";");
        boolean cont = true;
        int count = 0;
        while(cont && count < 5) {
            cont = false;
            for(String key : arr) {
                String searchFor = "%" + key + "%";
                String replaceWith = spooler_task.order().params().value(key);
                Pattern p = Pattern.compile("%.*%");
                Matcher m = p.matcher(replaceWith);
                if(m.find()) {          // variable value has unresolved placeholder
                    cont = true;
                } else {
                    for(String key2 : arr) {
                        String value = spooler_task.order().params().value(key2);
                        if(value.contains(searchFor))
                            spooler_task.order().params().set_value(key2,value.replaceAll(searchFor,replaceWith));
                    }
                }
            }
            count++;
        }
    }
	
	/**
	 * A running order has an id with the pattern order.jebnetid, but in the nodes table is stored only the
	 * order name, not the jobnetid. We need the order without id to reference the node in the db.
	 * @return
	 */
	private String getOrderIdForDBNode() {
		String id = spooler_task.order().id();
		int i = id.lastIndexOf(JobNetDAOConstants.JOBNET_ID_DELIMITER);
		return i > 0 ? id.substring(0,i) : id;
	}

	private void logOrderParams() {
		log(Level.INFO,"List of all order parameters after calculation:");
		log(Level.INFO,"===============================================");
        String[] arr = spooler_task.order().params().names().split(";");
        for(String key : arr) {
    		log(Level.INFO,key + "=" + spooler_task.order().params().value(key));
        }
	}
	
	@Override
	public void spooler_task_after() {

		// is already done if runner was skipped (see spooler_process_before)
		if (flgWaiterOK && !flgDispatcherDone) {
			try {
				do_task_after();
			} catch (Exception e) {
                String msg = "Error in spooler_task_after.";
                log(Level.ERROR,msg,e);
                throw new JobNetException(msg,e);
            }
		}
		log(Level.INFO,"Order is finished.");

	}

	private String getLastStateOfChain(final Job_chain_node job_chain_node) {
		String currentState = job_chain_node.state();
		Job_chain_node job_chain_next_node = job_chain_node.next_node();
		while(job_chain_next_node != null) {
			currentState = job_chain_next_node.state();
			job_chain_next_node = job_chain_next_node.next_node();
		}
		return currentState;
	}

	private void do_task_after() {
		final String conMethodName = conClassName + "::spooler_task_after";
		try {
			initializeLog4JAppender();

            objectFacory = new SchedulerObjectFactory(spooler.hostname(), spooler.tcp_port());
            objectFacory.initMarshaller(Spooler.class);

			JobnetOrderOptions jobnetOrderOptions = getJobnetOrderOptions();

			JobNetDispatcher jobNetDispatcher = new JobNetDispatcher();
			jobNetDispatcher.setIJobNetCallback(this);
			JobNetOptions dispatcherOptions = jobNetDispatcher.Options();
			String orderId = getOrderIdForDBNode();
			jobNetDispatcher.setOptionsUsingHashMap(getAllParametersAsProperties());

			initializeProtocol(orderId, dispatcherOptions.jobnet,dispatcherOptions.uuid_jobnet_identifier);
			logger.info(String.format("Checking order %1$s in job chain %2$s ",orderId,dispatcherOptions.jobnet.Value()));

			dispatcherOptions.scheduler_id.Value(spooler.id());
			dispatcherOptions.node.Value(orderId);
			setCheckMandatory(dispatcherOptions);
			jobNetDispatcher.Options().CheckMandatory();
			jobNetDispatcher.doInit();
			jobNetDispatcher.run();
			
			// in case of successful execution of the order an event is thrown
			
			/*
            spooler_log.info("Options of the jobnet order");
			spooler_log.info("===========================");
			for(String key : jobnetOrderOptions.Settings().keySet()) {
				spooler_log.info(key + "=" + jobnetOrderOptions.getOrderRecord(key));
			}
			*/
			String scope = spooler_task.exit_code() == 0 ? "success" : "error";
			String eventId = getEventId(jobnetOrderOptions, scope);
			if(!eventId.isEmpty()) {
				log(Level.INFO,"Try to throw event " + eventId);
				jobNetDispatcher.createEvent(eventId);
			} else {
				log(Level.INFO,"No event to throw.");
			}
			
			// put an end notification in the protocol
			log(Level.INFO,String.format("End: %1$s.",orderId));
			flgDispatcherDone = true;

        } catch (Exception e) {
            String msg = "Error parsing the parameter from the jobnet node.";
            log(Level.ERROR,msg,e);
            throw new JobNetException(msg,e);
        }
	}

    private JobnetOrderOptions getJobnetOrderOptions() {
        JobnetOrderOptions jobnetOrderOptions = new JobnetOrderOptions();
        try {
            jobnetOrderOptions.setAllOptions(getSchedulerParameterAsProperties(getJobOrOrderParameters()));
            jobnetOrderOptions.CheckMandatory();
        } catch (Exception e) {
            throw new JobNetException("Error parsing the parameter from the jobnet node.");
        }
        return jobnetOrderOptions;
    }

//	@Override
//	public StartModes getStartMode(JobnetOrderOptions jobnetOrderOptions) {
//		String startOptionString = jobnetOrderOptions.getItem(JobNetGlobalParams.StartOption.name());
//		StartModes startOption = StartModes.valueOf(startOptionString);
//		logger.info("Jobnet startet with StartOption " + startOption.name());
//		return startOption;
//	}

    /**
	 * The event will be unique by appending the id of the jobnet.
	 * 
	 * @return
	 */
	private String getEventId(final JobnetOrderOptions options, final String scope) {
		String eventName = scope.equals("success") 
				? options.event_on_success.Value()
				: options.event_on_error.Value();
		return eventName;
	}

	private void initializeLog4JAppender() {
        if (!appenderInitialized) {
		    initializeLog4jAppenderClass();
            appenderInitialized = true;
        }
	}

	private void initializeProtocol(final String orderId, final SOSOptionString jobnet, final SOSOptionUUID uuid) {
		jobnetProtocol = new JobNetProtocol(orderId, uuid.Value());
		statusProtocol = new StatusProtocol(orderId, uuid.Value());
	}

	// TODO move to optionsClass !!!!!!!!!!!!!!!!!!!!!!!
	private void setCheckMandatory(final JobNetOptions objOptions) {
		//Das fliegt hier raus und kommt in die Options.
		objOptions.uuid_jobnet_identifier.isMandatory(true);
		objOptions.hibernate_connection_config_file.isMandatory(true);
		objOptions.scheduler_id.isMandatory(true);
		objOptions.jobnet.isMandatory(true);
		objOptions.node.isMandatory(true);
 	}

	/**
	 * The exit code given from the job can be ignored by setting the option valid_errors.
	 * If affects the flow of the jobnet only, not the flow of the jobnet node order.
	 */
	@Override
	public int getExitCode() {
		return spooler_task.exit_code();
	}

    @Override
    public void setNextState() {
        String nextState = spooler_task.order().job_chain_node().next_state();
        logger.info("The order state was set to " + nextState);
        spooler_task.order().set_state(nextState);
    }

	/**
	 * Start an order for a given jobnet node.
     *
     * The order which will be started is stored as xml in field ORDER_XML. Additionally a node maybe get some
     * parameters from its predecessor. This parameters are store in field PARAM-XML and will be merged into the
     * existing order (only parameters which are not already parameters in the order take place).
	 */
	@Override
	public void startNode(final JobNetPlanDBItem planItem, final JobNetOptions options) {
		
		
		logger.info(String.format("Starting Plan.Node: %1s.%2s",planItem.getPlanId(),planItem.getNodeId()));
		logger.debug(String.format("  Node: %1s",planItem.getJobnetNodeItem().getNode()));
		logger.debug(String.format("  in Net: %1s",planItem.getJobnetNodeItem().getJobnet()));

        // String scriptRoot = spooler.variables().value(JobNetConstants.SCRIPT_ROOT_GLOBAL_PARAMETER_NAME);
        String jobnetId = planItem.getJobNetHistoryDBItem().getJobNetId();
		JobNetNodeOrderStarter starter = new JobNetNodeOrderStarter(spooler.hostname(), spooler.tcp_port(), planItem);
		starter.addMessageListener(this);

		JSObjOrder orderFromDB = new JSObjOrder(starter.getFactory());
		orderFromDB.getOrderFromXMLString(planItem.getOrderXml());

        JSObjParams paramsFromDB = orderFromDB.getJSObjParams();
        logger.debug("\nParameter of the order:\n" + paramsFromDB.toXMLString());

        JSObjParams paramsFromPreviousNode = new JSObjParams(starter.getFactory());
        paramsFromPreviousNode.setParamsFromString(planItem.getParamXml());
        logger.debug("\nParameter of the previous node:\n" + paramsFromPreviousNode.toXMLString());

 		Order currentOrder = spooler_task.order();
 		if(currentOrder == null)
 			throw new JobSchedulerException("The order object is not set for the current order.");
 		

		starter.setReplace(true);
		starter.setAt("now");
		starter.setTitle(orderFromDB.getTitle());
		starter.setState(orderFromDB.getState());
 		starter.setParams(paramsFromDB);
        starter.mergeParams(paramsFromPreviousNode);
 		starter.run();

	}

	/**
	 * Calculates a jobnet id for an order
	 */
	private JobnetIdCalculation calculateJobNetId() {
		JobnetIdCalculation result = new JobnetIdCalculation();
		try {
			JobNetIdCalculatorOptions calcOptions = new JobNetIdCalculatorOptions();
			calcOptions.setAllOptions(getAllParametersAsProperties());			// replaces all %params% occurences as well
			if(!calcOptions.getResultParameterName().IsEmpty()) {
				result.paramName = calcOptions.getResultParameterName().Value();
				result.originalValue = spooler_task.order().params().value(result.paramName);
				calcOptions.CheckMandatory();
				JobNetIdCalculator calculator = new JobNetIdCalculator(calcOptions,isOrderJob());
				result.paramValue = calculator.execute();
				result.isPresent = true;
			}
		} catch (Exception e) {
			String msg = "Error calculating JobNetId";
			log(Level.ERROR,msg);
			throw new JobSchedulerException(msg,e);
		}
		return result;
	}
	
	@Override
	public void skipRunner() {
		flgRunnerIsSkipped = true;
	}

	@Override
	public void setStateText(String stateText) {
		if(stateText.length() > maxLengthOfStatusText)
			stateText = stateText.substring(0,maxLengthOfStatusText - 4) + "...";		// maximum of state text in JobScheduler DB is 100
		spooler_task.order().set_state_text(stateText);
	}

	@Override
	public void setCC(final int arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void log(final Level level, final String stateText, final Throwable error) {
		// if (jobnetProtocol != null)	jobnetProtocol.log(level,stateText, error);
		Level.log(logger,level,stateText, error);
	}

	@Override
	public void log(final Level level, final String stateText) {
		// if (jobnetProtocol != null)	jobnetProtocol.log(level,stateText);
		Level.log(logger,level,stateText);
	}

	@Override
	public void logStatus(final JobNetPlanDBItem record, final NodeStatus oldStatus) {
        NodeStatus status = NodeStatus.valueOf(record.getStatusNode());
		String msg = "STATUS changed from " + oldStatus.name() + " to " + status.name();
		if (jobnetProtocol != null)	Level.log(jobnetProtocol,Level.INFO,msg);
		if (statusProtocol != null)	Level.log(statusProtocol,Level.INFO,msg);
	}

	@Override
	public void logContextStatus(final String context, final NodeStatus fromStatus, final NodeStatus toStatus) {
		String msg = context + " changed from  " + fromStatus.name() +  " to " + toStatus.name();
		if (jobnetProtocol != null)	Level.log(jobnetProtocol,Level.INFO,msg);
		if (statusProtocol != null)	Level.log(statusProtocol,Level.INFO,msg);
	}

	@Override
	public EventsDBItem createEventRecord(final String forEventClass, final String forEventId) {
        EventsDBItem record = new EventsDBItem();
        record.setSchedulerId(spooler.id());
        record.setHost(getHost(spooler));
        record.setPort(getPort(spooler));
        record.setJobChain(spooler_task.order().job_chain().name());
        record.setOrderId(spooler_task.order().id());
        record.setJobName(spooler_task.job().name());
        record.setEventClass(forEventClass.toLowerCase());
        record.setEventId(forEventId.toLowerCase());
        record.setParameters("");
        record.setExitCode(String.valueOf(spooler_task.exit_code()));
		return record;
	}
	private static String getHost(final sos.spooler.Spooler scheduler) {
		String result = null;
		try {
			Supervisor_client supervisor = scheduler.supervisor_client();
			result = supervisor.hostname();
		}
		catch (Exception e) { // there is no supervisor
			result = scheduler.hostname();
		}
		return result;
	}
	
	private static Long getPort(final sos.spooler.Spooler scheduler) {
		int result = -1;
		try {
			Supervisor_client supervisor = scheduler.supervisor_client();
			result = supervisor.tcp_port();
		}
		catch (Exception e) { // there is no supervisor
			result = scheduler.tcp_port();
		}
		return Long.valueOf(result);
	}


	@Override
	public void createJobNetStartedEvent(final JobNetWaiter jobNetwaiter) {
        String uuid = jobNetwaiter.Options().uuid_jobnet_identifier.Value();
        String eventId = JobNetConstants.JOBNET_STARTED_PREFIX + uuid;
        jobNetwaiter.createEvent(eventId);
        String logText = "Jobnet started.";
        log(Level.INFO,logText);
	}

    @Override
    /**
     * Gets a list of all parameters to provide to subsequent nodes. Parameters to control the jobnet itself (defined in com.sos.jobnetmodel.live.JobnetOrderOptions) are not provided.
     */
    public JSObjParams getOrderParameterToProvide() {
        Variable_set orderParams = spooler_task.order().params();
        String[] names = orderParams.names().split(";");
        JSObjParams p = objectFacory.createParams();
        JobnetOrderOptions op = new JobnetOrderOptions();
        for(String name : names) {
            if(!op.isOption(name)) {        // node specific parameters are not provided
                Param param = objectFacory.createParam();
                param.setName(name);
                param.setValue(orderParams.value(name));
                p.getParamOrCopyParamsOrInclude().add(param);
            }
        }
        logger.info("The order parameter as string: " + p.toXMLString());
        return p;
    }

    @Override
    public SchedulerObjectFactory getFactory() {
        return objectFacory;
    }

	@Override
	public void createJobNetEndedEvent(final JobNetDispatcher jobNetDispatcher) {
		String uuid = jobNetDispatcher.Options().uuid_jobnet_identifier.Value();
		String eventId = JobNetConstants.JOBNET_ENDED_PREFIX + uuid;
		// An order to create the jobnet graph will stop if this event is fired.
		jobNetDispatcher.createEvent(eventId);
		String logText = "All nodes of the jobnet are in state FINISHED or ERROR - jobnet ended.";
		log(Level.INFO,logText);
	}

	@Override
	public void onMessage(final Level level, final String message) {
		log(level,message);
	}

    @Override
    public void onMessage(org.apache.log4j.Level level, String message) {
        Level.log(logger,level,message);
    }

}
