package com.sos.jobnet.jobs.info;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Options.JSOptionsClass;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionUUID;

import java.util.HashMap;

@JSOptionClass(name = "JobNetInfoOptions", description = "Options to get various information about a jobnet instance.")
public class JobNetInfoOptions extends JSOptionsClass {

    private final String conClassName = JobNetInfoOptions.class.getSimpleName();

    public JobNetInfoOptions() {
        objParentClass = this.getClass();
    }

    public JobNetInfoOptions(com.sos.JSHelper.Listener.JSListener pobjListener) {
        this();
        this.registerMessageListener(pobjListener);
    }

    //

    public JobNetInfoOptions(HashMap<String, String> JSSettings) throws Exception {
        this();
        this.setAllOptions(JSSettings);
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * The unique id of the jobnet. If this parameter is not sepcified, the JObNetProcessor will be assign a unique identifier (UUID) for the
     * jobnet instance. In general it is not possible to start a second istance of a jobnet with the same ID.
     */
    @JSOptionDefinition(name = "JobNetId", description = "", key = "JobNetId", type = "SOSOptionUUID", mandatory = false)
    public SOSOptionUUID JobNetId = new SOSOptionUUID(this, conClassName
            + ".JobNetId", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            false // isMandatory
    );

    public void setJobNetId(SOSOptionUUID jobNetId) {
        this.JobNetId = jobNetId;
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * Name of the result parameter (it is always an order parameter).
     */
    @JSOptionDefinition(name = "ResultParameterName",
            description = "The name of the result parameter (it is always an order parameter).",
            key = "ResultParameterName",
            type = "SOSOptionString",
            mandatory = false)

    public SOSOptionString ResultParameterName = new SOSOptionString(this, getClass().getSimpleName() + ".ResultParameterName", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            true // isMandatory
    );


    /**
     * Rule for calculating the new load id.
     */
    @JSOptionDefinition(name = "CalculationRule",
            description = "The rule for calculating the new load id (the + and - sign and digits allowed).",
            key = "CalculationRule",
            type = "SOSOptionString",
            mandatory = false)

    public SOSOptionString CalculationRule = new SOSOptionString(this, getClass().getSimpleName() + ".CalculationRule", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            false // isMandatory
    );

    public SOSOptionString  getCalculationRule() {
        return CalculationRule;
    }

    /**
     * LoadIdDateFormat
     */
    @JSOptionDefinition(name = "LoadIdDateFormat",
            description = "The date pattern for the LoadId (followed the org.joda.time.format.DateTimeFormat).",
            key = "LoadIdDateFormat",
            type = "SOSOptionString",
            mandatory = false)

    public SOSOptionString LoadIdDateFormat = new SOSOptionUUID(this, getClass().getSimpleName() + ".LoadIdDateFormat", // HashMap-Key
            "", // Titel
            "yyyyMMdd", // InitValue
            "yyyyMMdd", // DefaultValue
            false // isMandatory
    );

    public SOSOptionString getLoadIdDateFormat() {
        return LoadIdDateFormat;
    }

    public void setLoadIdDateFormat(SOSOptionString format) {
        this.LoadIdDateFormat = format;
    }

    /**
     * ResultDateFormat
     */
    @JSOptionDefinition(name = "ResultDateFormat",
            description = "The date pattern for the result field (followed the org.joda.time.format.DateTimeFormat).",
            key = "ResultDateFormat",
            type = "SOSOptionString",
            mandatory = false)

    public SOSOptionString ResultDateFormat = new SOSOptionUUID(this, getClass().getSimpleName() + ".ResultDateFormat", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            false // isMandatory
    );

    public SOSOptionString getResultDateFormat() {
        return ResultDateFormat;
    }

    /**
     * LoadIdDurationType
     */
    @JSOptionDefinition(name = "LoadIdDurationType",
            description = "The duration type corresponding to LoadIdDateFormat (followed the org.joda.time.DurationFieldType).",
            key = "LoadIdDurationType",
            type = "SOSOptionString",
            mandatory = false)

    public SOSOptionString LoadIdDurationType = new SOSOptionUUID(this, getClass().getSimpleName() + ".LoadIdDurationType", // HashMap-Key
            "", // Titel
            "days", // InitValue
            "days", // DefaultValue
            false // isMandatory
    );

    public SOSOptionString getLoadIdDurationType() {
        return LoadIdDurationType;
    }

    public void setLoadIdDurationType(SOSOptionString type) {
        this.LoadIdDurationType = type;
    }

    /**
     * \brief setAllOptions - uebernimmt die Optionenwerte aus der HashMap
     * 
     * \details In der als Parameter anzugebenden HashMap sind Schluessel (Name)
     * und Wert der jeweiligen Option als Paar angegeben. Ein Beispiel fuer den
     * Aufbau einer solchen HashMap findet sich in der Beschreibung dieser
     * Klasse (\ref TestData "setJobSchedulerSSHJobOptions"). In dieser Routine
     * werden die Schluessel analysiert und, falls gefunden, werden die
     * dazugehoerigen Werte den Properties dieser Klasse zugewiesen.
     *
     * Nicht bekannte Schluessel werden ignoriert.
     *
     * \see JSOptionsClass::getOrderRecord
     *
     * @param pobjJSSettings
     * @throws Exception
     */
    public void setAllOptions(HashMap<String, String> pobjJSSettings) {
        @SuppressWarnings("unused")
        final String conMethodName = getClass().getName() + "::setAllOptions";
        flgSetAllOptions = true;
        objSettings = pobjJSSettings;
        super.Settings(objSettings);
        super.setAllOptions(pobjJSSettings);
        flgSetAllOptions = false;
    } // public void setAllOptions (HashMap <String, String> JSSettings)

    /**
     * \brief CheckMandatory - prueft alle Muss-Optionen auf Werte
     *
     * \details
     * @throws Exception
     *
     * @throws Exception
     * - wird ausgeloest, wenn eine mandatory-Option keinen Wert hat
     */
    @Override
    public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
            , Exception {
        try {
            super.CheckMandatory();
        }
        catch (Exception e) {
            throw new JSExceptionMandatoryOptionMissing(e.toString());
        }
    } // public void CheckMandatory ()

    /**
     *
     * \brief CommandLineArgs - uernehmen der Options/Settings aus der Kommandozeile
     *
     * \details Die in der Kommandozeile beim Starten der Applikation
     * angegebenen Parameter werden hier in die HashMap uebertragen und danach
     * den Optionen als Wert zugewiesen.
     *
     * \return void
     *
     * @param pstrArgs
     * @throws Exception
     */
    @Override
    public void CommandLineArgs(String[] pstrArgs) {
        super.CommandLineArgs(pstrArgs);
        this.setAllOptions(super.objSettings);
    }

}
