package com.sos.jobnet.jobs.waiter;

import com.sos.JSHelper.io.Files.JSFile;
import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.jobnet.classes.*;
import com.sos.jobnet.db.*;
import com.sos.jobnet.interfaces.IJobNetPlanEventListener;
import com.sos.jobnet.interfaces.IWaitHandler;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.jobnet.options.StartModes;
import com.sos.jobnetmodel.objects.NodeStatus;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
* \class JobNetDBWaiter 
* 
* \brief JobNetDBWaiter - 
* 
* \details
*
* \section JobNetDBWaiter.java_intro_sec Introduction
*
* \section JobNetDBWaiter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetDBWaiter.java 21059 2013-09-12 09:51:06Z ss $
* \see reference
*
* Created on 22.02.2012 13:56:28
 */
/**
 * @author oh
 *
 */
@I18NResourceBundle(baseName = "com_sos_jobnet_messages", defaultLocale = "en")
public class JobNetDBWaiter extends I18NBase implements IWaitHandler {
	
	private final String		conClassName			= "JobNetDBWaiter";
	private	final String		conSVNVersion			= "$Id: JobNetDBWaiter.java 21059 2013-09-12 09:51:06Z ss $";
	private static final Logger	logger					= LoggerFactory.getLogger(JobNetDBWaiter.class);
	private List<JobNetEdgesDBItem> jobNetEdgesList		= null;
	private JobNetCollection 	waitforNodes 			= new JobNetCollection();
	
	private static final String	JOBNETDBW_E_0010		= "JOBNETDBW_E_0010";		// %s: No item found for the current node (%s) in database table SCHEDULER_JOB_NET_PLAN.
	private static final String	JOBNETDBW_E_0011		= "JOBNETDBW_E_0011";		// %s: No predecessor with node_id=%d found in database table SCHEDULER_JOB_NET_PLAN.
	
	private final JobNetOptions options;
	private final String uuidJobnetIdentifier;
	private final String jobNet;
	private final String node;
	private final String schedulerId;
	private final JSFile hibernateConnectionConfigFile;
	private final String curNodeTitle;
	private final List<String> workWithEvents;

	private final EventsDBLayer eventsDBLayer;
	private final JobNetPlanDBLayer	jobNetPlanDBLayer;
	private final ConditionsDBLayer	conditionsDBLayer;
	private final JobNetHistoryDBLayer	historyDBLayer;
	private final JobNetCmdHistoryDBLayer cmdHistoryDBLayer;
    private final JobNetCmdHistoryDBItem commandRecord;
    private final JobNetHistoryDBItem historyRecord;
    private final Boolean createEvents;
    private Boolean jobnetStart = false;
    private final StartModes startMode;

    private JobNetPlanDBItem currentRecord;
	private IJobNetPlanEventListener eventListener = null;
	
	public JobNetDBWaiter(JobNetOptions jobNetOptions) {
		
		super(JobNetConstants.strBundleBaseName);
		this.options = jobNetOptions;
		this.uuidJobnetIdentifier = options.uuid_jobnet_identifier.Value();
		this.jobNet = options.jobnet.Value();
		this.node = options.node.Value();
		this.schedulerId = options.scheduler_id.Value();
		this.hibernateConnectionConfigFile 	= options.hibernate_connection_config_file.JSFile();
 		this.curNodeTitle = uuidJobnetIdentifier + ":" + schedulerId + ":" + jobNet + ":" + node;
		this.workWithEvents = getWaitForEvents(options.wait_for_events.Value());

		String conMethodName = this.getClass().getSimpleName(); 
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "uuid_jobnet_identifier", uuidJobnetIdentifier));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "jobnet", jobNet));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "node", node));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "scheduler_id", schedulerId));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernate_connection_config_file", hibernateConnectionConfigFile.getAbsolutePath()));
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "wait_for_events", workWithEvents));

		this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(hibernateConnectionConfigFile, schedulerId);
		this.conditionsDBLayer 	= new ConditionsDBLayer(hibernateConnectionConfigFile);
		this.historyDBLayer = new JobNetHistoryDBLayer(hibernateConnectionConfigFile, schedulerId);
		this.cmdHistoryDBLayer = new JobNetCmdHistoryDBLayer(hibernateConnectionConfigFile, schedulerId);
		this.eventsDBLayer = new EventsDBLayer(hibernateConnectionConfigFile);
		this.currentRecord = readCurrentRecordFromDBAndLock();

        this.historyRecord = readHistory(uuidJobnetIdentifier);
        this.commandRecord = readCmdHistory(historyRecord);
        logger.debug("Jobnet history information successfully read.");

        this.startMode = StartModes.valueOf(commandRecord.getStartOption());
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "StartOption", startMode.name()));
        
        String qualifier = options.getItem(JobNetGlobalParams.JobNetIdQualifier.name());
        if (qualifier == null) {
        	String msg = "The parameter " + JobNetGlobalParams.JobNetIdQualifier.name() + " not present in the options of the node.";
        	logger.error(msg);
        	throw new JobNetException(msg);
        }
        qualifier = qualifier.trim();
		logger.info(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "JobNetIdQualifier", qualifier));

		boolean isIndependent = (startMode == StartModes.start_independent || !qualifier.isEmpty());
        logger.info("The instance " + uuidJobnetIdentifier + " is an " + (isIndependent ? "in" : "") + "dependent one.");
        this.createEvents = !(isIndependent);

    }

    private JobNetHistoryDBItem readHistory(String uuid) {
        JobNetHistoryDBItem historyRecord = null;
        try {
            historyRecord = historyDBLayer.getJobNetInstance(uuid);
        } catch(Exception e) {
        	String msg = "Error retrieving jobnet history entry for UUID " + uuid;
        	logger.error(msg,e);
            throw new JobNetException(msg,e);
        }
        return historyRecord;
    }

    private JobNetCmdHistoryDBItem readCmdHistory(JobNetHistoryDBItem historyRecord) {
        JobNetCmdHistoryDBItem commandRecord = null;
        try {
            commandRecord = cmdHistoryDBLayer.getLastJobNetCmdHistoryEntry(historyRecord.getHistoryId());
        } catch(Exception e) {
        	String msg = "Error retrieving jobnet command entry for UUID " + historyRecord.getUuid();
        	logger.error(msg,e);
            throw new JobNetException(msg,e);
        }
        return commandRecord;
    }

	private JobNetPlanDBItem readCurrentRecordFromDBAndLock() {
		
		final String conMethodName = conClassName + "::readCurrentRecordFromDBAndLock";
		JobNetPlanDBItem result = null;
		JobNetPlanFilter filter = new JobNetPlanFilter();
		filter.setOrderCriteria(null);
		filter.setSchedulerId(schedulerId);
		filter.setUuid(uuidJobnetIdentifier);
		filter.setJobnet(jobNet);
		filter.setNode(node);
        logger.info("Set filter for JobNetPlan: schedulerID=" + schedulerId + ", UUID=" + uuidJobnetIdentifier + ", jobnet=" + jobNet + ", node=" + node);
		jobNetPlanDBLayer.setFilter(filter);
		
		List<JobNetPlanDBItem> jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
		if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
			String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, curNodeTitle);
			logger.error(msg);
			throw new JobNetException(msg);
		}
		else {
			result = jobNetPlanDBItem.get(0);
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0010, conMethodName, result.getTitle()));
		}
		return result;
		
	}
	
	private List<String> getWaitForEvents(String eventsString) {
        String s = eventsString.trim();
		return (s.isEmpty()) ? new ArrayList<String>() : Arrays.asList(s.split(";"));
	}
	
	/**
	 * \brief getCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem getCurrentRecord() {
		return currentRecord;
	}
		
	/**
	 * \brief getJobnetEdgesList
	 * 
	 * \details
	 */
	private List<JobNetEdgesDBItem> getJobnetEdgesList() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getJobnetEdgesList";
		getCurrentRecord();
		if (jobNetEdgesList == null) {
			File configurationFile 	= new File(hibernateConnectionConfigFile.getAbsolutePath());
			JobNetEdgesDBLayer jobNetEdgesDBLayer = new JobNetEdgesDBLayer(configurationFile);
			jobNetEdgesList = jobNetEdgesDBLayer.getPredecessors(getCurrentRecord().getJobnetNodeDBItem());
		}
		return jobNetEdgesList;
	} // public List<JobNetEdgesDBItem> getJobnetEdgesList
	
	
	/**
	 * \brief updateWaiterStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateWaiterStatus(NodeStatus status) {
		final String conMethodName = conClassName + "::updateWaiterStatus";
		JobNetPlanDBItem record = getCurrentRecord();
		if (record != null) {
			int oldState = record.getStatusWaiter();
			record.setStatusWaiter(status.getIndex());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), status.getMsg()));
			update();
			if (eventListener != null)
				eventListener.waiterStatusChanged(status, NodeStatus.valueOf(oldState) );
//			after each updateWaiterStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date());
		}
	} // private void updateWaiterStatus
	
	/**
	 * \brief updateRunnerStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateRunnerStatus(NodeStatus status) {
		final String conMethodName = conClassName + "::updateRunnerStatus";
		JobNetPlanDBItem record = getCurrentRecord();
		if (record != null) {
			int oldState = record.getStatusRunner();
			record.setStatusRunner(status.getIndex());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), status.getMsg()));
			update();
			if (eventListener != null)
				eventListener.runnerStatusChanged(status, NodeStatus.valueOf(oldState) );
//			after each updateRunnerStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date()); 
		}
	} // private void updateRunnerStatus
	
	
	/**
	 * \brief updateNodeStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateNodeStatus(NodeStatus status) {
		final String conMethodName = conClassName + "::updateNodeStatus";
		JobNetPlanDBItem rec = getCurrentRecord();
		if (rec!=null) {
			int oldState = rec.getStatusNode();
			rec.setStatusNode(status.getIndex());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0013), status.getMsg()));
			update();
			nodeStatusChanged(rec, NodeStatus.valueOf(oldState) );
		}
	} // private void updateNodeStatus
	
	
	/**
	 * \brief update
	 * 
	 * \details
	 */
	private void update() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::update";
		getCurrentRecord().setModified( new DateTime() );
		getCurrentRecord().setModifiedBy(JobNetDAOConstants.CURRENT_USER);
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(getCurrentRecord());
		jobNetPlanDBLayer.commit();
	} // private void update
	
	
	private String getI18NYesNo(boolean flgIsYes) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getI18NYesNo";
		String msg = Messages.getMsg(JobNetConstants.JOBNET_T_0002); //No
		if (flgIsYes) {
			msg = Messages.getMsg(JobNetConstants.JOBNET_T_0001); //Yes
		}
		return msg;
	}

	
	/**
	 * \brief isBootstrap
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isBootstrap() {
		final String conMethodName = conClassName + "::isBootstrap";
		boolean isBootstrap = getCurrentRecord().getBootstrap();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0021, conMethodName, getI18NYesNo(isBootstrap)));
		return isBootstrap;
	} // public boolean isBootstrap

	
	/**
	 * \brief setWaiterStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToRunning() {
		updateWaiterStatus(NodeStatus.RUNNING);
	}

	/**
	 * \brief setNodeStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToRunning() {
		updateNodeStatus(NodeStatus.RUNNING);
	}
	


	/**
	 * \brief isWaiterSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isWaiterSkipped() {
		final String conMethodName = conClassName + "::isWaiterSkipped";
		boolean isSkipped = getCurrentRecord().getIsWaiterSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	

	/**
	 * \brief isRunnerSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerSkipped() {
		final String conMethodName = conClassName + "::isRunnerSkipped";
		boolean isSkipped = getCurrentRecord().getIsRunnerSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	
	@Override
	public boolean isRunnerRunning() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isRunnerRunning";
        NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusRunner());
		return status.isRunning();
	}
	
	/**
	 * \brief isRunnerOnDemand
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerOnDemand() {
		final String conMethodName = conClassName + "::isRunnerOnDemand";
		boolean isOnDemand = getCurrentRecord().getIsRunnerOnDemand();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0022, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isOnDemand)));
		return isOnDemand;
	}

	/**
	 * \brief setNodeStatusToWaiting
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToWaiting(Integer taskId, String jobName) {
        getCurrentRecord().setTaskId( Long.valueOf(taskId) );
        getCurrentRecord().setJobName( jobName );
		updateNodeStatus(NodeStatus.WAITING);
	}

	/**
	 * \brief setRunnerStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setRunnerStatusToRunning() {
		updateRunnerStatus(NodeStatus.RUNNING);
	}

	/**
	 * \brief setWaiterStatusToFinish
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToFinish() {
		updateWaiterStatus(NodeStatus.FINISHED);
	}

	/**
	 * \brief setWaiterStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToError() {
		updateWaiterStatus(NodeStatus.ERROR);
	}

	/**
	 * \brief setNodeStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToError() {
		updateNodeStatus(NodeStatus.ERROR);
	}
	
	/**
	 * \brief setStartOfJobnet
	 * 
	 * \details
	 */
	@Override
	public void setStartTime() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setStartOfJobnet";
		if (getCurrentRecord() != null) {
			getCurrentRecord().setStartOfNode(new DateTime());
//			after each setStartOfJobnet follows updateNodeStatus which calls the commit,
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(now); 
		}
	}
	
	@Override
	public JobNetCollection getWaitingNodes() {
		final String conMethodName = conClassName + "::getWaitingNodes";
		getJobnetEdgesList();
		
		List<JobNetPlanDBItem> jobNetPlanDBItem = null;
		JobNetPlanDBItem jobnetPlanDBItem = null;
		waitforNodes = new JobNetCollection();
		
		for (JobNetEdgesDBItem jobnetEdgesItem : jobNetEdgesList) {
			jobNetPlanDBLayer.getFilter().setNode(null);
			jobNetPlanDBLayer.getFilter().setNodeId(jobnetEdgesItem.getChildNodeId());
			jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanList(0);
			
			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				String msg = Messages.getMsg(JOBNETDBW_E_0011, conMethodName, jobnetEdgesItem.getChildNodeId()); 
				logger.error(msg);
				throw new JobNetException(msg);
			}
			
			jobnetPlanDBItem = jobNetPlanDBItem.get(0);
			String predecessor = jobnetPlanDBItem.getJobnetNodeItem().getNode();
			NodeStatus status = NodeStatus.valueOf(jobnetPlanDBItem.getStatusNode());
			
			logger.info(Messages.getMsg(JobNetConstants.JOBNETW_I_0010, conMethodName, predecessor, status.getMsg()));
			if (status.isFinished()) {
				continue;
			}
			waitforNodes.add(jobnetPlanDBItem);
		}
		return waitforNodes;
	}

	@Override
	public void open() {
		logger.info(String.format("%1s %2s", conClassName, conSVNVersion));
		getCurrentRecord();
		//TODO skips in Options lesen und in DB setzen
		//Ist unnoetig, wenn das dashboard die Skip-Funktionalitaet zur Verfuegung stellt
	}

	@Override
	public void close() {
		eventsDBLayer.closeSession();
		jobNetPlanDBLayer.closeSession();
	}

	@Override
	public EventsCollection getMissingEvents() {
        EventsCollection missingEvents = null;
        if(workWithEvents()) {
            missingEvents = eventsDBLayer.checkEvents(workWithEvents);
        } else {
            String eventString = Arrays.toString(workWithEvents.toArray());
            logger.info("StartOption " + getStartMode().name() + " does not respect events - missing events " + eventString + " are ignored.");
            missingEvents = new EventsCollection();
        }
		return missingEvents;
	}

	@Override
	public List<ConditionsDBItem> getMissingConditions() {
        return conditionsDBLayer.getMissingConditions(getCurrentNode().getPlanId());
	}

    @Override
    public List<EventsDBItem> getFulfilledEvents() {
        List<EventsDBItem> fulfilledEvents = null;
        if(workWithEvents()) {
            fulfilledEvents = eventsDBLayer.getFulfilledEvents(workWithEvents);
        } else {
            String eventString = Arrays.toString(workWithEvents.toArray());
            logger.info("StartOption " + getStartMode().name() + " does not respect events - missing events " + eventString + " are ignored.");
            fulfilledEvents = new ArrayList<EventsDBItem>();
        }
        return fulfilledEvents;
    }

    @Override
    public JobNetPlanDBItem getCurrentNode() {
        return currentRecord;
    }

    @Override
	public boolean isNodeStarted() {
		NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusNode());
		return status.isStarted();
	}

	@Override
	public boolean isNodePreparedForRestart() {
		NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusNode());
		return status.isRestartPrepared();
	}

	@Override
	public boolean isNodeFinished() {
		NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusNode());
		return status.isFinished();
	}

	@Override
	public boolean isNodeNotProcessed() {
        NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusNode());
		return status.isNotProcessed();
	}


	public void nodeStatusChanged(JobNetPlanDBItem node, NodeStatus oldState) {
		if(eventListener != null) {
			eventListener.nodeStatusChanged(node, oldState);
		}
	}
	
	@Override
	public boolean isReadyForProcessing() {
		//return ( isNodeStarted() || isNodeInError() || isJobnetStart() );
		return ( isNodeStarted() || isNodePreparedForRestart() || isJobnetStart() || isNodeInError() );
	}
	
	@Override
	public void checkJobnetStart() {
        JobNetHistoryDBItem record = readHistory(uuidJobnetIdentifier);
        jobnetStart = (record.getStartOfJobnet() == null);
	}

	@Override
	public boolean isJobnetStart() {
        JobNetHistoryDBItem record = readHistory(uuidJobnetIdentifier);
        return jobnetStart;
	}

	public void setNodeStatusListener(IJobNetPlanEventListener listener) {
		this.eventListener = listener;
	}

	@Override
	public boolean isNodeInError() {
        NodeStatus status = NodeStatus.valueOf(getCurrentRecord().getStatusNode());
		return status.isError();
	}
	
	@Override
	public void createEvent(EventsDBItem record) {
        eventsDBLayer.beginTransaction();
        conditionsDBLayer.beginTransaction();
		eventsDBLayer.createEvent(record);
        conditionsDBLayer.setConditionFulfilled(record.getEventId());
        eventsDBLayer.commit();
	}

    // update the history record for the jobnetz
    @Override
    public void startHistory() {
        historyDBLayer.startHistory(uuidJobnetIdentifier);
    }

    // update the history record for the jobnetz
    @Override
    public void updateHistory() {
        historyDBLayer.updateHistory(uuidJobnetIdentifier);
    }

    // update the history record for the jobnetz
    @Override
    public void updateHistoryWithError(InternalJobNetError error, String... messageParameter) {
        // historyDBLayer.updateHistoryWithError(uuidJobnetIdentifier, exitCode, exitMessage);
        historyDBLayer.updateHistoryWithError(uuidJobnetIdentifier, error, messageParameter);
    }

    @Override
    public JobNetCmdHistoryDBItem getCmdHistory() {
        return commandRecord;
    }

    @Override
    public Boolean workWithEvents() {
        return createEvents;
    }

    @Override
    public StartModes getStartMode() {
        return startMode;
    }

    @Override
    public void readCurrentRecordAgain() {
        currentRecord = readCurrentRecordFromDBAndLock();
    }

    @Override
    public void setEventsFulfilled(List<EventsDBItem> eventsToSet) {
        conditionsDBLayer.setEventsFulfilled(getCurrentRecord(),eventsToSet);
    }

    @Override
    /**
     * The event conditions created by the PlanCreator can contain placeHolders for runtime attributes. Therefore
     * all active event conditions for the node will be replaced by conditions with real event names.
     */
    public void activateConditions() {

        // set all conditions of the node to state missing if there are still in state created
        List<ConditionsDBItem> conditions = conditionsDBLayer.switchToMissingState(currentRecord);
        logger.debug("{} conditions found for node {}.",conditions.size(),currentRecord.getNodeName());

        // replace placeholder in the event names
        for(ConditionsDBItem condition : conditions) {
            if(condition.getConditionType() == ConditionType.EVENT && condition.getConditionState() == ConditionState.MISSING) {
                JobNetHistoryDBItem jobnetHistoryRecord = currentRecord.getJobNetHistoryDBItem();
                JobNetDBItem jobnetRecord = jobnetHistoryRecord.getJobNetDBItem();
                ParameterSubstitution s = new ParameterSubstitution(jobnetHistoryRecord.getUuid(), jobnetRecord.getJobNetName(), jobnetHistoryRecord.getJobNetIdParameterName());
                condition.setConditionName(s.replace(condition.getConditionName()));
                condition.setNote(s.replace(condition.getNote()));
                conditionsDBLayer.update(condition);
            }
        }
    }

    @Override
    public void skipConditions() {
        conditionsDBLayer.switchToSkippedState(currentRecord);
    }

}
