package com.sos.jobnet.options;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.*;
import com.sos.hibernate.options.HibernateOptions;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.creator.JobNetConifigurationLayer;
import com.sos.jobnet.creator.ObjectModelConnector;
import com.sos.jobnet.creator.OrderNameCreator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;

/**
 * Options to create a jobnet.
 */
@JSOptionClass(name = "JobNetCreatorOptions", description = "JobNetCreatorOptions")
public class JobNetCreatorOptions extends HibernateOptions {
	
	private final static Logger logger = LoggerFactory.getLogger(JobNetCreatorOptions.class);

	private static final long serialVersionUID = 2632562825510369271L;
	private final String conClassName = JobNetCreatorOptions.class.getSimpleName();
	
	private ObjectModelConnector liveConnector = null;
    private OrderNameCreator nameComponents = null;
    private JobNetConifigurationLayer dbConfigurationLayer = null;

	@SuppressWarnings("unused")
	private final String conSVNVersion = "$Id: JobNetCreatorOptionsSuperClass.java 16850 2012-03-21 12:14:23Z ss $";

    // ---------------------------------------------------------------------------------------------------
	/**
	 * The unique id of the jobnet. If this parameter is not sepcified, the JObNetProcessor will be assign a unique identifier (UUID) for the
     * jobnet instance. In general it is not possible to start a second istance of a jobnet with the same ID.
	 */
	@JSOptionDefinition(name = "JobNetId", description = "", key = "JobNetId", type = "SOSOptionUUID", mandatory = false)
	public SOSOptionUUID JobNetId = new SOSOptionUUID(this, conClassName
			+ ".JobNetId", // HashMap-Key
			"", // Titel
			"", // InitValue
			"", // DefaultValue
			false // isMandatory
	);

	public void setJobNetId(SOSOptionUUID jobNetId) {
		this.JobNetId = jobNetId;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * The qualifier of a jobnet instance (e.g. _ind1)
	 */
	@JSOptionDefinition(name = "JobNetIdQualifier", description = "", key = "JobNetIdQualifier", type = "SOSOptionString", mandatory = false)
	public SOSOptionString JobNetIdQualifier = new SOSOptionString(this, conClassName
			+ ".JobNetIdQualifier", // HashMap-Key
			"", // Titel
			"", // InitValue
			"", // DefaultValue
			false // isMandatory
	);

	public void setJobNetIdQualifier(SOSOptionString jobNetIdQualifier) {
		this.JobNetIdQualifier = jobNetIdQualifier;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Name of the result parameter (it is always an order parameter).
	 */
	@JSOptionDefinition(name = "JobNetIdParameterName", description = "The name the parameter contains the uuid.", key = "JobNetIdParameterName", type = "SOSOptionString", mandatory = false)
	public SOSOptionString JobNetIdParameterName = new SOSOptionString(this,
			getClass().getSimpleName() + ".JobNetIdParameterName", // HashMap-Key
			"", // Titel
			"", // InitValue
			"", // DefaultValue
			false // isMandatory
	);

	public void setJobNetIdParameterName(
			SOSOptionString jobnet_id_parameter_name) {
		this.JobNetIdParameterName = jobnet_id_parameter_name;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * JobChainName.
	 */
	@JSOptionDefinition(name = "JobChainName", description = "", key = "JobChainName", type = "SOSOptionString", mandatory = true)
	public SOSOptionString JobChainName = new SOSOptionString(this,
			conClassName + ".JobChainName", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setJobChainName(SOSOptionString p_JobChainName) {
		this.JobChainName = p_JobChainName;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * OrderId.
	 */
	@JSOptionDefinition(name = "OrderId", description = "", key = "OrderId", type = "SOSOptionString", mandatory = true)
	public SOSOptionString OrderId = new SOSOptionString(this, conClassName
			+ ".OrderId", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setOrderId(SOSOptionString p_OrderId) {
		this.OrderId = p_OrderId;
	}

    // ---------------------------------------------------------------------------------------------------
    /**
     * Root folder of the job net orders relative to the configuration directory.
     */
    @JSOptionDefinition(name = "JobnetOrderDir", description = "", key = "JobnetOrderDir", type = "SOSOptionString", mandatory = false)
    public SOSOptionString JobnetOrderDir = new SOSOptionString(this,
            conClassName + ".JobnetOrderDir", // HashMap-Key
            "", // Titel
            " ", // InitValue
            " ", // DefaultValue
            false // isMandatory
    );

    public void setJobnetOrderDir(SOSOptionString p_folderName) {
        this.JobnetOrderDir = p_folderName;
    }

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Host
	 */
	@JSOptionDefinition(name = "SCHEDULER_HOST", description = "", key = "SCHEDULER_HOST", type = "SOSOptionHostName", mandatory = true)
	public SOSOptionHostName SCHEDULER_HOST = new SOSOptionHostName(this,
			conClassName + ".SCHEDULER_HOST", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setSCHEDULER_HOST(SOSOptionHostName p_host) {
		this.SCHEDULER_HOST = p_host;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Maximun number of nodes in one job net.
	 */
	@JSOptionDefinition(name = "MaxJobNetNodes", description = "", key = "MaxJobNetNodes", type = "SOSOptionInteger", mandatory = false)
	public SOSOptionInteger MaxJobNetNodes = new SOSOptionInteger(this,
			conClassName + ".MaxJobNetNodes", // HashMap-Key
			"", // Titel
			"500", // InitValue
			"500", // DefaultValue
			false // isMandatory
	);

	public void setMaxJobNetNodes(SOSOptionInteger p_maxJobNetNodes) {
		this.MaxJobNetNodes = p_maxJobNetNodes;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Additional to the order parameter successor a second parameter predecessor can be set. It defines the relationship to
	 * all predecessors of a node in the same manner as in case of the successors.
     * This parameter gives the ability to check the texture of the jobnet. Each successor defined at one node must have a predecessor
     * vice versa.
	 */
	@JSOptionDefinition(name = "CheckJobNetTexture", description = "", key = "CheckJobNetTexture", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean CheckJobNetTexture = new SOSOptionBoolean(this,
			conClassName + ".CheckJobNetTexture", // HashMap-Key
			"", // Titel
			"true", // InitValue
			"true", // DefaultValue
			false // isMandatory
	);

	public SOSOptionBoolean getCheckJobNetTexture() {
		return CheckJobNetTexture;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Flag to determine if an order should create and start after buildung the jobnet.
	 */
	@JSOptionDefinition(name = "CheckFrequency", description = "", key = "CheckFrequency", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean CheckFrequency = new SOSOptionBoolean(this,
			conClassName + ".CheckFrequency", // HashMap-Key
			"", // Titel
			"true", // InitValue
			"true", // DefaultValue
			false // isMandatory
	);

	public void setCheckFrequency(SOSOptionBoolean p_CheckFrequency) {
		this.CheckFrequency = p_CheckFrequency;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * A flag to choose if a static graphical representation of the jobnet will be created. To specify where the graph take
     * place use the option GraphDir.
	 */
	@JSOptionDefinition(name = "CreateGraph", description = "", key = "CreateGraph", type = "SOSOptionBoolean", mandatory = false)
	public SOSOptionBoolean CreateGraph = new SOSOptionBoolean(this,
			conClassName + ".CreateGraph", // HashMap-Key
			"", // Titel
			"false", // InitValue
			"false", // DefaultValue
			false // isMandatory
	);

	public void setCreateGraph(SOSOptionBoolean p_CreateGraph) {
		this.CreateGraph = p_CreateGraph;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Folder to store job net graphs. It has no effect if option CreateGraph is set to false.
	 */
	@JSOptionDefinition(name = "GraphDir", description = "", key = "GraphDir", type = "SOSOptionFolderName", mandatory = false)
	public SOSOptionFolderName GraphDir = new SOSOptionFolderName(this,
			conClassName + ".GraphDir", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setGraphDir(SOSOptionFolderName p_folderName) {
		this.GraphDir = p_folderName;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * This options specifies the Job chain to create the graph for the jobnet. If you use the default layout of the live folder for
     * jobnets it is not necessary to set this parameter.
	 */
	@JSOptionDefinition(name = "GraphBuilderJobChain", description = "", key = "GraphBuilderJobChain", type = "SOSOptionString", mandatory = false)
	public SOSOptionString GraphBuilderJobChain = new SOSOptionString(this,
			conClassName + ".GraphBuilderJobChain", // HashMap-Key
			"", // Titel
			"/jobnet/graph/jobnet_graph_builder", // InitValue
			"/jobnet/graph/jobnet_graph_builder", // DefaultValue
			true // isMandatory
	);

	public SOSOptionString getGraphBuilderJobChain() {
		return GraphBuilderJobChain;
	}

	public void setGraphBuilderJobChain(SOSOptionString jobChain) {
		this.GraphBuilderJobChain = jobChain;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * Port
	 */
	@JSOptionDefinition(name = "SCHEDULER_TCP_PORT", description = "", key = "SCHEDULER_TCP_PORT", type = "SOSOptionPortNumber", mandatory = true)
	public SOSOptionPortNumber SCHEDULER_TCP_PORT = new SOSOptionPortNumber(
			this, conClassName + ".SCHEDULER_TCP_PORT", // HashMap-Key
			"", // Titel
			"0", // InitValue
			"0", // DefaultValue
			true // isMandatory
	);

    public void setSCHEDULER_TCP_PORT(SOSOptionPortNumber p_port) {
		this.SCHEDULER_TCP_PORT = p_port;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * SCHEDULER_CONFIGURATION_DIRECTORY : root folder of the job net defintions.
	 */
	@JSOptionDefinition(name = "SCHEDULER_CONFIGURATION_DIRECTORY", description = "", key = "SCHEDULER_CONFIGURATION_DIRECTORY", type = "SOSOptionFolderName", mandatory = false)
	public SOSOptionFolderName SCHEDULER_CONFIGURATION_DIRECTORY = new SOSOptionFolderName(
			this, conClassName + ".SCHEDULER_CONFIGURATION_DIRECTORY", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setSCHEDULER_CONFIGURATION_DIRECTORY(
			SOSOptionFolderName p_liveFolder) {
		this.SCHEDULER_CONFIGURATION_DIRECTORY = p_liveFolder;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * The scheduler id.
	 */
	@JSOptionDefinition(name = "SCHEDULER_ID", description = "", key = "SCHEDULER_ID", type = "SOSOptionString", mandatory = false)
	public SOSOptionString SCHEDULER_ID = new SOSOptionString(this,
			conClassName + ".SCHEDULER_ID", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setSCHEDULER_ID(SOSOptionString p_schedulerId) {
		this.SCHEDULER_ID = p_schedulerId;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * The time horizon for which the jobnet instances will be calculated.
	 */
	@JSOptionDefinition(name = "TimeHorizon", description = "", key = "TimeHorizon", type = "SOSOptionTimeHorizon", mandatory = false)
	public SOSOptionTimeHorizon TimeHorizon = new SOSOptionTimeHorizon(this,
			conClassName + ".TimeHorizon", // HashMap-Key
			"", // Titel
			"1:00:00:00", // InitValue
			"1:00:00:00", // DefaultValue
			false // isMandatory
	);

	public void settime_horizon(SOSOptionTimeHorizon p_time_horizon) {
		this.TimeHorizon = p_time_horizon;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * A parameter to restart an existing jobnet instance (JobNetId has to be set).
	 */
	@JSOptionDefinition(name = "StartOption", description = "", key = "StartOption", type = "SOSOptionStartJobnet", mandatory = false)
	public SOSOptionStartJobnet StartOption = new SOSOptionStartJobnet(this, conClassName
			+ ".StartOption", // HashMap-Key
			"", // Titel
			"start", // InitValue
			"start", // DefaultValue
			false // isMandatory
	);

	public void setStartOption(SOSOptionStartJobnet startOption) {
		this.StartOption = startOption;
	}

    // ---------------------------------------------------------------------------------------------------
	/**
	 * The behaviour for start
	 */
	@JSOptionDefinition(name = "StartBehaviour", description = "", key = "StartBehaviour", type = "SOSOptionStartBehaviourJobnet", mandatory = false)
	public SOSOptionStartBehaviourJobnet StartBehaviour = new SOSOptionStartBehaviourJobnet(this, conClassName
			+ ".StartBehaviour", // HashMap-Key
			"", // Titel
			"immediately", // InitValue
			"immediately", // DefaultValue
			false // isMandatory
	);

	public void setStartBehaviour(SOSOptionStartBehaviourJobnet startBehaviour) {
		this.StartBehaviour = startBehaviour;
	}

    // ---------------------------------------------------------------------------------------------------
    /**
     * \var uuid_jobnet_identifier :
     */
    @JSOptionDefinition(name = "uuid_jobnet_identifier", description = "", key = "uuid_jobnet_identifier", type = "SOSOptionUUID", mandatory = false)
    public SOSOptionUUID uuid_jobnet_identifier = new SOSOptionUUID(this, conClassName + ".uuid_jobnet_identifier", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

    public void setuuid_jobnet_identifier (SOSOptionUUID p_uuid_jobnet_identifier) { 
        this.uuid_jobnet_identifier = p_uuid_jobnet_identifier;
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * The scheduler id.
     */
    @JSOptionDefinition(name = "APIStart", description = "", key = "APIStart", type = "SOSOptionBoolean", mandatory = false)
    public SOSOptionBoolean APIStart = new SOSOptionBoolean(this,
            conClassName + ".APIStart", // HashMap-Key
            "", // Titel
            "false", // InitValue
            "false", // DefaultValue
            false // isMandatory
    );

    public void setAPIStart(SOSOptionBoolean p_APIStart) {
        this.APIStart = p_APIStart;
    }

    // ---------------------------------------------------------------------------------------------------

	public ObjectModelConnector getOrderConnector() {
		if (liveConnector == null) {
			try {
				if (SCHEDULER_HOST.IsEmpty() || SCHEDULER_TCP_PORT.IsEmpty() || SCHEDULER_CONFIGURATION_DIRECTORY.IsEmpty())
					throw new JobNetException("Could not establish the connection to the live folder - SCHEDULER_HOST, SCHEDULER_TCP_PORT or SCHEDULER_CONFIGURATION_DIRECTORY is missing.");
                String configDir = SCHEDULER_CONFIGURATION_DIRECTORY.Value();
                if(!JobnetOrderDir.Value().trim().isEmpty()) {
                    configDir += JobnetOrderDir.Value();
                    logger.info("Special configuration directory for jobnet orders is given: " + JobnetOrderDir.Value());
                }
				liveConnector = ObjectModelConnector.getInstance(SCHEDULER_HOST, SCHEDULER_TCP_PORT, configDir);
                logger.info("The root directory to read the orders is " + liveConnector.getRootDirectory());
				// liveConnector = ObjectModelConnector.getInstance(SCHEDULER_HOST, SCHEDULER_TCP_PORT, SCHEDULER_CONFIGURATION_DIRECTORY);
			} catch (Exception e) {
				logger.error("Error connecting live folder: " + e.getMessage());
				throw new JobNetException("Error connecting live folder.", e);
			}
		}
		return liveConnector;
	}

    public String getJobChainNameWithoutPath() {
        return getNameComponents().getChainWithoutPath();
    }

    public String getOrderIdWithPath() {
        return getNameComponents().getOrderWithPath();
    }

    public OrderNameCreator getNameComponents() {
        if(nameComponents == null) {
            nameComponents = new OrderNameCreator(SCHEDULER_CONFIGURATION_DIRECTORY.Value(), OrderId.Value(),JobChainName.Value());
        }
        return nameComponents;
    }

    // ---------------------------------------------------------------------------------------------------

    public JobNetConifigurationLayer getDbConfigurationLayer() {
        if(dbConfigurationLayer == null)
            throw new JobNetException("No instance of JobNetConifigurationLayer is available - please call com.sos.jobnet.options.JobNetCreatorOptions.setAllOptions() first.");
        return dbConfigurationLayer;
    }

    // ---------------------------------------------------------------------------------------------------
    
	public JobNetCreatorOptions() {
		objParentClass = this.getClass();
	} // public JobNetCreatorOptionsSuperClass

	public JobNetCreatorOptions(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public JobNetCreatorOptionsSuperClass

	//

	public JobNetCreatorOptions(HashMap<String, String> JSSettings)
			throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public JobNetCreatorOptionsSuperClass (HashMap JSSettings)

	/**
	 * \brief setAllOptions - uebernimmt die OptionenWerte aus der HashMap
	 * 
	 * \see JSOptionsClass::getOrderRecord
	 * 
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	public void setAllOptions(HashMap<String, String> pobjJSSettings) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
        dbConfigurationLayer = new JobNetConifigurationLayer(this);
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	/**
	 * \brief CheckMandatory - prueft alle Muss-Optionen auf Werte
	 * 
	 * \details
	 * 
	 * @throws Exception
	 * 
	 * @throws Exception
	 *             - wird ausgeloest, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
			, Exception {
		try {
			super.CheckMandatory();
		} catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 * 
	 * \brief CommandLineArgs - uebernehmen der Options/Settings aus der
	 * Kommandozeile
	 * 
	 * \details Die in der Kommandozeile beim Starten der Applikation
	 * angegebenen Parameter werden hier in die HashMap uebertragen und danach
	 * den Optionen als Wert zugewiesen.
	 * 
	 * \return void
	 * 
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class JobNetCreatorOptionsSuperClass
