package com.sos.jobnet.utils;

import com.google.common.collect.ImmutableMap;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import org.joda.time.DurationFieldType;

/**
 * Tis class gets the DurationFieldType of the joda framework via the name of the duration type.
 *
 * @author  ss
 * @version 1.0
 */
public class NamedDurationFieldType {

    private final static ImmutableMap<String,DurationFieldType> fieldMap = ImmutableMap.<String,DurationFieldType>builder()
        .put("centuries", DurationFieldType.centuries())
        .put("eras", DurationFieldType.eras())
        .put("years", DurationFieldType.years())
        .put("months", DurationFieldType.months())
        .put("days", DurationFieldType.days())
        .put("halfdays", DurationFieldType.halfdays())
        .put("weeks", DurationFieldType.weeks())
        .put("weekyears", DurationFieldType.weekyears())
        .put("hours", DurationFieldType.hours())
        .put("minutes", DurationFieldType.minutes())
        .put("seconds", DurationFieldType.seconds())
        .put("millis", DurationFieldType.millis())
        .build();

    /**
     * Get the DurationFieldType via its name.
     *
     * @param durationType
     * @return
     */
    public static DurationFieldType of(String durationType) {
        if (!fieldMap.containsKey(durationType))
            throw new JobSchedulerException("The DurationType " + durationType + " does not exist.");
        return fieldMap.get(durationType);
    }

}
