package com.sos.jobnetmodel.live;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnetmodel.objects.ExternalJobnetNodeObject;
import com.sos.jobnetmodel.objects.JobnetAdapter;
import com.sos.scheduler.model.LiveConnector;
import com.sos.scheduler.model.SchedulerHotFolder;
import com.sos.scheduler.model.SchedulerHotFolderFileList;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.objects.JSObjOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.List;

public class JSJobnetAdapter extends JobnetAdapter {

    private final Logger logger = LoggerFactory.getLogger(JSJobnetAdapter.class);

    private final HashMap<String,ExternalJobnetNodeObject> records = new HashMap<String,ExternalJobnetNodeObject>();

    public JSJobnetAdapter(File liveFolder) {
        List<JSObjOrder> orderList = readRecords(liveFolder);
        for(JSObjOrder order : orderList) {
            JSOrderAdapter adaptedOrder = new JSOrderAdapter(order);
            addNode(adaptedOrder);
        }
        if(getBootstrapNode() == null)
            throw new JobSchedulerException("The jobnet in folder " + liveFolder.getAbsolutePath() + " has not bootstrap order.");
    }

    private static List<JSObjOrder> readRecords(File liveFolder) {
        SchedulerHotFolderFileList fileList = null;
        try {
            SchedulerObjectFactory factory = new SchedulerObjectFactory();
            LiveConnector connector = null;
            connector = new LiveConnector(liveFolder);
            SchedulerHotFolder hotFolder = factory.createSchedulerHotFolder(connector.getHotFolderHandle());
            fileList = hotFolder.loadRecursive();
        } catch (MalformedURLException e) {
            throw new JobSchedulerException("Error retrieving contents of folder " + liveFolder.getAbsolutePath(),e);
        }
        return  fileList.getOrderList();
    }

}
