package com.sos.jobnetmodel.objects;

import com.sos.JSHelper.Exceptions.JobSchedulerException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ImmutableJobnetNodeImpl implements ImmutableJobnetNode {

    public static final String EMPTY = "";

    private final String id;                // the unique id of the node
    private final JobnetNodeType type;

    protected String uniqueName;        // the unique name of the node - maybe identical with id
    protected String name;              // the name of the node - maybe not unique
    protected ExternalJobnetNodeObject externalNodeObject = null;
    protected Map<String,ImmutableJobnetNode> childNodes = new HashMap<String,ImmutableJobnetNode>();
    protected Map<String,ImmutableJobnetNode> successorNodes = new HashMap<String,ImmutableJobnetNode>();
    protected Map<String,ImmutableJobnetNode> predecessorNodes = new HashMap<String,ImmutableJobnetNode>();

    protected boolean isOnDemandNode = false;
    protected List<String> waitForEvents = new ArrayList<String>();
    protected String eventOnSuccess = EMPTY;
    protected String eventOnError = EMPTY;
    protected String processClass = EMPTY;
    protected List<Integer> validErrors = new ArrayList<Integer>();
    protected JobnetContainer container = null;
    protected JobnetNode connector = null;
    protected JobnetNode subnet = null;
    protected NodeStatus status = NodeStatus.NOT_PROCESSED;
    protected String title = EMPTY;

    public ImmutableJobnetNodeImpl(String id, JobnetNodeType type) {
        if(id == null || id.isEmpty())
            throw new JobSchedulerException("An empty id ist not valid.");
        this.id = id;
        this.uniqueName = id;
        this.name = id;
        this.type = type;
    }

    @Override
    public String getInternalId() {
        return id;
    }

    @Override
    public String getUniqueName() {
        return uniqueName;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public JobnetNodeType getType() {
        return type;
    }

    @Override
    public boolean hasSuccessorNodes() {
        return (!successorNodes.isEmpty());
    }

    @Override
    public boolean hasPredecessorNodes() {
        return (!predecessorNodes.isEmpty());
    }

    @Override
    public ExternalJobnetNodeObject getExternalNodeObject() {
        return externalNodeObject;
    }

    @Override
    public JobnetContainer getContainer() {
        return container;
    }

    @Override
    public JobnetNode getConnector() {
        return connector;
    }

    @Override
    public JobnetNode getSubnet() {
        return subnet;
    }

    @Override
    public Map<String, ImmutableJobnetNode> getChildNodes() {
        return childNodes;
    }

    @Override
    public Map<String, ImmutableJobnetNode> getSuccessorNodes() {
        return successorNodes;
    }

    @Override
    public Map<String, ImmutableJobnetNode> getPredeceessorNodes() {
        return predecessorNodes;
    }

    @Override
    public boolean isBootstrapNode() {
        return (getPredeceessorNodes().size() == 0);
    }

    @Override
    public boolean isOnDemandNode() {
        return isOnDemandNode;
    }

    @Override
    public List<String> getWaitForEvents() {
        return waitForEvents;
    }

    @Override
    public String getEventOnSuccess() {
        return eventOnSuccess;
    }

    @Override
    public String getEventOnError() {
        return eventOnError;
    }

    @Override
    public String getProcessClass() {
        return processClass;
    }

    @Override
    public List<Integer> getValidErrors() {
        return validErrors;
    }

    @Override
    public NodeStatus getStatus() {
        return status;
    }

    @Override
    public String getTitle() {
        return title;
    }

    @Override
    public String toString() {
        return getUniqueName();
    }

}
