/* MySQL */

set SQL_MODE=ANSI_QUOTES;

CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_PLAN
(
   "PLAN_ID"                 INT(10)        NOT NULL AUTO_INCREMENT, /* Id */
   "UUID"                    VARCHAR(42)    NOT NULL,                /* reference to a jobnet instance in table SCHEDULER_JOB_NET_HISTORY */
   "HISTORY_ID"              INT(10)        NOT NULL,                /* SCHEDULER_JOB_NET_HISTORY.HISTORY_ID */
   "NODE_ID"                 INT(10)        NOT NULL,                /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "SUBNET_ID"               INT(10)        DEFAULT 0 NOT NULL,      /* PLAN_ID referenced to the surrounded subnet node */
   "CONNECTOR_ID"            INT(10)        DEFAULT 0 NOT NULL,      /* PLAN_ID referenced to the surrounded connector node */
   "TASK_ID"                 INT(10)        DEFAULT 0 NOT NULL,      /* Reference of the JobScheduler task e.g. for the history */
   "JOB_NAME"                VARCHAR(255)           ,             
   "NODE_TYPE"               VARCHAR(10)            ,                /* Type of jobnet node: SUBNET, CONNECTOR, JOB */
   "SCRIPT_TO_EXECUTE"       VARCHAR(255)           ,             
   "USE_ERROR_HANDLER"       INT(1)         DEFAULT 0 NOT NULL,      /* flag to perform the default error handling */
   "EXIT_CODE"               INT(10)        DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"            VARCHAR(255)           ,        
   "STATUS"                  INT(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_WAITER"           INT(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_RUNNER"           INT(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_DISPATCHER"       INT(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "IS_WAITER_SKIPPED"       INT(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_RUNNER_SKIPPED"       INT(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_DISPATCHER_SKIPPED"   INT(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_RUNNER_ON_DEMAND"     INT(1)         DEFAULT 0 NOT NULL,      /* */
   "PLANNED_START_TIME"      DATETIME               ,                /* */
   "START_TIME"              DATETIME               ,                /* */
   "END_TIME"                DATETIME               ,                /* */
   "ORDER_XML"               LONGTEXT               ,                /* xml definition of the order */
   "BOOTSTRAP"               INT(1)         DEFAULT 0 NOT NULL,      /* */
   "FREQUENCE_OVERWRITE"     INT(1)         DEFAULT 0 NOT NULL,      /* */
   "MESSAGE"                 VARCHAR(255)           ,                /* */
   "CREATED"                 DATETIME               ,                /* */
   "CREATED_BY"              VARCHAR(255)           ,                /* */
   "MODIFIED"                DATETIME               ,                /* */
   "MODIFIED_BY"             VARCHAR(255)           ,                /* */
   INDEX JOBNET_UUID         ( "UUID" ),
   INDEX JOBNET_NODE_TYPE    ( "NODE_TYPE" ),
   INDEX JOBNET_CONNECTOR_ID ( "CONNECTOR_ID" ),
   INDEX JOBNET_SUBNET_ID    ( "SUBNET_ID" ),
   UNIQUE KEY SCHEDULER_JOB_NET_PLAN_UNIQUE ( "UUID", "NODE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_NODES
(
   "NODE_ID"             INT(10)         NOT NULL AUTO_INCREMENT,    /* Id */
   "SCHEDULER_ID"        VARCHAR(255)    NOT NULL,                   /* JobScheduler Id */
   "NODE"                VARCHAR(255)    NOT NULL,                   /* Name of the job net node */
   "JOB_NET"             VARCHAR(255)    NOT NULL,                   /* Name of the job net */
   PRIMARY KEY ( "NODE_ID" )
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_EDGES
(
   "EDGE_ID"             INT(10)         NOT NULL AUTO_INCREMENT,  /* Id */
   "EDGE_TYPE"           CHAR(1)         NOT NULL,                 /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      INT(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       INT(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   UNIQUE KEY SCHEDULER_JOB_NET_EDGES_UNIQUE ( "EDGE_TYPE", "PARENT_NODE_ID", "CHILD_NODE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_HISTORY
(
   "HISTORY_ID"                INT(10)         NOT NULL AUTO_INCREMENT,  /* Id */
   "SCHEDULER_ID"              VARCHAR(255)    NOT NULL,                 /* JobScheduler Id */
   "NET_ID"                    INT(10)         NOT NULL,                 /* SCHEDULER_JOB_NET.NET_ID */
   "UUID"                      VARCHAR(42)     NOT NULL,                 /* */
   "START_TIME"                DATETIME                ,                 /* */
   "END_TIME"                  DATETIME                ,                 /* */
   "RUN_TIME"                  DATETIME                ,                 /* */
   "STATUS"                    INT(10)         DEFAULT 0 NOT NULL,       /* */
   "START_NODE_ID"             INT(10)         NOT NULL,                 /* Id of a job net node in table SCHEDULER_JOB_NET_NODES */
   "NUMBER_OF_NODES"           INT(10)         DEFAULT 0 NOT NULL,         /* */
   "NUMBER_OF_FINISHED_NODES"  INT(10)         DEFAULT 0 NOT NULL,         /* */
   "EXIT_CODE"                 INT(10)         DEFAULT 0 NOT NULL,       /* */  
   "EXIT_MESSAGE"              VARCHAR(255)            ,        
   "JOBNET_ID"                 VARCHAR(255)            ,                 /* external (custom) Jobnet id, e.g. Timekey */
   "JOBNET_ID_PARAMETER_NAME"  VARCHAR(255)            ,        
   "CREATED"                   DATETIME                ,                 /* */
   "CREATED_BY"                VARCHAR(255)            ,                 /* */
   "MODIFIED"                  DATETIME                ,                 /* */
   "MODIFIED_BY"               VARCHAR(255)            ,                 /* */
   PRIMARY KEY ( "HISTORY_ID" )
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET
(
   "NET_ID"                    INT(10)         NOT NULL AUTO_INCREMENT,  /* Id */
   "SCHEDULER_ID"              VARCHAR(255)    NOT NULL,                 /* JobScheduler Id */
   "JOB_NET"                   VARCHAR(255)    NOT NULL,                 /* Name of the job net */
   "JOB_NET_NAME"              VARCHAR(255)    NOT NULL,                 /* Name of the job net */
   PRIMARY KEY ( "NET_ID" )
) ENGINE=InnoDB;

CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_CMD_HISTORY
(
   "CMD_HISTORY_ID"            INT(10)         NOT NULL AUTO_INCREMENT,  /* Id */
   "HISTORY_ID"                INT(10)         NOT NULL,                 /* reference to a jobnet instance in table SCHEDULER_JOB_NET_HISTORY */
   "START_OPTION"              VARCHAR(255)    NOT NULL,                 /* start, start_independent, repeat, restart_suspended etc. */
   "NODE_ID"                   INT(10)         NOT NULL,                 /* reference to the start node of the context in table SCHEDULER_JOB_NET_NODES */
   "ACKNOWLEDGED"              INT(1)          DEFAULT 0 NOT NULL,       /* */
   "MESSAGE"                   VARCHAR(255)            ,                 /* external (custom) Jobnet id, e.g. Timekey */
   "START_TIME"                DATETIME                ,                 /* */
   "CREATED"                   DATETIME               ,                  /* */
   "CREATED_BY"                VARCHAR(255)           ,                  /* */
    PRIMARY KEY ( "CMD_HISTORY_ID" )
) ENGINE=InnoDB;


CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_CONDITIONS
(
   "CONDITION_ID"              INT(10)         NOT NULL,                 /* Id */
   "NODE_ID"                   INT(10)         NOT NULL,                 /* reference to the node in table SCHEDULER_JOB_NET_NODES */
   "CONDITION_TYPE"            VARCHAR(32)     NOT NULL,                 /* event_expression, predecessor, approval */ 
   "CONDITION"                 VARCHAR(255)    NOT NULL,                 /* event name or node name */
   "CONDITION_STATE"           VARCHAR(32)     NOT NULL,                 /* waiting, fulfilled, approved */
   "NOTE"                      VARCHAR(255)    NOT NULL, 
   "MAIL_TO_ON_APPROVE"        VARCHAR(1024)   NOT NULL,
   "CHANGED_AT"                DATETIME        NOT NULL,                 /* */
   "CHANGED_BY"                VARCHAR(255)    NOT NULL,                 /* */
   PRIMARY KEY ( "CONDITION_ID" )
) ENGINE=InnoDB;
