/* Oracle */

DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_PLAN'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_PLAN
(
   "PLAN_ID"             NUMBER(10)      NOT NULL,         /* Id */
   "UUID"                VARCHAR2(42)    NOT NULL,         /* Unique Id */
   "HISTORY_ID"          NUMBER(10)      NOT NULL,         /* SCHEDULER_JOB_NET_HISTORY.HISTORY_ID */
   "NODE_ID"             NUMBER(10)      NOT NULL,         /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "PARENT_ID"           NUMBER(10)      DEFAULT 0 NOT NULL,      /* */
   "SUBNET_ID"           NUMBER(10)      DEFAULT 0 NOT NULL,      /* */
   "CONNECTOR_ID"        NUMBER(10)      DEFAULT 0 NOT NULL,      /* */
   "TASK_ID"             NUMBER(10)      DEFAULT 0 NOT NULL,      /* Reference e.g. for the history */
   "JOB_NAME"            VARCHAR2(255)           ,             
   "NODE_NAME"           VARCHAR2(255)   NOT NULL,             
   "NODE_TYPE"           VARCHAR2(10)            ,         /* Type of jobnet node: HEAD, CONNECTOR, JOB */
   "SCRIPT_TO_EXECUTE"   VARCHAR2(255)           ,             
   "USE_ERROR_HANDLER"   NUMBER(1)       DEFAULT 0 NOT NULL,      /* flag to perform the default error handling */
   "EXIT_CODE"           NUMBER(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"        VARCHAR2(255)           ,        
   "STATUS"              NUMBER(2)               ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_WAITER"       NUMBER(2)               ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_RUNNER"       NUMBER(2)               ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_DISPATCHER"   NUMBER(2)               ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "IS_WAITER_SKIPPED"       NUMBER(1)  DEFAULT 0 NOT NULL,  /* */
   "IS_RUNNER_SKIPPED"       NUMBER(1)  DEFAULT 0 NOT NULL,  /* */
   "IS_DISPATCHER_SKIPPED"   NUMBER(1)  DEFAULT 0 NOT NULL,  /* */
   "IS_RUNNER_ON_DEMAND"     NUMBER(1)  DEFAULT 0 NOT NULL,  /* */
   "PLANNED_START_TIME"  DATE                    ,         /* */
   "START_TIME"          DATE                    ,         /* */
   "END_TIME"            DATE                    ,         /* */
   "PARAM_XML"           CLOB                    ,         /* parameters set by predecessor nodes */
   "ORDER_XML"           CLOB                    ,         /* xml definition of the order */
   "BOOTSTRAP"           NUMBER(1)       DEFAULT 0 NOT NULL,  /* */
   "FREQUENCE_OVERWRITE" NUMBER(1)       DEFAULT 0 NOT NULL,  /* */
   "MESSAGE"             VARCHAR2(255)           ,         /* */
   "CREATED"             DATE                    ,         /* */
   "CREATED_BY"          VARCHAR2(255)           ,         /* */
   "MODIFIED"            DATE                    ,         /* */
   "MODIFIED_BY"         VARCHAR2(255)           ,         /* */
   CONSTRAINT SCHEDULER_JOB_NET_PLAN_UNIQUE UNIQUE( "UUID", "NODE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
)
'; END IF; END;
/

DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_INDEXES WHERE "INDEX_NAME"='JOBNET_UUID'; IF c = 0 THEN EXECUTE IMMEDIATE '
  CREATE INDEX JOBNET_UUID ON SCHEDULER_JOB_NET_PLAN ( "UUID" )
'; END IF; END;
/
DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_INDEXES WHERE "INDEX_NAME"='JOBNET_NODE_TYPE'; IF c = 0 THEN EXECUTE IMMEDIATE '
  CREATE INDEX JOBNET_NODE_TYPE ON SCHEDULER_JOB_NET_PLAN ( "NODE_TYPE" )
'; END IF; END;
/
DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_INDEXES WHERE "INDEX_NAME"='JOBNET_CONNECTOR_ID'; IF c = 0 THEN EXECUTE IMMEDIATE '
  CREATE INDEX JOBNET_CONNECTOR_ID ON SCHEDULER_JOB_NET_PLAN ( "CONNECTOR_ID" )
'; END IF; END;
/
DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_INDEXES WHERE "INDEX_NAME"='JOBNET_SUBNET_ID'; IF c = 0 THEN EXECUTE IMMEDIATE '
  CREATE INDEX JOBNET_SUBNET_ID ON SCHEDULER_JOB_NET_PLAN ( "SUBNET_ID" )
'; END IF; END;
/




DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_NODES'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_NODES
(
   "NODE_ID"             NUMBER(10)      NOT NULL,         /* Id */
   "SCHEDULER_ID"        VARCHAR2(255)   NOT NULL,         /* JobScheduler Id */
   "NODE"                VARCHAR2(255)   NOT NULL,         /* Name of the job net node */
   "JOB_NET"             VARCHAR2(255)   NOT NULL,         /* Name of the job net */
   PRIMARY KEY ( "NODE_ID" )
)
'; END IF; END;
/

DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_EDGES'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_EDGES
(
   "EDGE_ID"             NUMBER(10)      NOT NULL,         /* Id */
   "EDGE_TYPE"           CHAR(1)         NOT NULL,         /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      NUMBER(10)      NOT NULL,         /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       NUMBER(10)      NOT NULL,         /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHEDULER_JOB_NET_EDGES_UNIQUE UNIQUE( "EDGE_TYPE", "PARENT_NODE_ID", "CHILD_NODE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
)
'; END IF; END;
/

DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_HISTORY'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_HISTORY
(
   "HISTORY_ID"                NUMBER(10)      NOT NULL,         /* Id */
   "SCHEDULER_ID"              VARCHAR2(255)   NOT NULL,         /* JobScheduler Id */
   "NET_ID"                    NUMBER(10)      NOT NULL,         /* SCHEDULER_JOB_NET.NET_ID */
   "UUID"                      VARCHAR2(42)    NOT NULL,         /* Unique Id */
   "START_TIME"                DATE                    ,         /* */
   "END_TIME"                  DATE                    ,         /* */
   "RUN_TIME"                  DATE                    ,         /* */
   "STATUS"                    NUMBER(10)              ,         /* */
   "START_NODE_ID"             NUMBER(10)      NOT NULL,         /* Id of a job net node in table SCHEDULER_JOB_NET_NODES */
   "NUMBER_OF_NODES"           NUMBER(10)      DEFAULT 0 NOT NULL,         /* */
   "NUMBER_OF_FINISHED_NODES"  NUMBER(10)      DEFAULT 0 NOT NULL,         /* */
   "EXIT_CODE"                 NUMBER(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"              VARCHAR2(255)           ,        
   "JOBNET_ID"                 VARCHAR2(255)           ,         /* external (custom) Jobnet id, e.g. Timekey. Need not be unique in the history */
   "JOBNET_ID_PARAMETER_NAME"  VARCHAR2(255)           ,        
   "CREATED"             DATE                    ,         /* */
   "CREATED_BY"          VARCHAR2(255)           ,         /* */
   "MODIFIED"            DATE                    ,         /* */
   "MODIFIED_BY"         VARCHAR2(255)           ,         /* */
   PRIMARY KEY ( "HISTORY_ID" )
)
'; END IF; END;
/


DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET
(
   "NET_ID"                    NUMBER(10)      NOT NULL,         /* Id */
   "SCHEDULER_ID"              VARCHAR2(255)   NOT NULL,         /* JobScheduler Id */
   "JOB_NET"                   VARCHAR2(255)   NOT NULL,         /* */
   "JOB_NET_NAME"              VARCHAR2(255)   NOT NULL,         /* */
   PRIMARY KEY ( "NET_ID" )
)
'; END IF; END;
/


DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_CMD_HISTORY'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_CMD_HISTORY
(
   "CMD_HISTORY_ID"            NUMBER(10)      NOT NULL,                 /* Id */
   "HISTORY_ID"                NUMBER(10)      NOT NULL,                 /* reference to a jobnet instance in table SCHEDULER_JOB_NET_HISTORY */
   "START_OPTION"              VARCHAR2(255)   NOT NULL,                 /* start, start_independent, repeat, restart_suspended etc. */
   "NODE_ID"                   NUMBER(10)      NOT NULL,                 /* reference to the start node of the context in table SCHEDULER_JOB_NET_NODES */
   "ACKNOWLEDGED"              NUMBER(1)       DEFAULT 0 NOT NULL,       /* */
   "MESSAGE"                   VARCHAR2(255)           ,                 /* */
   "START_TIME"                DATE                    ,                 /* */
   "CREATED"                   DATE                    ,                 /* */
   "CREATED_BY"                VARCHAR2(255)           ,                 /* */
   PRIMARY KEY ( "CMD_HISTORY_ID" )
)
'; END IF; END;
/

DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_TABLES WHERE "TABLE_NAME"='SCHEDULER_JOB_NET_CONDITIONS'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE TABLE SCHEDULER_JOB_NET_CONDITIONS
(
   "CONDITION_ID"              NUMBER(10)      NOT NULL,                 /* Id */
   "PLAN_ID"                   NUMBER(10)      NOT NULL,                 /* reference to the node in table SCHEDULER_JOB_NET_PLAN */
   "CONDITION_TYPE"            VARCHAR2(32)     NOT NULL,                 /* event_expression, predecessor, approval */ 
   "CONDITION"                 VARCHAR2(255)   NOT NULL,                 /* event name or node name */
   "CONDITION_STATE"           VARCHAR2(32)     NOT NULL,                 /* waiting, fulfilled, approved */
   "NOTE"                      VARCHAR2(255)   NOT NULL, 
   "MAIL_TO_ON_APPROVE"        VARCHAR2(1024),
   "CHANGED_AT"                DATE            NOT NULL,                 /* */
   "CHANGED_BY"                VARCHAR2(255)    NOT NULL,                 /* */
   "MISSED_FROM"               DATE,                 /* */
   "FINISHED_AT"               DATE,                 /* */
   PRIMARY KEY ( "CONDITION_ID" )
)
'; END IF; END;
/

COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CONDITION_ID IS 'the primary key';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.NODE_ID IS 'reference to the jobnet node in table SCHEDULER_JOB_NET_NODES';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CONDITION_TYPE IS 'event_expression, predecessor, approval';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CONDITION IS 'event expression, name of the predecessor node or something else';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CONDITION_STATE IS '0=waiting, 1=fulfilled, 2=approved';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.NOTE IS 'a frre text to descripte the cause of an approval';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.MAIL_TO_ON_APPROVE IS 'a comma separated list of mail recipients';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CHANGED_AT IS 'last modification time';
COMMENT ON COLUMN SCHEDULER_JOB_NET_CONDITIONS.CHANGED_BY IS 'last modification user';

/* Sequence for hibernate */
DECLARE c number; BEGIN SELECT COUNT(*) INTO c FROM USER_SEQUENCES WHERE "SEQUENCE_NAME"='HIBERNATE_SEQUENCE'; IF c = 0 THEN EXECUTE IMMEDIATE '
CREATE SEQUENCE HIBERNATE_SEQUENCE 
  NOCACHE
'; END IF; END;
/

