package com.sos.jobnet.classes;

import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.scheduler.cmd.JSAddOrder;
import com.sos.scheduler.model.answers.Answer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sos.spooler.Spooler;

/**
 * Start a jobnet order.
 *
 * @author Stefan Schaedlich
 * at 22.09.13 13:00
 */
public class JobNetNodeOrderStarter extends JSAddOrder {

    private Logger logger = LoggerFactory.getLogger(JobNetNodeOrderStarter.class);

    private Spooler spooler;

    public JobNetNodeOrderStarter(String host, Integer port, JobNetPlanDBItem planDBItem) {
        super(host, port, planDBItem.getOrderId(), planDBItem.getJobChain());
        logger.info("Order will be started for host " + host + ":" + port);
    }

    public void setSpooler(Spooler spooler) {
        this.spooler = spooler;
    }

    @Override
    public void run() {
        if (spooler == null)
            super.run();
        else {
            logger.info("*********************************** ORDER START VIA JOBSCHEDULER **********************************");
            try {
                logger.info(getXml());
                String request = getXml();
                String response = spooler.execute_xml(request);
                logger.info("JS answer received.");
                Answer answer = getFactory().getAnswer(response);
                if (answer != null) {
                    if (answer.getERROR() != null) {
                        String msg = "Error fetching XML command." + "\n" +
                                "command:\n" + request + "\n" +
                                "answer:\n" + response;
                        logger.error(msg);
                        throw new JobNetException(msg);
                        // an invalid or wrong command\n
                    }
                }
            } catch (Exception e) {
                String msg = "Error starting order.";
                logger.error(msg);
                throw new JobNetException(msg,e);
            }

        }
    }

}
