package com.sos.jobnet.context;

import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.db.*;
import com.sos.jobnetmodel.objects.NodeStatus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @uthor ss
 * at 20.09.13 11:21
 */
public class ContextBuilder {

    private final Logger logger = LoggerFactory.getLogger(ContextBuilder.class);

    private final String uuid;
    private final String orderIdWithoutPath;
    private final JobNetPlanDBLayer planDbLayer;
    private final JobNetHistoryDBLayer historyDBLayer;
    private final JobNetHistoryDBItem historyRecord;
    private final JobNetPlanDBItem startNode;
    private final String schedulerId;
    private final ContextList contextList;

    public ContextBuilder(String uuid, String orderId, JobNetPlanDBLayer planDbLayer) {
        this.uuid = uuid;
        this.orderIdWithoutPath = readOrderWithoutPath(orderId);
        this.planDbLayer = planDbLayer;
        this.historyDBLayer = new JobNetHistoryDBLayer(planDbLayer.getConfigurationFile());
        this.historyRecord = historyDBLayer.getJobNetInstance(uuid);

        if(historyRecord == null)
            throw new JobNetException("No history record found in table SCHEDULER_JOB_NET_HISTORY with uuid " + uuid);

        this.schedulerId = historyRecord.getSchedulerId();
        this.startNode = readStartNode();
        this.contextList = readContextList(startNode);
    }

    private String readOrderWithoutPath(String orderId) {
        String result = orderId;
        int i = orderId.lastIndexOf('/');
        if (i > 0)
            result = orderId.substring(i+1);
        return result;
    }


    /**
     * Read the startnode for a given jobnet instance
     * @return
     */
    private JobNetPlanDBItem readStartNode() {
        JobNetPlanFilter startNodeFilter = new JobNetPlanFilter();
        logger.info("Search jobnet node with filter: schedulerID=" + schedulerId + ", uuid=" + uuid + ", oderID=" + orderIdWithoutPath);
        startNodeFilter.setSchedulerId(schedulerId);
        startNodeFilter.setUuid(uuid);
        startNodeFilter.setNode(orderIdWithoutPath);
        planDbLayer.setFilter(startNodeFilter);
        List<JobNetPlanDBItem> records = planDbLayer.getJobnetPlanList(0);
        if(records.size() != 1)
            throw new JobNetException("The orderIdWithoutPath " + orderIdWithoutPath + " is not unique in the jobnet instance with uuid " + uuid + " - found " + records.size() + " nodes.");
        return records.get(0);
    }

    private ContextList readContextList(JobNetPlanDBItem startNode) {
        logger.info("Reading context nodes for order " + orderIdWithoutPath + " with UUID " + uuid);
        List<JobNetPlanDBItem> contextNodes = planDbLayer.getMemberListForContext(startNode);
        ContextList list = new ContextList(planDbLayer, contextNodes,startNode);
        logger.info(list.getNodesInContext().size() + " nodes found in context.");
        for(NodeStatus status : NodeStatus.values())
            logger.info(list.countNodesWithStatus(status) + " nodes are in status " + status.name() + ".");
        return list;
    }

    public ContextList getContextList() {
        return contextList;
    }

    public JobNetPlanDBItem getStartNode() {
        return startNode;
    }

    public JobNetHistoryDBItem getHistoryRecord() {
        return historyRecord;
    }

}
