package com.sos.jobnet.creator;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.db.*;
import com.sos.jobnet.options.JobNetCreatorOptions;
import com.sos.localization.Messages;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;

public class JobNetConifigurationLayer {

	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id: JobNetConifigurationLayer.java 23331 2014-03-25 09:48:33Z sos $";

	private final static Logger logger = LoggerFactory.getLogger(JobNetConifigurationLayer.class);

	private static final Messages msg = new JSToolBox(JobNetConstants.strBundleBaseName).getMessageObject();
	private final static String JOBNETC_I_0009 = "JOBNETC_I_0009"; // SAVE: jobchain=%1$s, node=%2$s, schedulerid=%3$s
	private final static String JOBNETC_E_0005 = "JOBNETC_E_0005"; // could not create %1$s for %2$s - no record found in job net nodes.

	private final JobNetCreatorOptions options;

	private final JobNetDBLayer dbLayerJobnet;
	private final JobNetCmdHistoryDBLayer dbLayerCommands;
	private final JobNetNodeDBLayer dbLayerNodes;
	private final JobNetEdgesDBLayer dbLayerEdges;
    private final JobNetHistoryDBLayer dbLayerHistory;
    private final JobNetPlanDBLayer dbLayerPlan;
    private final EventsDBLayer dbLayerEvents;
    private final ConditionsDBLayer dbLayerConditions;

    public JobNetConifigurationLayer(JobNetCreatorOptions options) {
        File configFile = new File(options.hibernate_connection_config_file.Value());
        String clusterId = options.SCHEDULER_ID.Value();
        this.dbLayerJobnet = new JobNetDBLayer(configFile);
        this.dbLayerCommands = new JobNetCmdHistoryDBLayer(configFile);
        this.dbLayerNodes = new JobNetNodeDBLayer(configFile,clusterId);
        this.dbLayerHistory = new JobNetHistoryDBLayer(configFile,clusterId);
        this.dbLayerPlan = new JobNetPlanDBLayer(configFile);
        this.dbLayerEdges = new JobNetEdgesDBLayer(configFile);
        this.dbLayerEvents = new EventsDBLayer(configFile);
        this.dbLayerConditions = new ConditionsDBLayer(configFile);
        this.options = options;
	}

    public void beginTransaction() {
        dbLayerJobnet.beginTransaction();
        dbLayerCommands.beginTransaction();
        dbLayerNodes.beginTransaction();
        dbLayerEdges.beginTransaction();		// is necessary for writing the edges
        dbLayerHistory.beginTransaction();
        dbLayerPlan.beginTransaction();
        dbLayerConditions.beginTransaction();
        dbLayerEvents.beginTransaction();
    }

    public void commit() {
        dbLayerJobnet.commit();
    }

    public void deleteAllEntriesInJobNetTables() {
        dbLayerJobnet.setFilter(new JobNetFilter());
        dbLayerCommands.setFilter(new JobNetCmdHistoryFilter());
        dbLayerNodes.setFilter(new JobNetNodeFilter());
        dbLayerEdges.setFilter(new JobNetEdgesFilter());
        dbLayerPlan.setFilter(new JobNetPlanFilter());
        dbLayerHistory.setFilter( new JobNetHistoryFilter() );
        beginTransaction();
        dbLayerJobnet.delete();
        dbLayerCommands.delete();
        dbLayerNodes.delete();
        dbLayerEdges.delete();
        dbLayerHistory.delete();
        dbLayerPlan.delete();
        dbLayerConditions.delete();
        dbLayerEvents.delete();
        commit();
    }

    public void closeSession() {
        dbLayerJobnet.closeSession();
        dbLayerCommands.closeSession();
        dbLayerNodes.closeSession();
        dbLayerEdges.closeSession();
        dbLayerHistory.closeSession();
        dbLayerConditions.closeSession();
        dbLayerEvents.closeSession();
    }

    public JobNetDBLayer getDbLayerJobnet() {
        return dbLayerJobnet;
    }

    public JobNetCmdHistoryDBLayer getDbLayerCommands() {
        return dbLayerCommands;
    }

    public JobNetNodeDBLayer getDbLayerNodes() {
        return dbLayerNodes;
    }

    public JobNetEdgesDBLayer getDbLayerEdges() {
        return dbLayerEdges;
    }

    public JobNetHistoryDBLayer getDbLayerHistory() {
        return dbLayerHistory;
    }

    public JobNetPlanDBLayer getDbLayerPlan() {
        return dbLayerPlan;
    }

    public ConditionsDBLayer getDbLayerConditions() {
        return dbLayerConditions;
    }

    public EventsDBLayer getDbLayerEvents() {
        return dbLayerEvents;
    }

    public JobNetCreatorOptions getOptions() {
        return options;
    }

}
