package com.sos.jobnet.creator;

import com.sos.JSHelper.Options.SOSOptionHostName;
import com.sos.JSHelper.Options.SOSOptionPortNumber;
import com.sos.VirtualFileSystem.Interfaces.ISOSVfsFileTransfer;
import com.sos.jobnet.classes.JobNetException;
import com.sos.scheduler.model.LiveConnector;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.objects.Spooler;
import com.sos.scheduler.model.tools.PathResolver;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ObjectModelConnector {

    private final static Logger logger = LoggerFactory.getLogger(ObjectModelConnector.class);

    private final SchedulerObjectFactory objectFactory;
    private final LiveConnector connector;

    private static ObjectModelConnector instance = null;

    private ObjectModelConnector(SOSOptionHostName host, SOSOptionPortNumber port, String folder) {
        try {
            objectFactory = new SchedulerObjectFactory();
            objectFactory.initMarshaller(Spooler.class);
            objectFactory.Options().ServerName = host;
            objectFactory.Options().PortNumber = port;
            logger.info("Object factory connected with " + getHostAndPort() + ".");
        } catch(Exception e) {
            throw new JobNetException("Unable to connect with SchedulerObjectFactory for server " + getHostAndPort());
        }

        String path = PathResolver.resolvePath(folder);
        try {
            connector = new LiveConnector( LiveConnector.getUrl(path) );
            logger.info("The configuration directory is " + getRootDirectory() + ".");
        } catch(Exception e) {
            throw new JobNetException("Unable to connect with live folder" + path);
        }
    }

    private String getHostAndPort() {
        String host = (objectFactory.Options().ServerName.Value().trim().isEmpty()) ? "???" : objectFactory.Options().ServerName.Value();
        String port = (objectFactory.Options().PortNumber.Value().equals("0")) ? "???" : objectFactory.Options().PortNumber.Value();
        return host + ":" + port;
    }

    public static ObjectModelConnector getInstance(SOSOptionHostName host, SOSOptionPortNumber port, String folder) {
        if (instance == null)
            instance = new ObjectModelConnector(host, port, folder);
        return instance;
    }

    public SchedulerObjectFactory getFactory() {
        return objectFactory;
    }

    public ISOSVfsFileTransfer getFileHandler() {
        return connector.getFileSystemHandler();
    }

    public String getWorkingDirectory() {
        return connector.getCurrentFolder();
    }

    public String getRootDirectory() {
        return connector.getLiveFolder();
    }

    public void setWorkingDirectory(String directory) {
        connector.setCurrentFolder(directory);
    }

    /**
     * Gets the base folder for an order. An order like /folder/orderId bases on the root directory (e.g. the
     * live folder.
     * Order with a relative names such as ../folder/orderId based on the working directory.
     * @param baseName
     * @return
     */
    public String selectBaseFolder(String baseName) {
        return connector.selectBaseFolder(baseName);
    }

}
