package com.sos.jobnet.creator;

public class ObjectNameCreator {

	protected final static String DELIMITER = "/";
	
	private final String baseFolder;			// the base folder (live folder)
	private final String objectId;
	private final Type type;
	private final String objectName;			// the object name used in JobScheduler 
	private final String objectFileName;		// the filename of the object (based on base folder)
	private final String objectFilePath;		// the file path of the object (based on base folder) 
	private final String fullName;				// the full filename of the object
	private final String fullPath;				// the full path of the folder the object stored in
	
	public ObjectNameCreator(String baseFolder, String objectId, Type type) {
		this.baseFolder = stripTrailingSlash(baseFolder.trim());
		this.objectId = objectId.trim();
		this.type = type;
		String[] arr = this.objectId.split(DELIMITER);
		this.objectName = getObjectName(arr[arr.length-1]);
		this.objectFilePath = normalizeSlashes(stripTrailingSlash( getObjectPath(this.objectId)));
//		this.objectFileName = objectFilePath + DELIMITER + this.objectName + "." + this.type.getExtension();
		this.objectFileName = this.objectName + "." + this.type.getExtension();
		this.fullPath = stripTrailingSlash(this.baseFolder + ensureLeadingSlash(objectFilePath) );
		this.fullName = this.fullPath + ensureLeadingSlash(this.objectFileName);
	}
	
	public static String concat(String... text) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < text.length; i++) {
			sb.append(text[i]);
			sb.append(DELIMITER);
		}
		return stripTrailingSlash(sb.toString());
	}
	
	public static String stripTrailingSlash(String text) {
		return (text.endsWith(DELIMITER)) ? text.substring(0,text.length()-1) : text;
	}
	
	public static String normalizeSlashes(String text) {
		return text.replace("//", "/");
	}
	
	public static String ensureLeadingSlash(String text) {
		return (text.startsWith(DELIMITER)) ? text : DELIMITER + text;
	}
	
	private String getObjectName(String id) {
		return id.replace("." + type.getExtension(), "");
	}
	
	private static String getObjectPath(String id) {
		StringBuffer result = new StringBuffer();
		String[] arr = id.split(DELIMITER);
		for (int i = 0; i < arr.length-1; i++) {
//			if (!arr[i].equals(".")) {
				result.append(arr[i]);
				result.append(DELIMITER);
//			}
		}
		return result.toString();
	}

	public String getObjectId() {
		return objectId;
	}

	public Type getType() {
		return type;
	}

	public String getObjectName() {
		return objectName;
	}

	public String getObjectFileName() {
		return objectFileName;
	}

	public String getObjectFilePath() {
		return objectFilePath;
	}

	public String getFullPath() {
		return fullPath;
	}

	public String getFullName() {
		return fullName;
	}

	public String getBasePath() {
		return baseFolder;
	}

	public enum Type {
		ORDER,
		JOB_CHAIN,
		SCHEDULE,
		JOB,
		LOCK,
		PROCESS_CLASS
		;
		
		public String getExtension() {
			return name().toLowerCase() + ".xml";
		}

	}
}
