package com.sos.jobnet.creator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OrderNameCreator extends ObjectNameCreator {
	
	private static final Logger logger = LoggerFactory.getLogger(OrderNameCreator.class);

	private String orderName;
	private final String objectNameExtended;
	private final String chainWithoutPath;
	private final String orderWithPath;
	private final String orderAndChain;

	public OrderNameCreator(String baseFolder, String orderId, String jobChain) {
		super(baseFolder, getOrderAndChain(orderId,jobChain), ObjectNameCreator.Type.ORDER);
		this.orderAndChain = getOrderAndChain(orderId,jobChain);
		this.objectNameExtended = orderAndChain + "." + Type.ORDER.getExtension();
		this.orderName = orderAndChain.split(",")[1];
        this.chainWithoutPath = getNameWithoutPath(jobChain);
        this.orderWithPath = getOrderWithPath(jobChain,orderId);
	}

	public OrderNameCreator(String orderId, String jobChain) {
		super("", getOrderAndChain(orderId,jobChain), ObjectNameCreator.Type.ORDER);
		this.orderAndChain = getOrderAndChain(orderId,jobChain);
		this.objectNameExtended = orderAndChain + "." + Type.ORDER.getExtension();
		this.orderName = orderAndChain.split(",")[1];
        this.chainWithoutPath = getNameWithoutPath(jobChain);
        this.orderWithPath = getOrderWithPath(jobChain,orderId);
	}

	private static String getOrderAndChain(String orderId, String jobChain) {
		if ( hasPath(jobChain)) {
            // throw new JobNetException("a path introduction in jobChain (" + jobChain + ") is not allowed - specify the path with the order.");
            int i = jobChain.lastIndexOf('/');
            String chainWithoutPath = jobChain.substring(i+1);
            String path = jobChain.substring(0,i+1);
            orderId = path + orderId;
            jobChain = chainWithoutPath;
        }

		ObjectNameCreator creator = null;
		String name = "";
		creator = new ObjectNameCreator("",orderId, ObjectNameCreator.Type.ORDER);
		name = jobChain + "," + creator.getObjectName();
		
		logger.debug("getOrderAndChain: orderId=" + orderId + ", jobChain=" + jobChain + ", chain+order=" + name);
		String result = ObjectNameCreator.concat(creator.getObjectFilePath(), name);
		logger.debug("result=" + result);
		return result;
	}

    private static String getNameWithoutPath(String jobChain) {
        String result = jobChain;
        if(hasPath(jobChain)) {
            int i = jobChain.lastIndexOf('/');
            result = jobChain.substring(i+1);
        }
        return result;
    }

    private static String getOrderWithPath(String jobChain, String order) {
        String result = order;
        if(!hasPath(order)) {
            String path = jobChain.replaceAll(getNameWithoutPath(jobChain) + "$","");
            result = path + order;
        }
        return result;
    }

    public String getChainWithoutPath() {
        return chainWithoutPath;
    }

    public String getOrderWithPath() {
        return orderWithPath;
    }

    public String getOrderAndChain() {
        return orderAndChain;
    }

	private static boolean hasPath(String objectName) {
		return objectName.contains("/");
	}

	@Override
	public String getObjectName() {
		return orderName;
	}

	@Override
	public String getObjectFileName() {
		return objectNameExtended;
	}

	@Override
	public String getFullName() {
		return getBasePath() + ensureLeadingSlash(getObjectFileName());
	}
	
}
