package com.sos.jobnet.creator.jobs;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnet.creator.JobNetCreator;
import com.sos.jobnet.options.JobNetCreatorOptions;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
 * \class 		JobNetCreatorJSAdapterClass - JobScheduler Adapter for "creates the job net for a given bootstrap order"
 *
 * \brief AdapterClass of JobNetCreator for the SOSJobScheduler
 *
 * This Class JobNetCreatorJSAdapterClass works as an adapter-class between the SOS
 * JobScheduler and the worker-class JobNetCreator.
 *

 *
 * see \see C:\Users\schaedi\AppData\Local\Temp\scheduler_editor-3377211190692187569.html for more details.
 *
 * \verbatim ;
 * mechanicaly created by C:\ProgramData\sos-berlin.com\jobscheduler\scheduler-1.3.12.2032\config\JOETemplates\java\xsl\JSJobDoc2JSAdapterClass.xsl from http://www.sos-berlin.com at 20120228153423 
 * \endverbatim
 */
public class JobNetCreatorJSAdapterClass extends JobNetJobAdapterClass  {

	private static Logger logger = LoggerFactory.getLogger(JobNetCreatorJSAdapterClass.class);
	
	@SuppressWarnings("unused")
	private	final String conSVNVersion = "$Id: JobNetCreatorJSAdapterClass.java 16850 2012-03-21 12:14:23Z ss $";
	
	private JobNetCreatorOptions options = new JobNetCreatorOptions();
    private JobNetCreator creator = null;

    public void init() {
	}

	@Override
	public boolean spooler_init() {
		return super.spooler_init();
	}

	@Override
	public boolean spooler_process() throws Exception {
		
		final String conMethodName = this.getClass().getSimpleName() + "::spooler_process"; //$NON-NLS-1$

		try {
			super.spooler_process();
			options = new JobNetCreatorOptions();
			options.setAllOptions(getAllParametersAsProperties());
			logger.debug(options.toString());
			creator = new JobNetCreator(options);
			creator.run();
		}
		catch (Exception e) {
			String msgText = String.format(Messages.getMsg("JSJ-I-107"), conMethodName );
            spooler_task.set_error(e.getMessage());
            // spooler_task.order().set_state_text(e.getMessage());
			throw new JobSchedulerException(msgText,e);
		} finally {
            creator.closeSession();
        }
        return (spooler_task.job().order_queue() != null);

	}

	@Override
	public void spooler_exit() {
		super.spooler_exit();
	}
	
	public JobNetCreatorOptions getOptions() {
		return options;
	}

}

