package com.sos.jobnet.db;

import com.sos.jobnet.classes.JobNetStatus;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.hibernate.annotations.Type;
import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;

import javax.persistence.*;


@Entity
@Table(name="SCHEDULER_JOB_NET_HISTORY")
public class JobNetHistoryDBItem extends JobNetTreeDBItem {
	
	private long historyId;
	private String schedulerId;
    private long netId;
    private String uuid;
    private DateTime startTime;
    private DateTime endTime;
    private DateTime runTime;
    private Integer status;
    private Integer exitCode;
    private String exitMessage;
    private Integer numberOfNodes;
    private Integer numberOfFinishedNodes;

    private long startNodeId;
    private JobNetNodeDBItem jobNetStartNodeDBItem;

    private String jobNetId;
    private String jobNetIdParameterName;
    
    private DateTime created;
    private String createdBy;
    private DateTime modified;
    private String modifiedBy;
    
    private JobNetDBItem jobNetDBItem;

	public JobNetHistoryDBItem() {
		super();
	}

    @Id
    @GeneratedValue(strategy=GenerationType.AUTO)
    @Column(name="HISTORY_ID")
    public Long getHistoryId() {
		return historyId;
	}

	@Column(name="HISTORY_ID")
    public void setHistoryId(Long historyId) {
		this.historyId = historyId;
	}
      
    @ManyToOne (optional=true)
    @NotFound( action = NotFoundAction.IGNORE )
    @JoinColumn (name="`NET_ID`",  insertable = false, updatable = false)
    public JobNetDBItem getJobNetDBItem() {
        return jobNetDBItem;
    }
    public void setJobNetDBItem(JobNetDBItem jobNetDBItem)  {
       this.jobNetDBItem = jobNetDBItem; 
    }

    @Column(name="NET_ID",nullable=true)
    public void setNetId(long netId) {
        this.netId = netId;
    }
 
    @Column(name="NET_ID",nullable=true)
    public long getNetId() {
        return netId;
    }


    @OneToOne (optional=true)
    @NotFound( action = NotFoundAction.IGNORE )
    @JoinColumn (name="`START_NODE_ID`", insertable = false, updatable = false)
    public JobNetNodeDBItem getJobNetStartNodeDBItem() {
        return jobNetStartNodeDBItem;
    }

    public void setJobNetStartNodeDBItem(JobNetNodeDBItem jobNetStartNodeDBItem)  {
        this.jobNetStartNodeDBItem = jobNetStartNodeDBItem;
    }


    @Column(name="START_NODE_ID")
    public Long getStartNodeId() {
        return startNodeId;
    }

    @Column(name="START_NODE_ID")
    public void setStartNodeId(Long startNodeId) {
        this.startNodeId = startNodeId;
    }

	
    @Column(name="SCHEDULER_ID",nullable=false)
	public void setSchedulerId(String schedulerId) {
		this.schedulerId = schedulerId;
	}
    
    @Column(name="SCHEDULER_ID",nullable=false)
	public String getSchedulerId() {
		return schedulerId;
	}
    
    
    @Column(name="UUID")
    public String getUuid() {
		return uuid;
	}

	@Column(name="UUID")
    public void setUuid(String uuid) {
		this.uuid = uuid;
	}
	
	    
    @Column(name="START_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public void setStartOfJobnet(DateTime startTime) {
        this.startTime = (startTime != null) ? startTime.toDateTime(DateTimeZone.UTC) : null;
    }
 
    @Column(name="START_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public DateTime getStartOfJobnet() {
        return (startTime != null) ? startTime.toDateTime(getDateTimeZone4Getters()) : null;
    } 
    
    @Column(name="END_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public void setEndOfJobnet(DateTime endTime) {
        this.endTime = (endTime != null) ? endTime.toDateTime(DateTimeZone.UTC) : null;
    }
 

    @Column(name="END_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public DateTime getEndOfJobnet() {
        return (endTime != null) ? endTime.toDateTime(getDateTimeZone4Getters()) : null;
    } 
  
  
        
    @Column(name="RUN_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public void setRunTime(DateTime runTime) {
        this.runTime = (runTime != null) ? runTime.toDateTime(DateTimeZone.UTC) : null;
    }
 
    @Column(name="RUN_TIME",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public DateTime getRunTime() {
        return (runTime != null) ? runTime.toDateTime(getDateTimeZone4Getters()) : null;
    } 
  
  
    @Column(name="EXIT_MESSAGE",nullable=true)
	public void setExitMessage(String exitMessage) {
		this.exitMessage = exitMessage;
	}
 
    @Column(name="EXIT_MESSAGE",nullable=true)
    public String getExitMessage() {
        return exitMessage;
    } 


    
    @Column(name="JobNet_ID",nullable=true)
	public void setJobNetId(String JobNetId) {
		this.jobNetId = JobNetId;
	}
 
    @Column(name="JobNet_ID",nullable=true)
	public String getJobNetId() {
		return jobNetId;
	} 
    
    @Column(name="JobNet_ID_PARAMETER_NAME",nullable=true)
	public void setJobNetIdParameterName(String JobNetIdParameterName) {
		this.jobNetIdParameterName = JobNetIdParameterName;
	}
 
    @Column(name="JobNet_ID_PARAMETER_NAME",nullable=true)
	public String getJobNetIdParameterName() {
		return jobNetIdParameterName;
	} 
     
    @Column(name="EXIT_CODE",nullable=true)
	public void setExitCode(Integer exitCode) {
		this.exitCode = exitCode;
	}

    @Column(name="EXIT_CODE",nullable=true)
    public Integer getExitCode() {
        return exitCode;
    }

    @Column(name="NUMBER_OF_NODES",nullable=true)
    public Integer getNumberOfNodes() {
        return numberOfNodes;
    }

    @Column(name="NUMBER_OF_NODES",nullable=true)
    public void setNumberOfNodes(Integer numberOfNodes) {
        this.numberOfNodes = numberOfNodes;
    }

    @Column(name="NUMBER_OF_FINISHED_NODES",nullable=true)
    public Integer getNumberOfFinishedNodes() {
        return numberOfFinishedNodes;
    }

    @Column(name="NUMBER_OF_FINISHED_NODES",nullable=true)
    public void setNumberOfFinishedNodes(Integer numberOfFinishedNodes) {
        this.numberOfFinishedNodes = numberOfFinishedNodes;
    }

    @Column(name="STATUS",nullable=true)
    public Integer getStatus() {
        return status;
    }

    @Column(name="STATUS",nullable=true)
    public void setStatus(Integer status) {
        this.status = status;
    }

    @Column(name="CREATED",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public void setCreated(DateTime created) {
        this.created = (created != null) ? created.toDateTime(DateTimeZone.UTC) : null;
    }
    
    @Column(name = "CREATED", nullable = true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public DateTime getCreated() {
        return (created!=null) ? created.toDateTime(getDateTimeZone4Getters()) : null;
    }
    
    @Column(name="MODIFIED",nullable=true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public void setModified(DateTime modified) {
        this.modified = (modified != null) ? modified.toDateTime(DateTimeZone.UTC) : null;
    }
    
    @Column(name="CREATED_BY",nullable=true)
    public void setCreatedBy(String createdBy) {
        this.createdBy = createdBy;
    }
 
    @Column(name="CREATED_BY",nullable=true)
    public String getCreatedBy() {
        return createdBy;
    }
    
    @Column(name="MODIFIED_BY",nullable=true)
    public void setModifiedBy(String modifiedBy) {
        this.modifiedBy = modifiedBy;
    }
 
    @Column(name="MODIFIED_BY",nullable=true)
    public String getModifiedBy() {
        return modifiedBy;
    }

    @Column(name = "MODIFIED", nullable = true)
    @Type(type="org.joda.time.contrib.hibernate.PersistentDateTime")
    public DateTime getModified() {
        return (modified != null) ? modified.toDateTime(getDateTimeZone4Getters()) : null;
    }
    @Transient
    @Override
    public String getIdentifier() {
        return uuid;
    }
    
    @Transient
    @Override
    public String getTitle() {
        return uuid;
    }
     
    
    @Transient
    @Override    public boolean isJobNetHistoryNode(){
        return true;
    }       
    
    @Transient 
    public String getRunTimeFormatted(){
        return getDateFormatted(this.getRunTime().toDate());
    }

    @Transient 
    public String getStartTimeFormatted(){
        return getDateFormatted(this.getStartOfJobnet().toDate());
    }    
    
    @Transient 
    public String getEndTimeFormatted(){
        return getDateFormatted(this.getEndOfJobnet().toDate());
    }
    
    @Transient 
    public String getNetIdValue(){
        return String.valueOf(netId);
    }

    @Transient 
    public String getStatusValue(){
        return String.valueOf(status);
    }
    

    @Transient 
    public String getExitCodeValue(){
        return String.valueOf(exitCode);
    } 
    
    @Transient 
    @Override
    public boolean haveError(){
        return status==JobNetStatus.ERROR.getIndex();
    } 
 
    @Transient 
    @Override
    public JobNetNodeDBItem getJobNetNodeDBItem() {
        return jobNetDBItem.getJobNetNodeDBItem();
    }
    
    @Transient 
    public String getNumberOfFinishedNodesValue() {
       return String.valueOf(numberOfFinishedNodes);
   }
    @Transient 
    public String getNumberOfNodesValue() {
       return String.valueOf(numberOfNodes);
   }

}
