package com.sos.jobnet.dialog;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.prefs.Preferences;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.hibernate.Session;

import com.sos.dailyschedule.db.DailyScheduleDBItem;
import com.sos.hibernate.classes.DbItem;
import com.sos.hibernate.classes.SOSSearchFilterData;
import com.sos.hibernate.interfaces.ISOSDashboardDataProvider;
import com.sos.hibernate.interfaces.ISOSHibernateDataProvider;
import com.sos.jobnet.SchedulerJobNetTreeItem;
import com.sos.jobnet.classes.JobNetStatus;
import com.sos.jobnet.creator.JobNetConifigurationLayer;
import com.sos.jobnet.db.JobNetCmdHistoryDBLayer;
import com.sos.jobnet.db.JobNetDBItem;
import com.sos.jobnet.db.JobNetDBLayer;
import com.sos.jobnet.db.JobNetFilter;
import com.sos.jobnet.db.JobNetHistoryDBItem;
import com.sos.jobnet.db.JobNetHistoryDBLayer;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanTree;
import com.sos.jobnet.db.JobNetTreeDBItem;
import com.sos.jobnet.options.JobNetCreatorOptions;
import com.sos.scheduler.history.SchedulerOrderHistoryDataProvider;
import com.sos.scheduler.history.SchedulerTaskHistoryDataProvider;

/**
 * \class SchedulerJobNetDataProvider
 * 
 * \brief SchedulerJobNetDataProvider -
 * 
 * \details
 * 
 * 
 * \code 
 *      .... code goes here ... 
 * \endcode
 * 
 * <p style="text-align:center">
 * <br />
 * --------------------------------------------------------------------------- <br />
 * APL/Software GmbH - Berlin <br />
 * ##### generated by ClaviusXPress (http://www.sos-berlin.com) ######### <br />
 * ---------------------------------------------------------------------------
 * </p>
 * \author Uwe Risse \version 12.08.2013 \see reference
 * 
 * Created on 12.08.2013 09:31:01
 */
public class SchedulerJobNetDataProvider implements ISOSHibernateDataProvider, ISOSDashboardDataProvider {
    @SuppressWarnings("unused")
    private final String                     conClassName                  = "SchedulerJobNetDataProvider";
    private List<JobNetDBItem>               schedulerJobNetDBItems        = null;
    private List<JobNetHistoryDBItem>        schedulerJobNetHistoryDBItems = null;
    private JobNetDBLayer                    schedulerJobNetDBLayer        = null;
    private static JobNetConifigurationLayer jobNetConfigurationLayer;

    private JobNetHistoryDBLayer             schedulerJobNetHistoryDBLayer = null;
    private JobNetCmdHistoryDBLayer          jobNetCmdHistoryDBLayer;

    private static Logger                    logger                        = Logger.getLogger(SchedulerJobNetDataProvider.class);
    private Table                            tableSchedulerJobNet          = null;
    private static JobNetCreatorOptions      jobNetCreatorOptions          = null;
    private Tree tree;

    public SchedulerJobNetDataProvider(File configurationFile) {
        this.schedulerJobNetDBLayer = new JobNetDBLayer(configurationFile);
        jobNetCreatorOptions = new JobNetCreatorOptions();

        jobNetCreatorOptions.hibernate_connection_config_file.Value(configurationFile.getAbsolutePath());

        this.jobNetConfigurationLayer = new JobNetConifigurationLayer(jobNetCreatorOptions);
        this.schedulerJobNetHistoryDBLayer = new JobNetHistoryDBLayer(configurationFile);
        this.jobNetCmdHistoryDBLayer = new JobNetCmdHistoryDBLayer(configurationFile);
    }

    public JobNetFilter getFilter() {
        return schedulerJobNetDBLayer.getFilter();

    }

    public void resetFilter() {
        schedulerJobNetDBLayer.resetFilter();
    }

    public void getData(int limit) {
        schedulerJobNetDBItems = schedulerJobNetDBLayer.getJobNetList(limit);
    }

    public void fillSchedulerIds(CCombo cbSchedulerId) {
        if (schedulerJobNetDBItems != null) {
            Iterator<JobNetDBItem> schedulerJobNetEntries = schedulerJobNetDBItems.iterator();
            while (schedulerJobNetEntries.hasNext()) {
                JobNetDBItem h = (JobNetDBItem) schedulerJobNetEntries.next();
                if (cbSchedulerId.indexOf(h.getSchedulerId()) < 0) {
                    logger.debug("... cbSchedulerId --> : " + h.getSchedulerId());
                    cbSchedulerId.add(h.getSchedulerId());
                }
            }
        }
    }

    public String getLogAsString(Table tableDailySchedule, SchedulerTaskHistoryDataProvider schedulerTaskHistoryDataProvider, SchedulerOrderHistoryDataProvider schedulerOrderHistoryDataProvider) {
        String log = "";
        if (tableDailySchedule.getSelectionIndex() >= 0) {
            TableItem t = tableDailySchedule.getItem(tableDailySchedule.getSelectionIndex());
            DailyScheduleDBItem h = (DailyScheduleDBItem) t.getData();
            if (h.isStandalone()) {
                if (h.getSchedulerHistoryId() != null) {
                    log = schedulerTaskHistoryDataProvider.getLogAsString(h.getSchedulerHistoryId());
                }
            }
            else {
                if (h.getSchedulerOrderHistoryId() != null) {
                    log = schedulerOrderHistoryDataProvider.getLogAsString(h.getSchedulerOrderHistoryId());
                }
            }
        }
        return log;
    }

   

    private  void getAllItems(Tree tree,  HashMap<String, Boolean> allItems)  {
        int i=0;
        for(TreeItem item : tree.getItems())
        {
            getAllItems(i,item, allItems);
            i = i+1;
        }
    }
    
    private  void getAllItems(int nodeOnFirstLevel,TreeItem currentItem,  HashMap<String, Boolean> allItems)
    {
        TreeItem[] children = currentItem.getItems();
    
        for(int i = 0; i < children.length; i++)
        {
          
            String p = "";       
            if (children[i].getParent() != null) {
                p = children[i].getParent().getItem(nodeOnFirstLevel).getText();
            }
            allItems.put(p+"/"+children[i].getText(),children[i].getExpanded());
            getAllItems(nodeOnFirstLevel,children[i], allItems);
        }
    }
    
    private  void iterateAllItems(Tree tree, HashMap<String, Boolean> allTreeItems)   {
        int i=0;
        for(TreeItem item : tree.getItems())
        {
            iterateAllItems(i,item, allTreeItems);
            i = i+1;
        }
    }
    
    private  void iterateAllItems(int nodeOnFirstLevel, TreeItem currentItem, HashMap<String, Boolean> allTreeItems)
    {
        TreeItem[] children = currentItem.getItems();
        
        String p = "";       
        if (currentItem.getParent() != null) {
            p = currentItem.getParent().getItem(nodeOnFirstLevel).getText();
        }
        
        Boolean oldTreeItem = allTreeItems.get(p+"/"+currentItem.getText());
        if (oldTreeItem != null && oldTreeItem)   {
            currentItem.setExpanded(true);
        }
        
        for(int i = 0; i < children.length; i++)
        {
            iterateAllItems(nodeOnFirstLevel,children[i], allTreeItems);
        }
    }
    
    private  void iterateAllItems(Tree tree, JobNetTreeDBItem jobNetPlanDBItem)   {
        int i=0;
        for(TreeItem item : tree.getItems())
        {
            iterateAllItems(i,item, jobNetPlanDBItem);
            i = i+1;
        }
    }
    
    private  void iterateAllItems(int nodeOnFirstLevel, TreeItem currentItem, JobNetTreeDBItem jobNetPlanDBItem)
    {
        TreeItem[] children = currentItem.getItems();
       
        JobNetTreeDBItem n = (JobNetTreeDBItem)currentItem.getData();
        if (n.isJobNetPlanNode() && n.getIdentifier().equals(jobNetPlanDBItem.getIdentifier())){
            currentItem.setData(jobNetPlanDBItem);
        }
          
        for(int i = 0; i < children.length; i++)
        {
            iterateAllItems(nodeOnFirstLevel,children[i], jobNetPlanDBItem);
        }
    }
    
   
        
    private void reExpand(Tree tree, HashMap<String, Boolean> allTreeItems) {
        iterateAllItems(tree, allTreeItems);
    }

    public void fillTree(Tree tree_) {
        tree = tree_;
        HashMap<String, Boolean> allTreeItems = new HashMap<String, Boolean>();
        getAllItems(tree, allTreeItems);
   
        tree.removeAll();
        Iterator<JobNetDBItem> schedulerJobNetEntries = schedulerJobNetDBItems.iterator();
        while (schedulerJobNetEntries.hasNext()) {
            JobNetTreeDBItem h = schedulerJobNetEntries.next();
            if (schedulerJobNetDBLayer.getFilter().isFiltered(h)) {
            }
            else {

                final SchedulerJobNetTreeItem newTreeItem = new SchedulerJobNetTreeItem(tree, SWT.BORDER);
                newTreeItem.setDBItem(h);
                newTreeItem.setHistoryNode(false);
                newTreeItem.setJobNetNode(true);

                newTreeItem.setText(h.getTitle());
                addAll(newTreeItem);

               //setDefaultChilds(newTreeItem, h);

            }
        }
        reExpand(tree,allTreeItems);
 
    }

    public void expand(Tree tree, TreeItem item, boolean expanded) {
        tree.setRedraw(false);
        item.setExpanded(expanded);
        expandTreeItem(item, expanded);
        tree.setRedraw(true);
        tree.redraw();
    }
    
   
    private static void expandTreeItem(TreeItem item, boolean expanded){
        int c = item.getItemCount();
        TreeItem[] s = item.getItems();
        for(int i = 0; i<c; i++) {
           item.setExpanded(expanded);
           if(c > 0) {
            expandTreeItem(s[i],expanded);
           }
        } 
    }
    
    private void setDefaultChilds(TreeItem t, JobNetTreeDBItem h) {
        schedulerJobNetHistoryDBLayer.resetFilter();
        SchedulerJobNetTreeItem running = new SchedulerJobNetTreeItem(t, SWT.NONE);
        running.setDBItem(h);
        running.setText("Running");
        addRunning(running);

        SchedulerJobNetTreeItem waiting = new SchedulerJobNetTreeItem(t, SWT.NONE);
        waiting.setDBItem(h);
        waiting.setText("Waiting");
        addWaiting(waiting);

        SchedulerJobNetTreeItem history = new SchedulerJobNetTreeItem(t, SWT.NONE);
        history.setDBItem(h);
        history.setText("History");
        addHistory(history);

    }

    private void iterate(JobNetPlanTree tree, TreeItem treeItem, int level) {
        JobNetPlanDBItem node = tree.getJobNetPlanDBItem();

        final SchedulerJobNetTreeItem newTreeItem = new SchedulerJobNetTreeItem(treeItem, SWT.BORDER);
        newTreeItem.setDBItem(node);
        newTreeItem.setHistoryNode(true);
        newTreeItem.setJobNetNode(false);

        
        newTreeItem.setColor(tree.getStatusForDisplay());
        newTreeItem.setText(node.getTitle());

        for (JobNetPlanTree t : tree.getSuccessors()) {
            iterate(t, newTreeItem, (level + 1));
        }
    }

    private void buildNet(DbItem h, TreeItem treeItem) {

        JobNetHistoryDBItem jobNetHistoryDBItem = (JobNetHistoryDBItem) h;
        JobNetPlanDBItem jobnetPlanDBItem = jobNetConfigurationLayer.getDbLayerPlan().getByNodeIdOrNull(jobNetHistoryDBItem.getStartNodeId());

        JobNetPlanTree tree = jobNetConfigurationLayer.getDbLayerPlan().getMemberTreeForContext(jobnetPlanDBItem);
        iterate(tree, treeItem, 0);

    }

    public void actualizeItem(JobNetTreeDBItem jobNetPlanDBItem) {
          iterateAllItems(tree, jobNetPlanDBItem);

    }

    private void addItems(TreeItem t,List<JobNetHistoryDBItem> schedulerJobNetHistoryDBItems, boolean isJobNetHistoryNode ) {
         Iterator<JobNetHistoryDBItem> schedulerJobNetHistoryEntries = schedulerJobNetHistoryDBItems.iterator();
        while (schedulerJobNetHistoryEntries.hasNext()) {
            JobNetTreeDBItem h = schedulerJobNetHistoryEntries.next();
            if (schedulerJobNetHistoryDBLayer.getFilter().isFiltered(h)) {
            }
            else {
                final SchedulerJobNetTreeItem newTreeItem = new SchedulerJobNetTreeItem(t, SWT.BORDER);
                newTreeItem.setDBItem(h);
                newTreeItem.setHistoryNode(isJobNetHistoryNode);
                newTreeItem.setJobNetNode(false);

                newTreeItem.setText(h.getIdentifier());
                buildNet(h, newTreeItem);
            }
        }
    }
    
    
    private void addAll(TreeItem t) {
        schedulerJobNetHistoryDBLayer.resetFilter();
        schedulerJobNetHistoryDBItems = schedulerJobNetHistoryDBLayer.getSchedulerJobNetHistoryList((JobNetDBItem) t.getData(), 0);
        addItems(t,schedulerJobNetHistoryDBItems,false );
    }
    
    private void addRunning(TreeItem t) {
        schedulerJobNetHistoryDBLayer.getFilter().setStatus(JobNetStatus.RUNNING);
        schedulerJobNetHistoryDBItems = schedulerJobNetHistoryDBLayer.getSchedulerJobNetHistoryList((JobNetDBItem) t.getData(), 0);
        addItems(t,schedulerJobNetHistoryDBItems,false );
    }

    private void addWaiting(TreeItem t) {
        schedulerJobNetHistoryDBLayer.getFilter().setStatus(JobNetStatus.NOT_PROCESSED);
        schedulerJobNetHistoryDBItems = schedulerJobNetHistoryDBLayer.getSchedulerJobNetHistoryList((JobNetDBItem) t.getData(), 0);
        addItems(t,schedulerJobNetHistoryDBItems,false );
    }
    
    private void addHistory(TreeItem t) {
        schedulerJobNetHistoryDBLayer.getFilter().setStatus(null);
        schedulerJobNetHistoryDBLayer.getFilter().setFinishedStatus(true);
        schedulerJobNetHistoryDBItems = schedulerJobNetHistoryDBLayer.getSchedulerJobNetHistoryList((JobNetDBItem) t.getData(), 0);
        addItems(t,schedulerJobNetHistoryDBItems,true );
    }

    public void createJobNetCmdHistory(Long nodeId, Long historyId, String startMode, String message) {
        jobNetCmdHistoryDBLayer.createJobNetCmdHistory(nodeId, historyId, startMode, message);
    }
    

    public File getConfigurationFile() {
        return schedulerJobNetDBLayer.getConfigurationFile();
    }

    @Override
    public void beginTransaction() {
        schedulerJobNetDBLayer.beginTransaction();
    }

    public void save(DailyScheduleDBItem dbItem) {
        schedulerJobNetDBLayer.save(dbItem);
    }

    @Override
    public void update(DbItem dbItem) {
        schedulerJobNetDBLayer.update(dbItem);
    }

    public Session getSession() {
        return schedulerJobNetDBLayer.getSession();
    }

    @Override
    public void commit() {
        schedulerJobNetDBLayer.commit();
    }

    @Override
    public void setSchedulerId(String schedulerId) {
        this.getFilter().setSchedulerId(schedulerId);
    }

    @Override
    public void setFrom(Date d) {
    }

    @Override
    public void setTo(Date d) {
    }

    @Override
    public void setSearchField(SOSSearchFilterData s) {
    }

    @Override
    public void setShowJobs(boolean b) {
    }

    @Override
    public void setShowJobChains(boolean b) {
    }

    @Override
    public void setIgnoreList(Preferences prefs) {
    }

    @Override
    public void addToIgnorelist(Preferences prefs, DbItem h) {
    }

    @Override
    public void disableIgnoreList(Preferences prefs) {
    }

    @Override
    public void resetIgnoreList(Preferences prefs) {
    }

    @Override
    public void setLate(boolean b) {
    }

    @Override
    public void setStatus(String status) {
    }

    @Override
    public void setShowWithError(boolean b) {
    }

    @Override
    public void setShowRunning(boolean b) {
    }

    @Override
    public void fillTable(Table table) {
        // TODO Auto-generated method stub
        
    }

  
}
