package com.sos.jobnet.frontend;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.layout.FillLayout;

import com.sos.dashboard.globals.DashBoardConstants;
import com.sos.dialog.interfaces.ITableView;
import com.sos.hibernate.interfaces.ISOSDashboardDataProvider;
import com.sos.jobnet.dataprovider.JobNetPlanDataProvider;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnet.db.JobNetPlanFilter;
import com.sos.jobnet.db.JobNetTreeDBItem;
import com.sos.jobnet.options.JobNetCreatorOptions;
import com.sos.jobnet.options.StartModes;
import com.sos.scheduler.db.SchedulerInstancesDBItem;
import com.sos.scheduler.db.SchedulerInstancesDBLayer;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.commands.JSCmdAddOrder;
import com.sos.scheduler.model.objects.Params;
import com.sos.scheduler.model.objects.Spooler;

public class JobNetStartOrderDialog extends Dialog {

    protected Object               result;
    protected Shell                shell;
    private JobNetStartOrder       jobNetStartOrder;
    private JobNetCreatorOptions   jobNetCreatorOptions;
    private SchedulerInstancesDBItem schedulerInstanceDBItem;
    private String answer; 
    private JobnetShowTree caller;
    /**
     * 
     * Create the dialog.
     * @param parent
     * @param style
     */
    public JobNetStartOrderDialog(Shell parent, int style, JobNetCreatorOptions jobNetCreatorOptions_,SchedulerInstancesDBItem schedulerInstanceDBItem_ ) {
        super(parent, style);
        jobNetCreatorOptions = jobNetCreatorOptions_;
        schedulerInstanceDBItem= schedulerInstanceDBItem_;
     
    }

    /**
     * Open the dialog.
     * @return the result
     */
    public Object open(JobnetShowTree caller_) {
        createContents();
        caller=caller_;
        
        shell.open();
        shell.layout();
        Display display = getParent().getDisplay();
        while (!shell.isDisposed()) {
            if (!display.readAndDispatch()) {
                display.sleep();
            }
        }
        return result;
    }
    
    protected SchedulerInstancesDBItem start(JobNetCreatorOptions jobNetCreatorOptions) {
        
        SchedulerObjectFactory objSchedulerObjectFactory = new SchedulerObjectFactory(schedulerInstanceDBItem.getHostName(), schedulerInstanceDBItem.getTcpPort());
        objSchedulerObjectFactory.initMarshaller(Spooler.class);

        if (schedulerInstanceDBItem != null) {
            try {
               JSCmdAddOrder objOrder = objSchedulerObjectFactory.createAddOrder();
               Params params = objSchedulerObjectFactory.createParams();
               objOrder.setParams(params);

               objOrder.getParams().getParamOrCopyParamsOrInclude().add(objSchedulerObjectFactory.createParam(jobNetCreatorOptions.OrderId.getKey(), jobNetCreatorOptions.OrderId.Value()));
               objOrder.getParams().getParamOrCopyParamsOrInclude().add(objSchedulerObjectFactory.createParam(jobNetCreatorOptions.StartOption.getKey(), jobNetCreatorOptions.StartOption.Value()));
               objOrder.getParams().getParamOrCopyParamsOrInclude().add(objSchedulerObjectFactory.createParam(jobNetCreatorOptions.JobChainName.getKey(), jobNetCreatorOptions.JobChainName.Value()));
               
               objOrder.setJobChain(DashBoardConstants.conSOSJobnet_Jobnet_jobchain_name);
               objOrder.run();
               if (objOrder.getAnswer().getOk() != null){
                    answer = String.format("Command %s %s: %s", jobNetCreatorOptions.StartOption.Value(),jobNetCreatorOptions.OrderId.Value(),"OK");
                }else {
                    answer = String.format("Command %s %s: %s", jobNetCreatorOptions.OrderId.Value(),objOrder.getAnswer().getERROR());
               }
             }
            catch (Exception ee) {
                answer = String.format("Command %s %s: %s", jobNetCreatorOptions.OrderId.Value(),ee.getMessage());
                ee.printStackTrace();
            }
            finally {
                caller.setMsg(answer);
            }

        }
        return schedulerInstanceDBItem;

    }

    /**
     * Create contents of the dialog.
     */
    private void createContents() {
        shell = new Shell(getParent(), getStyle());
        shell.setSize(600, 150);
        shell.setText(getText());
        shell.setLayout(new FillLayout(SWT.HORIZONTAL));

        jobNetStartOrder = new JobNetStartOrder(shell, SWT.NONE,jobNetCreatorOptions);
        jobNetStartOrder.getBtnOk().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                start(jobNetCreatorOptions);
                shell.close();
            }
        });

        jobNetStartOrder.getBtnCancel().addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                shell.close();
            }
        });
    }

  
}
