package com.sos.jobnet.frontend;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Timer;
import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;

import org.eclipse.swt.widgets.Composite;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Logging.Log4JHelper;
import com.sos.dashboard.globals.DashBoardConstants;
import com.sos.dashboard.globals.SOSDashboardOptions;
import com.sos.dialog.BrowserViewForm;
import com.sos.jobnet.db.JobNetHistoryDBItem;
import com.sos.jobnet.db.JobNetTreeDBItem;
import com.sos.jobnet.dialog.classes.SOSJobNetImageViewer;
import com.sos.jobnet.graph.JobNetGraphBuilder;
import com.sos.jobnet.graph.JobNetGraphBuilderOptions;

/*
 * Created on 23.06.2008
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */

public class JobnetShowGraph extends JSToolBox {

    private static Logger        logger        = Logger.getLogger(JobnetShowGraph.class);

    @SuppressWarnings("unused")
    private static Log4JHelper   objLogger     = null;
    private final static String  conSVNVersion = "$Id: Editor.java 18748 2013-01-09 21:19:21Z kb $";
    protected Composite          composite     = null;
    private Timer                timer;
    private BrowserViewForm      browser;
    private SOSJobNetImageViewer sosJobNetImageViewer;
    private JobNetTreeDBItem     jobNetTreeDBItem;
    private boolean              isActive      = false;
    private boolean              changed       = false;
    SOSDashboardOptions          objOptions;

    public JobnetShowGraph(Composite composite_,SOSDashboardOptions objOptions_) throws Exception {
        super(DashBoardConstants.conPropertiesFileName);
        this.composite = composite_;
        this.objOptions = objOptions_;
        createContent();
        timer = new Timer();

    }

    private void createContent() {
        try {
            composite.layout();
            //sosJobNetImageViewer = new SOSJobNetImageViewer(composite);

            browser = new BrowserViewForm(composite, SWT.NONE, "about:blank");
        }
        catch (Exception e) {
            logger.error(String.format("could not create jobnet graph view: %s", e.getMessage()));
            e.printStackTrace();
        }
    }

    private void createGraph() throws JSExceptionMandatoryOptionMissing, Exception {
        try {
        if (isActive ) {
            JobNetGraphBuilderOptions options = new JobNetGraphBuilderOptions();
            options.setAllOptions(getOptions());
            options.CheckMandatory();
            JobNetGraphBuilder graph = new JobNetGraphBuilder(options);
            try {
            graph.run();
            }catch (Exception e) {e.printStackTrace();}
            //sosJobNetImageViewer.setImageFile(new File(graph.getTargetFile().getAbsolutePath()));
            if (graph.getTargetFile() != null) {
               browser.setUrl("file:///" + new File(graph.getTargetFile().getAbsolutePath()));
            }
            composite.layout();
            changed = false;

        }
        }catch (Exception e) {e.printStackTrace();};
    }

    private HashMap<String, String> getOptions() throws IOException {
        HashMap<String, String> parameters = new HashMap<String, String>();
        if (jobNetTreeDBItem != null) {

            File f = File.createTempFile("jobnet", "tmp");
            String targetDirName = f.getParent();
            f.delete();

            parameters.put("hibernate_connection_config_file", objOptions.hibernateConfigFile.Value());
            parameters.put("uuid_jobnet_identifier", jobNetTreeDBItem.getUuid());
            parameters.put("GraphType", "svg");
            parameters.put("GraphRatio", "fill");
            parameters.put("GraphSize", "16 9");
            if (jobNetTreeDBItem.getJobNetNodeDBItem() != null) {
                parameters.put("OrderId", jobNetTreeDBItem.getJobNetNodeDBItem().getNode());
            }

            parameters.put("GraphDir", targetDirName);

        }
         
        return parameters;
    }

    public Timer getTimer() {
        return timer;
    }

    public void resetRefreshTimer() {
        timer.cancel();
        timer = new Timer();
        isActive = true;
    }

    public void disableRefresh() {
        timer.cancel();
        isActive = false;
    }

    public void resetRefresh() {
        changed = true;
    }
    
    public void setJobNetHistoryDBItem(JobNetTreeDBItem jobNetHistoryDBItem_) throws JSExceptionMandatoryOptionMissing, Exception {
        
        changed = (jobNetHistoryDBItem_ == null || jobNetTreeDBItem == null || !jobNetTreeDBItem.getIdentifier().equals(jobNetHistoryDBItem_.getIdentifier()));
        this.jobNetTreeDBItem = jobNetHistoryDBItem_;
        if (changed) {
            createGraph();
        }
    }

    public void actualizeGraph() {
        if (changed) {
            try {
                createGraph();
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}