package com.sos.jobnet.graph;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.io.Files.JSFile;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.db.EventsDBLayer;
import com.sos.localization.Messages;

public class JobNetGraphBuilderJob implements Runnable {
	
	private final Logger logger = LoggerFactory.getLogger(JobNetGraphBuilderJob.class);
	private static final Messages msg = new JSToolBox(JobNetConstants.strBundleBaseName).getMessageObject();
	
	private final String uuid;
	private final JSFile hibernateConnectionConfigFile;
	private final EventsDBLayer eventsDBLayer;
	private final String eventIdJobnetStarted;
	private final JobNetGraphBuilder builder;

	private boolean jobnetStarted = false;

	public JobNetGraphBuilderJob(JobNetGraphBuilderOptions options) {
	    this.uuid = options.uuid_jobnet_identifier.Value();
		this.hibernateConnectionConfigFile 	= options.hibernate_connection_config_file.JSFile();
		this.eventsDBLayer = new EventsDBLayer(hibernateConnectionConfigFile);
		this.eventIdJobnetStarted = JobNetConstants.JOBNET_STARTED_PREFIX + uuid;
		this.builder = new JobNetGraphBuilder(options);

		String conMethodName = JobNetGraphBuilderJob.class.getSimpleName();
		logger.debug(msg.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "UUID", uuid));
		logger.debug(msg.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernateConnectionConfigFile", hibernateConnectionConfigFile.getAbsolutePath()));
	}

	@Override
	public void run() {

		// configure the job with a setback to wait until the jobnet is started
		jobnetStarted = eventsDBLayer.isEventThrown(eventIdJobnetStarted);
		if (jobnetStarted) {
            builder.run();			// creates the jobnet graph
            if(!builder.isJobnetPresentInDB())
                throw new JobSchedulerException("A jobnet with id " + uuid + " is not present in database.");
		} else {
			String msg = "Jobnet with id " + uuid + " is not started yet.";
			logger.warn(msg);
		}

	}

	public boolean isJobnetStarted() {
		return jobnetStarted;
	}
}
