package com.sos.jobnet.graph;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.SOSOptionFolderName;
import com.sos.JSHelper.Options.SOSOptionString;
import com.sos.JSHelper.Options.SOSOptionUUID;
import com.sos.hibernate.options.HibernateOptions;

import java.util.HashMap;

/**
 * JobNetGraphBuilderOptions - creates a graph for a given jobnet instance
 */
@JSOptionClass(name = "JobNetGraphBuilderOptions", description = "JobNetGraphBuilderOptions")
public class JobNetGraphBuilderOptions extends HibernateOptions {

	private static final long serialVersionUID = -7201337187084469930L;

	private final String conClassName = JobNetGraphBuilderOptions.class
			.getSimpleName();

	@SuppressWarnings("unused")
	private final String conSVNVersion = "$Id: JobNetCreatorOptionsSuperClass.java 16850 2012-03-21 12:14:23Z ss $";

	public JobNetGraphBuilderOptions() {
		objParentClass = this.getClass();
	}

	public JobNetGraphBuilderOptions(JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	}

	public JobNetGraphBuilderOptions(HashMap<String, String> JSSettings)
			throws Exception {
		this();
		this.setAllOptions(JSSettings);
	}


    // ---------------------------------------------------------------------------------------------------
	/**
	 * \var uuid_jobnet_identifier :
	 */
	@JSOptionDefinition(name = "uuid_jobnet_identifier", description = "", key = "uuid_jobnet_identifier", type = "SOSOptionUUID", mandatory = false)
	public SOSOptionUUID uuid_jobnet_identifier = new SOSOptionUUID(this,
			conClassName + ".uuid_jobnet_identifier", // HashMap-Key
			"", // Titel
			"", // InitValue
			"", // DefaultValue
			false // isMandatory
	);

	public void setuuid_jobnet_identifier(SOSOptionUUID p_uuid_jobnet_identifier) {
		this.uuid_jobnet_identifier = p_uuid_jobnet_identifier;
	}

    // ---------------------------------------------------------------------------------------------------
    /**
     * OrderId.
     */
    @JSOptionDefinition(name = "OrderId", description = "", key = "OrderId", type = "SOSOptionString", mandatory = true)
    public SOSOptionString OrderId = new SOSOptionString(this, conClassName
            + ".OrderId", // HashMap-Key
            "", // Titel
            " ", // InitValue
            " ", // DefaultValue
            true // isMandatory
    );

    public void setOrderId(SOSOptionString p_OrderId) {
        this.OrderId = p_OrderId;
    }

    // ---------------------------------------------------------------------------------------------------
	/**
	 * root folder of the job net graphs.
	 */
	@JSOptionDefinition(name = "GraphDir", description = "", key = "GraphDir", type = "SOSOptionFolderName", mandatory = false)
	public SOSOptionFolderName GraphDir = new SOSOptionFolderName(this,
			conClassName + ".GraphDir", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			true // isMandatory
	);

	public void setGraphDir(SOSOptionFolderName p_folderName) {
		this.GraphDir = p_folderName;
	}

    // ---------------------------------------------------------------------------------------------------
   /**
     * root folder of the job net graphs.
     */
    @JSOptionDefinition(name = "GraphType", description = "", key = "GraphType", type = "SOSOptionString", mandatory = false)
    public SOSOptionString GraphType = new SOSOptionString(this,
            conClassName + ".GraphType", // HashMap-Key
            "", // Titel
            "svg", // InitValue
            "svg", // DefaultValue
            false // isMandatory
    );

    public void setGraphType(SOSOptionString p_graphType) {
        this.GraphType = p_graphType;
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * size of the graph (corresponds with the size attribute of the dot language)
     */
    @JSOptionDefinition(name = "GraphSize", description = "", key = "GraphSize", type = "SOSOptionString", mandatory = false)
    public SOSOptionString GraphSize = new SOSOptionString(this,
            conClassName + ".GraphSize", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            false // isMandatory
    );

    public void setGraphSize(SOSOptionString p_graphSize) {
        this.GraphSize = p_graphSize;
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * ratio of the graph (corresponds with the ratio attribute of the dot language)
     */
    @JSOptionDefinition(name = "GraphRatio", description = "", key = "GraphRatio", type = "SOSOptionString", mandatory = false)
    public SOSOptionString GraphRatio = new SOSOptionString(this,
            conClassName + ".GraphRatio", // HashMap-Key
            "", // Titel
            "", // InitValue
            "", // DefaultValue
            false // isMandatory
    );

    public void setGraphRatio(SOSOptionString p_graphRatio) {
        this.GraphRatio = p_graphRatio;
    }

    // ---------------------------------------------------------------------------------------------------
    /**
     * the data folder of the JobScheduler installation
     */
    @JSOptionDefinition(name = "SCHEDULER_DIRECTORY", description = "", key = "SCHEDULER_DIRECTORY", type = "SOSOptionFolderName", mandatory = false)
    public SOSOptionFolderName SCHEDULER_DIRECTORY = new SOSOptionFolderName(
            this, conClassName + ".SCHEDULER_DIRECTORY", // HashMap-Key
            "", // Titel
            " ", // InitValue
            " ", // DefaultValue
            true // isMandatory
    );

    public void setSCHEDULER_DIRECTORY(SOSOptionFolderName dataFolder) {
        this.SCHEDULER_DIRECTORY = dataFolder;
    }

    // ---------------------------------------------------------------------------------------------------
	/**
	 * \brief setAllOptions - uebernimmt die OptionenWerte aus der HashMap
	 * 
	 * \see JSOptionsClass::getOrderRecord
	 * 
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	public void setAllOptions(HashMap<String, String> pobjJSSettings) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	/**
	 * \brief CheckMandatory - prueft alle Muss-Optionen auf Werte
	 * 
	 * \details
	 * 
	 * @throws Exception
	 * 
	 * @throws Exception
	 *             - wird ausgeloest, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing //
			, Exception {
		try {
			super.CheckMandatory();
		} catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 * 
	 * \brief CommandLineArgs - uebernehmen der Options/Settings aus der
	 * Kommandozeile
	 * 
	 * \details Die in der Kommandozeile beim Starten der Applikation
	 * angegebenen Parameter werden hier in die HashMap uebertragen und danach
	 * den Optionen als Wert zugewiesen.
	 * 
	 * \return void
	 * 
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class JobNetCreatorOptionsSuperClass