package com.sos.jobnet.jobs;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import sos.scheduler.job.JobSchedulerJobAdapter;
import sos.spooler.Order;
import sos.spooler.Variable_set;
import sos.spooler.Web_service_operation;
import sos.spooler.Web_service_response;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.jobnet.options.ListFilesWebServiceOptions;
// Super-Class for JobScheduler Java-API-Jobs
/**
 * \class 		ListFilesWebServiceJSAdapterClass - JobScheduler Adapter for "Returns folder content"
 *
 * \brief AdapterClass of ListFilesWebService for the SOSJobScheduler
 *
 * This Class ListFilesWebServiceJSAdapterClass works as an adapter-class between the SOS
 * JobScheduler and the worker-class ListFilesWebService.
 *

 *
 * see \see C:\Users\oh\AppData\Local\Temp\scheduler_editor-7579366387269665015.html for more details.
 *
 * \verbatim ;
 * mechanicaly created by C:\ProgramData\sos-berlin.com\jobscheduler\scheduler.oh\config\JOETemplates\java\xsl\JSJobDoc2JSAdapterClass.xsl from http://www.sos-berlin.com at 20130710012632
 * \endverbatim
 */
public class ListFilesWebServiceJSAdapterClass extends JobSchedulerJobAdapter  {
	private final String					conClassName						= "ListFilesWebServiceJSAdapterClass";
	private static Logger		logger			= LoggerFactory.getLogger(ListFilesWebServiceJSAdapterClass.class);
	protected ListFilesWebServiceOptions	objOptions			= null;

	public void init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init";
		doInitialize();
	}

	private void doInitialize() {
	} // doInitialize

	@Override
	public boolean spooler_init() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_init";
		return super.spooler_init();
	}

	@Override
	public boolean spooler_process() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_process";

		try {
			super.spooler_process();
			doProcessing();
		}
		catch (Exception e) {
            throw new JobSchedulerException("Fatal Error:" + e.getMessage(), e);
   		}
		finally {
		} // finally
        return signalSuccess();

	} // spooler_process

	@Override
	public void spooler_exit() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::spooler_exit";
		super.spooler_exit();
	}

	private void doProcessing() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::doProcessing";

		ListFilesWebServiceOptions objO = this.Options();

        objO.CurrentNodeName(this.getCurrentNodeName());
		objO.setAllOptions(getSchedulerParameterAsProperties(getJobOrOrderParameters()));
//		objO.CheckMandatory();
        this.setJSJobUtilites(this);
		this.Execute();
	} // doProcessing
	
	
	
	private void Execute() {
		Order order = spooler_task.order();
		Web_service_operation operation = null;
		String responseStr = "";
		String charset = "UTF-8";
		ByteArrayOutputStream byos = new ByteArrayOutputStream();
		Writer writer = null;
		
		try {
			operation = order.web_service_operation();
			Variable_set webServiceParams = operation.web_service().params();
			Options().source_dir.Value(webServiceParams.value("source_dir"));
			Options().base_url.Value(webServiceParams.value("base_url"));
		}
		catch (Exception e) {
			throw new JobSchedulerException("There is no Web Service operation attached to this order", e);
		}
		Options().CheckMandatory();
		String source_dir = Options().source_dir.Value();
		String base_url = Options().base_url.Value();
		
		File folder = new File(source_dir);
		if (folder.isDirectory() == false) {
			throw new JobSchedulerException(String.format("%1$s is not a directory", source_dir));
		}
		
		File[] files = folder.listFiles();
		
		responseStr = "<?xml version=\"1.0\" encoding=\""+charset+"\" ?>";
		responseStr += "<folder path=\""+source_dir+"\" url=\""+base_url+"\" length=\""+files.length+"\">";
		for(File file : files) {
			responseStr += "<file name=\""+file.getName()+"\" modifcation_date=\""+file.lastModified()+"\" size=\""+file.length()+"\"/>";
		}
		responseStr += "</folder>";
		
		try {
			writer = new OutputStreamWriter( byos, charset );
			writer.write( responseStr );
		}
		catch (UnsupportedEncodingException e) {
			throw new JobSchedulerException(e);
		}
		catch (IOException e) {
			throw new JobSchedulerException(e);
		}
		finally {
			if(writer != null) {
				try {
					writer.close();
				}
				catch (IOException e) {}
			}
		}
	    
		Web_service_response response = operation.response();
		
		response.set_content_type( "text/xml" );
	    response.set_charset_name( charset );
	    response.set_binary_content( byos.toByteArray() );
		
//		response.set_string_content(responseStr);
		response.send();
	}

	/**
	 * 
	 * \brief Options - returns the ListFilesWebServiceOptionClass
	 * 
	 * \details
	 * The ListFilesWebServiceOptionClass is used as a Container for all Options (Settings) which are
	 * needed.
	 *  
	 * \return ListFilesWebServiceOptions
	 *
	 */
	public ListFilesWebServiceOptions Options() {

		@SuppressWarnings("unused")  
		final String conMethodName = conClassName + "::Options";  //$NON-NLS-1$

		if (objOptions == null) {
			objOptions = new ListFilesWebServiceOptions();
		}
		return objOptions;
	}

}

