package com.sos.jobnet.jobs.dispatcher;

import com.sos.JSHelper.io.Files.JSFile;
import com.sos.i18n.I18NBase;
import com.sos.jobnet.classes.*;
import com.sos.jobnet.db.*;
import com.sos.jobnet.interfaces.IDispatchHandler;
import com.sos.jobnet.interfaces.IJobNetPlanEventListener;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.jobnet.options.StartModes;
import com.sos.jobnetmodel.objects.NodeStatus;
import com.sos.scheduler.model.objects.Params;
import org.joda.time.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;

/**
* \class JobNetDBDispatcher 
* 
* \brief JobNetDBDispatcher - 
* 
* \details
*
* \section JobNetDBDispatcher.java_intro_sec Introduction
*
* \section JobNetDBDispatcher.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetDBDispatcher.java 21059 2013-09-12 09:51:06Z ss $
* \see reference
*
* Created on 22.02.2012 13:56:28
 */
/**
 * @author oh
 *
 */
public class JobNetDBDispatcher  extends I18NBase  implements IDispatchHandler {

	private final String	conClassName	= "JobNetDBDispatcher";
	public  final String	conSVNVersion	= "$Id: JobNetDBDispatcher.java 21059 2013-09-12 09:51:06Z ss $";
	
 	private static final Logger	logger					= LoggerFactory.getLogger(JobNetDBDispatcher.class);
	private JobNetCollection 	dispatchNodes 		    = new JobNetCollection();
	private IJobNetPlanEventListener eventListener = null;
	
	private static final String	JOBNETDBW_E_0010		= "JOBNETDBW_E_0010";		// %s: No item found for the current node (%s) in database table SCHEDULER_JOB_NET_PLAN.
 	
	private final JobNetOptions options;
	private final File configurationFile;

	private final JobNetNodeDBLayer jobNetNodeDBLayer;
	private final JobNetPlanDBLayer jobNetPlanDBLayer;
    private final JobNetEdgesDBLayer jobNetEdgesDBLayer;
    private final EventsDBLayer eventsDBLayer;
    private final JobNetHistoryDBLayer historyDBLayer;
    private final ConditionsDBLayer	conditionsDBLayer;
    private final JobNetCmdHistoryDBLayer cmdHistoryDBLayer;
    private final JobNetCmdHistoryDBItem commandRecord;
    private final Boolean workWithEvents;
    private final StartModes startMode;

    private final String uuidJobnetIdentifier;
    private final String jobNet;
    private final String schedulerId;
    private final JSFile hibernateConnectionConfigFile;
    private final String currentNode;
    private final String currentNodeTitle;
	private final JobNetPlanDBItem	currentRecord;

	
	public JobNetDBDispatcher(JobNetOptions jobNetOptions) {
		super(JobNetConstants.strBundleBaseName);
		this.options = jobNetOptions;
		this.configurationFile 	= new File(options.hibernate_connection_config_file.Value());
	    this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(configurationFile, jobNetOptions.scheduler_id.Value());
        this.conditionsDBLayer = new ConditionsDBLayer(configurationFile);
		this.jobNetEdgesDBLayer = new JobNetEdgesDBLayer(configurationFile);
		this.jobNetNodeDBLayer = new JobNetNodeDBLayer(configurationFile, jobNetOptions.scheduler_id.Value());
		this.historyDBLayer = new JobNetHistoryDBLayer(configurationFile, jobNetOptions.scheduler_id.Value());
		this.cmdHistoryDBLayer = new JobNetCmdHistoryDBLayer(configurationFile, jobNetOptions.scheduler_id.Value());
		this.uuidJobnetIdentifier = options.uuid_jobnet_identifier.Value();
		this.jobNet = options.jobnet.Value();
		this.currentNode = options.node.Value();
		this.schedulerId = jobNetOptions.scheduler_id.Value();
		this.hibernateConnectionConfigFile = options.hibernate_connection_config_file.JSFile();
		this.currentNodeTitle = getNodeTitle();
		logOptions();

		this.currentRecord = readCurrentRecordFromDBAndLock();
		this.eventsDBLayer = new EventsDBLayer(hibernateConnectionConfigFile);

        this.commandRecord = readCmdHistory(uuidJobnetIdentifier);
        this.startMode = StartModes.valueOf(commandRecord.getStartOption());
        String qualifier = options.getItem(JobNetGlobalParams.JobNetIdQualifier.name()).trim();
        String conMethodName = this.getClass().getSimpleName();
        logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "StartOption", startMode.name()));
        logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "JobNetIdQualifier", qualifier));

        boolean isIndependent = (startMode == StartModes.start_independent || !qualifier.isEmpty());
        logger.info("The instance " + uuidJobnetIdentifier + " is an " + (isIndependent ? "in" : "") + "dependent one.");
        this.workWithEvents = !(isIndependent);	}

    private JobNetCmdHistoryDBItem readCmdHistory(String uuid) {
        JobNetCmdHistoryDBItem commandRecord = null;
        try {
            JobNetHistoryDBItem historyRecord = historyDBLayer.getJobNetInstance(uuid);
            commandRecord = cmdHistoryDBLayer.getLastJobNetCmdHistoryEntry(historyRecord.getHistoryId());
        } catch(Exception e) {
            throw new JobNetException("Error retrieving jobnet command entry for UUID " + uuid,e);
        }
        return commandRecord;
    }

	private void logOptions() {
		final String conMethodName = conClassName + "::logOptions";
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "uuid_jobnet_identifier", uuidJobnetIdentifier));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "jobnet", jobNet));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "node", currentNode));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "scheduler_id", schedulerId));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernate_connection_config_file", hibernateConnectionConfigFile.getAbsolutePath()));
	}
	
	/**
	 * \brief getCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem readCurrentRecordFromDBAndLock() {

		final String conMethodName = conClassName + "::setCurrentRecord";
		JobNetPlanDBItem result = null;
		JobNetPlanFilter jobNetPlanFilter = new JobNetPlanFilter();
		jobNetPlanFilter.setSchedulerId(schedulerId);
		jobNetPlanFilter.setUuid(uuidJobnetIdentifier);
		jobNetPlanFilter.setJobnet(jobNet);
		jobNetPlanFilter.setNode(currentNode);
		jobNetPlanDBLayer.setFilter(jobNetPlanFilter);
		
		List<JobNetPlanDBItem> jobNetPlanDBItem  = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
		if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
			String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, currentNodeTitle);
			logger.error(msg);
			throw new JobNetException(msg);
		}
		else {
			result = jobNetPlanDBItem.get(0);
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0010, conMethodName, result.getTitle()));
		}
		return result;
		
	}

	public JobNetPlanDBItem getCurrentRecord() {
		return currentRecord;
	}

    @Override
    public List<JobNetPlanDBItem> getChildNodes() {
        return jobNetPlanDBLayer.getChildNodes(getCurrentRecord());
    }

    private String getNodeTitle() {
		return uuidJobnetIdentifier + "/" + schedulerId + ":" + jobNet + ":" + currentNode;
	}
	
	/**
	 * \brief updateDispatcherStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateDispatcherStatus(NodeStatus status) {
		JobNetPlanDBItem record = getCurrentRecord();
		int oldState = record.getStatusDispatcher();
		record.setStatusDispatcher(status.getIndex());
		update();
		if (eventListener != null)
			eventListener.dispatcherStatusChanged(status, NodeStatus.valueOf(oldState));
 	}
	
	
	/**
	 * \brief updateRunnerStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateRunnerStatus(NodeStatus status) {
		JobNetPlanDBItem record = getCurrentRecord();
		int oldState = record.getStatusRunner();
 		record.setStatusRunner(status.getIndex());
		update();
		if (eventListener != null)
			eventListener.runnerStatusChanged(status, NodeStatus.valueOf(oldState));
 	} // private void updateRunnerStatus
	
	private void updateNodeStatus(NodeStatus status) {
		JobNetPlanDBItem rec = getCurrentRecord();
		int oldState = rec.getStatusNode();
		rec.setStatusNode(status.getIndex());
		update();
		nodeStatusChanged(rec, NodeStatus.valueOf(oldState) );
 	}

	/**
	 * Updates the status of a JobNetPlanDBItem to the given value.
	 * @param status
	 */
	private void updateNodeStatusAndLock(NodeStatus status) {
        jobNetPlanDBLayer.beginTransaction();
		JobNetPlanDBItem rec = jobNetPlanDBLayer.getNodeByNodeIdAndLock(getCurrentRecord().getNodeId());
		int oldState = rec.getStatusNode();
		rec.setStatusNode(status.getIndex());
        jobNetPlanDBLayer.saveOrUpdate(getCurrentRecord());
        jobNetPlanDBLayer.commit();
		nodeStatusChanged(rec, NodeStatus.valueOf(oldState) );
 	}


    // update the status in the node_record
	private void updateNodeStatus(JobNetPlanDBItem planDBRecord, NodeStatus status) {
		int oldState = planDBRecord.getStatusNode();
		planDBRecord.setStatusNode(status.getIndex());
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(planDBRecord);
		jobNetPlanDBLayer.commit();
		nodeStatusChanged(planDBRecord, NodeStatus.valueOf(oldState) );
 	}

	/**
	 * \brief update
	 * 
	 * \details
	 */
	private void update() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::update";
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(getCurrentRecord());
		jobNetPlanDBLayer.commit();
	} // private void update

    @Override
    public void update(JobNetPlanDBItem item) {
        jobNetPlanDBLayer.beginTransaction();
        jobNetPlanDBLayer.saveOrUpdate(item);
        jobNetPlanDBLayer.commit();
    }

    @Override
    public void updateParams(JobNetPlanDBItem item, Params params) {
        item.setParamXml(params.toXMLString());
        item.setModified(new DateTime());
        item.setModifiedBy( JobNetDAOConstants.CURRENT_USER );
        update(item);

    }

    /**
	 * \brief setDispatcherStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToRunning() {
		updateDispatcherStatus(NodeStatus.RUNNING);
	}

	/**
	 * \brief isDispatcherSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isDispatcherSkipped() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isDispatcherSkipped";
		return getCurrentRecord().getIsDispatcherSkipped();
	}

	/**
	 * \brief setRunnerStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setRunnerStatusToFinished() {
		updateRunnerStatus(NodeStatus.FINISHED);
	}

	/**
	 * \brief setDispatcherStatusToFinish
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToFinished() {
		updateDispatcherStatus(NodeStatus.FINISHED);
	}

	/**
	 * \brief setDispatcherStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToError() {
		updateDispatcherStatus(NodeStatus.ERROR);
	}

	@Override
	public void setRunnerStatusToError() {
		updateRunnerStatus(NodeStatus.ERROR);
		
	}

	@Override
	public boolean isRunnerSkipped() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isRunnerSkipped";
		return getCurrentRecord().getIsRunnerSkipped();	
	}

    @Override
    public boolean isRunnerInError() {
        NodeStatus state = NodeStatus.valueOf(getCurrentRecord().getStatusRunner());
        return state == NodeStatus.ERROR;
    }


    @Override
	public int getNumberOfDispatchingNodes() {
		return dispatchNodes.size();
	}

	@Override
	public JobNetCollection getDispatchingNodes() {

		final String conMethodName = conClassName + "::getDispatchforNodes";
		// getCurNode();

        logger.info("Collecting nodes to dispatch");
		List<JobNetEdgesDBItem> jobnetEdgesList = jobNetEdgesDBLayer.getSuccessors(getCurrentRecord().getJobnetNodeDBItem());

        jobNetPlanDBLayer.beginTransaction();
		for (JobNetEdgesDBItem jobnetEdgesItem : jobnetEdgesList) {

            Long successorNodeId = jobnetEdgesItem.getChildNodeId();

            /*
			JobNetNodeDBItem jobNetNodeDBItem = jobNetNodeDBLayer.getNode(jobnetEdgesItem.getChildNodeId());
			Long successorNodeId = jobNetNodeDBItem.getPlanId();

            /*
			JobNetPlanFilter jobNetPlanFilter = new JobNetPlanFilter();
			jobNetPlanDBLayer.setFilter(jobNetPlanFilter);

			jobNetPlanDBLayer.getFilter().setPlanId(successor);
			jobNetPlanFilter.setSchedulerId(schedulerId);
			jobNetPlanFilter.setUuid(uuidJobnetIdentifier);
			jobNetPlanFilter.setJobnet(jobNet);
			
			JobNetPlanDBItem jobnetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanList(0).get(0);
			*/

            JobNetPlanDBItem jobnetPlanDBItem = jobNetPlanDBLayer.getNodeByNodeIdAndLock(successorNodeId);
			NodeStatus status = NodeStatus.valueOf(jobnetPlanDBItem.getStatusNode());
			logger.info(String.format("The node %1s has the status %2s", jobnetPlanDBItem.getJobnetNodeDBItem().getNode(), status.name()));
            if (status.isNotProcessed() || status.isRestartPrepared()) {
                logger.info("Node " + jobnetPlanDBItem.getJobnetNodeDBItem().getNode() + " will be dispatched.");
                jobnetPlanDBItem.setStatusNode(NodeStatus.STARTED.getIndex());
				dispatchNodes.add(jobnetPlanDBItem);
                jobNetPlanDBLayer.update(jobnetPlanDBItem);
			}else{
                logger.info("Node " + jobnetPlanDBItem.getJobnetNodeDBItem().getNode() + " will NOT be dispatched - already running.");
			}
		}
        jobNetPlanDBLayer.commit();
		return dispatchNodes;
	}

    @Override
    public JobNetCollection getConnectorNodes() {
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }


    @Override
	public void open() {
		logger.debug(String.format("....Class: %s    conSVN: %s",conClassName, conSVNVersion));
	    logger.debug(String.format("....Scheduler / Cluster: %s",schedulerId));
		logger.debug(String.format("....Using hibernate configuration file: %s",hibernateConnectionConfigFile.getAbsoluteFile()));
	    logger.debug(String.format("....Dispatching node: %s in jobnet %s",currentNode,jobNet));
 	}

	@Override
	public void close() {
		logger.debug(String.format("\n..........Dispatching finished"));
	    this.jobNetPlanDBLayer.closeSession();
		this.jobNetEdgesDBLayer.closeSession();
		this.jobNetNodeDBLayer.closeSession();
	}

    /*
	@Override
	public void setNodeStatusToStarting(JobNetPlanDBItem planItem) {
		updateNodeStatus(planItem, enuStatus.STARTING);
	}
	*/

	@Override
	public void setNodeStatusToStarted(JobNetPlanDBItem planItem) {
		updateNodeStatus(planItem, NodeStatus.STARTED);
	}

    @Override
    public void setNodeStatusToFinished() {
        // getCurrentRecord().setIsWaiterSkipped(true);
        // getCurrentRecord().setIsRunnerSkipped(true);
        logger.info("The runner of node " + getCurrentRecord().getJobnetNodeDBItem().getNode() + " will be skipped next time, because it is finished.");
        JobNetPlanDBItem rec = getCurrentRecord();
        rec.setEndOfNode( new DateTime() );
        updateNodeStatus(NodeStatus.FINISHED);
    }

    @Override
    public void setNodeStatusToDispatching() {
        updateNodeStatus(NodeStatus.DISPATCHING);
        //updateNodeStatusAndLock(NodeStatus.DISPATCHING);
    }

    @Override
    public void setNodeStatusToError(long exitCode) {
        JobNetPlanDBItem rec = getCurrentRecord();
        rec.setExitCode(exitCode);
        updateNodeStatus(rec, NodeStatus.ERROR);
    }

    /*
    @Override
    public void setNodeStatusToError() {
        updateNodeStatus(enuStatus.ERROR);
    }
    */

	public void nodeStatusChanged(JobNetPlanDBItem node, NodeStatus oldState) {
		if(eventListener != null) {
			eventListener.nodeStatusChanged(node, oldState);
		}
	}

    // update the history record for the jobnetz
    @Override
    public void updateHistoryTimeStamp() {
        historyDBLayer.updateHistory(uuidJobnetIdentifier);
    }

    @Override
    public Boolean updateHistoryNodeEnds() {
        return historyDBLayer.updateHistoryAndIncreaseFinishedOrders( uuidJobnetIdentifier );
    }

    @Override
	public void setNodeStatusListener(IJobNetPlanEventListener listener) {
		this.eventListener = listener;
	}
	
	@Override
	public void createEvent(EventsDBItem record) {
        eventsDBLayer.beginTransaction();
        conditionsDBLayer.beginTransaction();
		eventsDBLayer.createEvent(record);
        conditionsDBLayer.setConditionFulfilled(record.getEventId());
        eventsDBLayer.commit();
    }

    @Override
    public JobNetCmdHistoryDBItem getCmdHistory() {
        return commandRecord;
    }

    @Override
    public Boolean workWithEvents() {
        return workWithEvents;
    }

    @Override
    public StartModes getStartMode() {
        return startMode;
    }

    @Override
    public void setPredecessorConditionFulfilled() {
        String conditionName = getCurrentRecord().getConditionName();
        logger.info("Try to set condition " + conditionName + " to fulfilled.");
        conditionsDBLayer.beginTransaction();
        conditionsDBLayer.setConditionFulfilled(getCurrentRecord().getConditionName());
        conditionsDBLayer.commit();
    }

}
