package com.sos.jobnet.jobs.waiter;

import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.jobnet.classes.*;
import com.sos.jobnet.db.ConditionsDBItem;
import com.sos.jobnet.db.ConditionsDBLayer;
import com.sos.jobnet.db.EventsDBItem;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.interfaces.IJobNetCallback;
import com.sos.jobnet.interfaces.IJobNetPlanEventListener;
import com.sos.jobnet.interfaces.IWaitHandler;
import com.sos.jobnet.jobs.dispatcher.JobNetDispatcher;
import com.sos.jobnet.logging.Level;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.jobnetmodel.objects.NodeStatus;
import com.sos.scheduler.model.SchedulerObjectFactory;
import com.sos.scheduler.model.objects.JSObjParams;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
* \class JobNetWaiter 
* 
* \brief JobNetWaiter - 
* 
* \details
*
* \section JobNetWaiter.java_intro_sec Introduction
*
* \section JobNetWaiter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetWaiter.java 21059 2013-09-12 09:51:06Z ss $
* \see reference
*
* Created on 24.02.2012 13:02:19
 */
/**
 * @author oh
 *
 */
@I18NResourceBundle(baseName = "com_sos_jobnet_messages", defaultLocale = "en")
public class JobNetWaiter extends I18NBase implements IJobNetCallback, IJobNetPlanEventListener {

	private final String		conClassName	= "JobNetWaiter";
	@SuppressWarnings("unused")
	private	final String		conSVNVersion	= "$Id: JobNetWaiter.java 21059 2013-09-12 09:51:06Z ss $";
	private static final Logger	logger			= LoggerFactory.getLogger(JobNetWaiter.class);
	private IWaitHandler		objWaitHandler	= null;
	private JobNetOptions		objOptions		= null;
    private final Integer taskId;
    private final String jobName;

	private static final String	JOBNETW_I_0001		= "JOBNETW_I_0001";			// %s: Check %d of %d
	private static final String	JOBNETW_I_0002		= "JOBNETW_I_0002";			// %s: Check %d
	// private static final String	JOBNETW_I_0003		= "JOBNETW_I_0003";			// %s: %s is called because the %s is skipped
	private static final String	JOBNETW_I_0005		= "JOBNETW_I_0005";			// %s: Max. checks reached
	private static final String	JOBNETW_T_0001		= "JOBNETW_T_0001";			// Check(%d|%d): %s
	private static final String	JOBNETW_T_0002		= "JOBNETW_T_0002";			// Start check for waiting nodes
	private static final String	JOBNETW_T_0003		= "JOBNETW_T_0003";			// DISPATCHER is started, RUNNER is skipped
	private static final String	JOBNETW_T_0004		= "JOBNETW_T_0004";			// RUNNER is started
	private static final String	JOBNETW_T_0005		= "JOBNETW_T_0005";			// RUNNER is 'On Demand'!
	private static final String	JOBNETW_T_0006		= "JOBNETW_T_0006";			// Waiting for
	
	private IJobNetCallback objJNCallback = this;

	public JobNetWaiter(Integer taskId, String jobName) {
		super(JobNetConstants.strBundleBaseName);
        this.taskId = taskId;
        this.jobName = jobName;
	}
	
	public void setIJobNetCallback(IJobNetCallback pobjJobNetCallback) {
		objJNCallback = pobjJobNetCallback;
	}
	
	public JobNetOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";
		if (objOptions == null) {
			objOptions = new JobNetOptions();
		}
		return objOptions;
	} // public JobNetOptions Options


	public void setOptions(JobNetOptions objOptions) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptions";
		this.objOptions = objOptions;
	} // public void setOptions

	public void setOptionsUsingHashMap(HashMap<String, String> objHshMap) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptionsUsingHashMap";
		try {
			Options().setAllOptions(objHshMap);
		}
		catch (Exception e) {
			throw new JobNetException(e);
		}
	} // public void setOptionsUsingHashMap

	public void doInit() {
		try {
			objWaitHandler = new JobNetDBWaiter(Options());
			objWaitHandler.setNodeStatusListener(this);
		} catch(Exception e) {
			throw new JobNetException("Error initializing JobNetWaiter",e);
		}
	}
	
	
	public boolean run() {
		final String conMethodName 	= conClassName + "::run";
		boolean ok 					= true;
		int intNoOfChecks 			= 1;
		int numberOfMissingConditions = 0;
		boolean bMaxChecksReached 	= false;
		boolean isRunnerOnDemand 	= false;
        JobNetCollection jobNetCollection = new JobNetCollection();
        List<EventsDBItem> fulfilledEvents = new ArrayList<EventsDBItem>();
        List<ConditionsDBItem> missingConditions = new ArrayList<ConditionsDBItem>();
		int intMaxNumberOfChecks 	= Options().max_number_of_checks.value();
		int intCheckInterval 		= Options().check_interval.getTimeAsSeconds();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "max_number_of_checks", intMaxNumberOfChecks));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "check_interval", intCheckInterval));
		
		try {
			objJNCallback.setStateText(Messages.getMsg(JOBNETW_T_0002));
			objWaitHandler.open();

            objWaitHandler.checkJobnetStart();          // check if its the first node of the jobnet
			if(objWaitHandler.isJobnetStart()) {
                objWaitHandler.startHistory();
				if (objWaitHandler.workWithEvents()) 
					objJNCallback.createJobNetStartedEvent(this);
				else
		            logger.info("Event to signal jobnet start not thrown, because jobnet instance is independent (StartOption is " + objWaitHandler.getStartMode().name() + ")");
			}

			if (objWaitHandler.isReadyForProcessing()) {

				objWaitHandler.setNodeStatusToWaiting(taskId,jobName);
				objWaitHandler.setWaiterStatusToRunning();
                objWaitHandler.activateConditions();
                objWaitHandler.updateHistory();

                if (objWaitHandler.isWaiterSkipped()) {
                    objWaitHandler.skipConditions();
                } else {
                    do {
                        logTrial(conMethodName, intNoOfChecks, intMaxNumberOfChecks);
                        fulfilledEvents = objWaitHandler.getFulfilledEvents();

                        objWaitHandler.setEventsFulfilled(fulfilledEvents);
                        missingConditions = objWaitHandler.getMissingConditions();
                        numberOfMissingConditions = missingConditions.size();

                        if (numberOfMissingConditions > 0) {
                            String stateText = getStateText(missingConditions, intNoOfChecks, intMaxNumberOfChecks);
                            objJNCallback.setStateText(stateText);
                            if (intMaxNumberOfChecks > 0) {
                                if (intMaxNumberOfChecks <= intNoOfChecks) {
                                    bMaxChecksReached = true;
                                    String msg = Messages.getMsg(JOBNETW_I_0005, conMethodName);
                                    objJNCallback.log(Level.INFO,msg);
                                    break;
                                }
                            }
                            objWaitHandler.close();

                            Thread.sleep(intCheckInterval*1000);
                            objWaitHandler.open();
                            // objWaitHandler.refresh();
                        }

                        String logText = getLogText(missingConditions, intNoOfChecks, intMaxNumberOfChecks);
                        logger.info(logText);
                        objJNCallback.log(Level.INFO,logText);
                        intNoOfChecks++;

                        String msg = missingConditions.size() + " conditions are still missing.";
                        logger.debug(msg);
                        objJNCallback.log(Level.DEBUG,msg);

                        objWaitHandler.readCurrentRecordAgain();

                    }
                    while (numberOfMissingConditions > 0);
                }

				if (bMaxChecksReached) {
					ok = false;
					objWaitHandler.setWaiterStatusToError();
					objWaitHandler.setNodeStatusToError();
                    //objWaitHandler.updateHistoryWithError(900, "Preconditions are not fullfilled after " + intMaxNumberOfChecks + " attempts.");
                    objWaitHandler.updateHistoryWithError(InternalJobNetError.PRECONDITIONS_MISSING, String.valueOf(intMaxNumberOfChecks));
					Thread.sleep(5*1000); //Damit der letzte StateText in JOC auch zu lesen ist
				}
				else {
					if (objWaitHandler.isRunnerSkipped()) {
						objWaitHandler.setWaiterStatusToFinish();
						objWaitHandler.setNodeStatusToRunning();
						objJNCallback.skipRunner();
						String msg = Messages.getMsg(JOBNETW_T_0003);
						objJNCallback.setStateText(msg);
						logger.info(msg);
					}
					else {
						objWaitHandler.setWaiterStatusToFinish();
						objWaitHandler.setRunnerStatusToRunning();
						objWaitHandler.setStartTime();
						objWaitHandler.setNodeStatusToRunning();
						String msg = Messages.getMsg(JOBNETW_T_0004);
						objJNCallback.setStateText(msg);
						logger.info(msg);
					}
                    objWaitHandler.updateHistory();
				}
			} else {
                logger.info("Current Status of the order is " + objWaitHandler.getCurrentNode().getStatusValue());
                String msg =
                        "Order is not in status " + NodeStatus.STARTED.name() +
                        " or in status " + NodeStatus.RESTART_PREPARED.name() +
                        " or in status " + NodeStatus.ERROR.name() +
                        " (or " + NodeStatus.NOT_PROCESSED.name() + " for bootstrap order) - Waiter will be skipped.";
                logger.error(msg);
                objJNCallback.log(Level.ERROR, msg);
                throw new JobNetException(msg);
			}
		}
		
		catch (Exception e) {
			ok = false;
			try {
				objWaitHandler.setWaiterStatusToError();
				objWaitHandler.setNodeStatusToError();
                // objWaitHandler.updateHistoryWithError(901,"Error in Waiter: " + e.getMessage());
                objWaitHandler.updateHistoryWithError(InternalJobNetError.WAITER_ERROR,e.getMessage());
			}
			catch (Exception e1) {
				String msg = "Error setting error NodeStatus database.";
				logger.error(msg,e);
				objJNCallback.log(Level.ERROR, msg, e);
				throw new JobNetException(msg,e);
			}
			String msg = "Error processing the waiter.";
			logger.error(msg,e);
			objJNCallback.log(Level.ERROR, msg, e);
			throw new JobNetException(msg,e);
		}
		
		finally {
			objJNCallback.setStateText("Preprocessing finished - order will be processed ...");
			objWaitHandler.close();
		}
		return ok;
	} // public JobNetWaiter run

	
	public void createEvent(String eventId) {
		String uuid = Options().uuid_jobnet_identifier.Value();
        if(objWaitHandler.workWithEvents() ) {
            EventsDBItem eventRecord = objJNCallback.createEventRecord(uuid, eventId);
            objWaitHandler.createEvent(eventRecord);
        } else {
            logger.info("Event " + eventId + " not thrown, because jobnet instance is independent (StartOption is " + objWaitHandler.getStartMode().name() + ")");
        }
	}

    /*
	private boolean isRunnerOnDemand() {
		return (!objWaitHandler.isRunnerSkipped()) ? objWaitHandler.isRunnerOnDemand() : false;
	}
	*/
	
	private void logTrial(String conMethodName, int intNoOfChecks, int intMaxNumberOfChecks) {
		String msg = (intMaxNumberOfChecks > 0) 
			? Messages.getMsg(JOBNETW_I_0001, conMethodName, intNoOfChecks, intMaxNumberOfChecks)
			: Messages.getMsg(JOBNETW_I_0002, conMethodName, intNoOfChecks);
		objJNCallback.log(Level.INFO, msg);
	}
	
	private String getStateText(List<ConditionsDBItem> missingConditions, int intNoOfChecks, int intMaxNumberOfChecks) {
		
		StringBuffer msg = ConditionsDBLayer.asStringBuffer(missingConditions);
		String trials = "";

        if (intMaxNumberOfChecks > -1) {
			trials = String.format("(%d|%d)", intNoOfChecks, intMaxNumberOfChecks);
		}
		else {
			trials = String.format(" %d", intNoOfChecks);
		}
		
		String message = Messages.getMsg(JOBNETW_T_0001, trials, msg.toString());
		return message;
	}

	private String getLogText(List<ConditionsDBItem> missingConditions, int intNoOfChecks, int intMaxNumberOfChecks) {

		StringBuffer message = new StringBuffer();
		message.append("Trial " + intNoOfChecks);
		if(intMaxNumberOfChecks > 0)
			message.append(" of " + ((intMaxNumberOfChecks>0) ? intMaxNumberOfChecks :"unlimited"));
		message.append(": ");
		
		if(missingConditions.size() == 0)
			message.append( "All conditions are fulfilled." );
		else {
			message.append( "Waiting for\n" );
			message.append(ConditionsDBLayer.asStringBuffer(missingConditions).toString());
		}

		return message.toString();
	}

	@Override
	public int getExitCode() {
		// TODO Auto-generated method stub
		return 0;
	}

    @Override
    public void setNextState() {
        //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
	public void log(Level level, String message, Throwable error) {
		Level.log(logger,level, message, error);	}

	@Override
	public void log(Level level, String message) {
		Level.log(logger,level, message);
	}

	@Override
	public void setStateText(String stateText) {
	}

	@Override
	public void startNode(JobNetPlanDBItem planItem, JobNetOptions options) {
	}

	@Override
	public void skipRunner() {
	}

	@Override
	public void logStatus(JobNetPlanDBItem record, NodeStatus oldStatus) {
        NodeStatus status = NodeStatus.valueOf(record.getStatusNode());
		logger.info("Node status of id " + record.getNodeId() +  " is set from " + oldStatus.name() + " to " + status.name());
	}

	@Override
	public void nodeStatusChanged(JobNetPlanDBItem node, NodeStatus oldState) {
		objJNCallback.logStatus(node, oldState);
	}

	@Override
	public void waiterStatusChanged(NodeStatus newState, NodeStatus oldState) {
		objJNCallback.logContextStatus("STATUS_WAITER", oldState, newState);
	}

	@Override
	public void runnerStatusChanged(NodeStatus newState, NodeStatus oldState) {
		objJNCallback.logContextStatus("STATUS_RUNNER", oldState, newState);
	}

	@Override
	public void dispatcherStatusChanged(NodeStatus newState, NodeStatus oldState) {
		// not implemented
	}

	@Override
	public void logContextStatus(String context, NodeStatus fromStatus, NodeStatus toStatus) {
		String msg = context + " changed from  " + fromStatus.name() +  " to " + toStatus.name();
		logger.info(msg);
	}

	@Override
	public EventsDBItem createEventRecord(String forEventClass,	String forEventId) {
		// has to be implemented in the callback class.
		return null;
	}

	@Override
	public void createJobNetStartedEvent(JobNetWaiter waiter) {
		// has to be implemented in the callback class.
	}

    @Override
    public JSObjParams getOrderParameterToProvide() {
        // has to be implemented in the callback class.
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
    public SchedulerObjectFactory getFactory() {
        // has to be implemented in the callback class.
        return null;  //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
	public void createJobNetEndedEvent(JobNetDispatcher dispatcher) {
		// has to be implemented in the callback class.
	}

}
