package com.sos.jobnet.logging;

import com.sos.tools.logback.MultiThreadProtocol;

import java.io.File;

/**
 * This class implements a jobnet protocol for the instance of a jobnet.
 *
 * To get a jobnet wide protocol it is necessary to use logback as logging api.
 *
 * A logger named <i>jobnet.protocol.all</i> is expected as in the sample configuration below:
 <pre><code>
 <appender name="SIFT-FILE" class="ch.qos.logback.classic.sift.SiftingAppender">
    <discriminator>
        <key>fileIndicator</key>
        <defaultValue>unknown</defaultValue>
    </discriminator>
    <sift>
        <appender name="FILE" class="ch.qos.logback.core.FileAppender">
            <file>${user.dir}/logs/${fileIndicator}.log</file>
            <append>true</append>
            <layout class="ch.qos.logback.classic.PatternLayout">
                <pattern>%d %level{5}: [%-35mdc{itemIndicator}] %msg%n</pattern>
            </layout>
        </appender>
    </sift>
 </appender>

 <logger name="jobnet.protocol.all" level="TRACE" additivity="false">
    <appender-ref ref="SIFT-FILE" />
 </logger>
 </pre></code>
 *
 * It is also possible to send the status protocol to database tables:
 <pre><code>
 &lt;appender name="SIFT-DB" class="ch.qos.logback.classic.sift.SiftingAppender"&gt;
    &lt;discriminator&gt;
        &lt;key&gt;fileIndicator&lt;/key&gt;
        &lt;defaultValue&gt;unknown&lt;/defaultValue&gt;
    &lt;/discriminator&gt;
    &lt;sift&gt;
        &lt;appender name="DB" class="com.sos.tools.logback.OracleDBAppender"&gt;
            &lt;connectionSource class="ch.qos.logback.core.db.DriverManagerConnectionSource"&gt;
                &lt;driverClass&gt;oracle.jdbc.driver.OracleDriver&lt;/driverClass&gt;
                &lt;url&gt;jdbc:oracle:thin:@localhost:1521:XE&lt;/url&gt;
                &lt;user&gt;scheduler&lt;/user&gt;
                &lt;password&gt;****&lt;/password&gt;
            &lt;/connectionSource&gt;
        &lt;/appender&gt;
    &lt;/sift&gt;
 &lt;/appender&gt;

 &lt;logger name="jobnet.protocol.status" level="TRACE" additivity="false"&gt;
    &lt;appender-ref ref="SIFT-DB" /&gt;
 &lt;/logger&gt;
 </pre></code>
 *
 * In this case you need to create the necessary table in the database first.
 *
 * The necessary scripts to do this for different databases are included in the logback-classic package
 * (see <a href="http://logback.qos.ch/download.html">http://logback.qos.ch/download.html</a>).
 */
public class JobNetProtocol extends MultiThreadProtocol {

    public final static String JOBNET_PROTOCOL_LOGGER      = "jobnet.protocol.all";

    public JobNetProtocol(String orderId, String uuid) {
        super(uuid, orderId, JOBNET_PROTOCOL_LOGGER);
    }

    public static String getProtocolAsText(File hibernateConfigurationFile, String forUUID) {
        return MultiThreadProtocol.getProtocolAsText(hibernateConfigurationFile, forUUID, JOBNET_PROTOCOL_LOGGER);
    }

    public static String getProtocolAsText(File hibernateConfigurationFile, String forUUID, String forOrder) {
        return MultiThreadProtocol.getProtocolAsText(hibernateConfigurationFile, forUUID, forOrder, JOBNET_PROTOCOL_LOGGER);
    }

}
