package com.sos.jobnet.options;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Annotations.JSOptionDefinition;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Options.*;
import com.sos.jobnetmodel.live.JobnetOrderOptions;

import java.util.HashMap;

/**
 * Options to control a jobnet order.
 *
 * Options defined in this class are jobnet wide parameters. They will provide to ech node of the JobNet. This class extends com.sos.jobnetmodel.live.JobnetOrderOptions class, which contains node specific parameters.
 *
 */
@JSOptionClass(name = "JobNetOptions", description = "JobNetOptions")
public class JobNetOptions  extends JobnetOrderOptions {

	private static final long	serialVersionUID	= 2628303960726088794L;
	private final String	conClassName	= "JobNetOptionsSuperClass";

    // --------------------------------------------------------------------------------------------------------------
    /**
     * Interval to check preconditions in the waiter in seconds.
     */
    @JSOptionDefinition(name = "check_interval", description = "", key = "check_interval", type = "SOSOptionTime", mandatory = false)
    
    public SOSOptionTime check_interval = new SOSOptionTime(this, conClassName + ".check_interval", // HashMap-Key
                                                                "", // Titel
                                                                "00:00:30", // InitValue
                                                                "00:00:30", // DefaultValue
                                                                false // isMandatory
                    );

    public void setcheck_interval (SOSOptionTime p_check_interval) {
        this.check_interval = p_check_interval;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * Delay_start_dispatcher :
     */
    @JSOptionDefinition(name = "delay_start_dispatcher", description = "", key = "delay_start_dispatcher", type = "SOSOptionTime", mandatory = false)
    
    public SOSOptionTime delay_start_dispatcher = new SOSOptionTime(this, conClassName + ".delay_start_dispatcher", // HashMap-Key
                                                                "", // Titel
                                                                "00:00:03", // InitValue
                                                                "00:00:03", // DefaultValue
                                                                false // isMandatory
                    );

    public void setdelay_start_dispatcher (SOSOptionTime p_delay_start_dispatcher) {
        this.delay_start_dispatcher = p_delay_start_dispatcher;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * Name of the job net
     */
    @JSOptionDefinition(name = "JobNet", description = "", key = "JobNet", type = "SOSOptionString", mandatory = false)
    
    public SOSOptionString jobnet = new SOSOptionString(this, conClassName + ".JobNet", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                true // isMandatory
                    );

    public void setjobnet (SOSOptionString p_jobnet) {
        this.jobnet = p_jobnet;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * Maximum number of checks.
     */
    @JSOptionDefinition(name = "max_number_of_checks", description = "", key = "max_number_of_checks", type = "SOSOptionInteger", mandatory = false)
    
    public SOSOptionInteger max_number_of_checks = new SOSOptionInteger(this, conClassName + ".max_number_of_checks", // HashMap-Key
                                                                "", // Titel
                                                                "-1", // InitValue
                                                                "-1", // DefaultValue
                                                                false // isMandatory
                    );

    public void setmax_number_of_checks (SOSOptionInteger p_max_number_of_checks) {
        this.max_number_of_checks = p_max_number_of_checks;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * Name of the job net node.
     */
    @JSOptionDefinition(name = "node", description = "", key = "node", type = "SOSOptionString", mandatory = false)

    public SOSOptionString node = new SOSOptionString(this, conClassName + ".node", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

    public void setnode (SOSOptionString p_node) {
        this.node = p_node;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * ID of the JobScheduler (Cluster)
     */
    @JSOptionDefinition(name = "scheduler_id", description = "", key = "scheduler_id", type = "SOSOptionSchedulerID", mandatory = false)
    
    public SOSOptionSchedulerID scheduler_id = new SOSOptionSchedulerID(this, conClassName + ".scheduler_id", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

    public void setscheduler_id (SOSOptionSchedulerID p_scheduler_id) {
        this.scheduler_id = p_scheduler_id;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * \var skipDispatcher :
     */
    @JSOptionDefinition(name = "skipDispatcher", description = "", key = "skipDispatcher", type = "SOSOptionBoolean", mandatory = false)
    
    public SOSOptionBoolean skipDispatcher = new SOSOptionBoolean(this, conClassName + ".skipDispatcher", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

    public void setskipDispatcher (SOSOptionBoolean p_skipDispatcher) {
        this.skipDispatcher = p_skipDispatcher;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * \var skipRunner :
     */
    @JSOptionDefinition(name = "skipRunner", description = "", key = "skipRunner", type = "SOSOptionBoolean", mandatory = false)
    
    public SOSOptionBoolean skipRunner = new SOSOptionBoolean(this, conClassName + ".skipRunner", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

    public void setskipRunner (SOSOptionBoolean p_skipRunner) {
        this.skipRunner = p_skipRunner;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * \var skipWaiter :
     */
    @JSOptionDefinition(name = "skipWaiter", description = "", key = "skipWaiter", type = "SOSOptionBoolean", mandatory = false)
    
    public SOSOptionBoolean skipWaiter = new SOSOptionBoolean(this, conClassName + ".skipWaiter", // HashMap-Key
                                                                "", // Titel
                                                                "false", // InitValue
                                                                "false", // DefaultValue
                                                                false // isMandatory
                    );

    public void setskipWaiter (SOSOptionBoolean p_skipWaiter) {
        this.skipWaiter = p_skipWaiter;
    }

    // --------------------------------------------------------------------------------------------------------------
    /**
     * The unique jobnet id.
     */
    @JSOptionDefinition(name = "uuid_jobnet_identifier", 
    description = "", 
    key = "uuid_jobnet_identifier", 
    type = "SOSOptionUUID", 
    mandatory = false)
    
    public SOSOptionUUID uuid_jobnet_identifier = new SOSOptionUUID(this, conClassName + ".uuid_jobnet_identifier", // HashMap-Key
                                                                "", // Titel
                                                                "", // InitValue
                                                                "", // DefaultValue
                                                                false // isMandatory
                    );

    public void setuuid_jobnet_identifier (SOSOptionUUID p_uuid_jobnet_identifier) {
        this.uuid_jobnet_identifier = p_uuid_jobnet_identifier;
    }


    // --------------------------------------------------------------------------------------------------------------
	/**
	 * A list of task comma separated list of task errors that will be ignored. Ranges of errors can be specified with
     * the pattern "from-to", e.g. 2-4 means the errors 2, 3 and 4 will be ignored.
	 */
	@JSOptionDefinition(name = "valid_errors", description = "", key = "valid_errors", type = "SOSOptionExitCodeList", mandatory = false)
	public SOSOptionExitCodeList valid_errors = new SOSOptionExitCodeList(this,
			conClassName + ".valid_errors", // HashMap-Key
			"", // Titel
			" ", // InitValue
			" ", // DefaultValue
			false // isMandatory
	);

	public void setvalid_errors(SOSOptionExitCodeList valid_errors) {
		this.valid_errors = valid_errors;
	}

    // --------------------------------------------------------------------------------------------------------------


    public JobNetOptions() {
		objParentClass = this.getClass();
	} // public JobNetOptionsSuperClass    
        
	//
	public JobNetOptions(HashMap<String, String> JSSettings) throws Exception {
		this();
		this.setAllOptions(JSSettings);
	} // public JobNetOptionsSuperClass (HashMap JSSettings)


	/**
	 * \brief setAllOptions - uebernimmt die OptionenWerte aus der HashMap
	 *
	 * \details In der als Parameter anzugebenden HashMap sind Schluessel (Name)
	 * und Wert der jeweiligen Option als Paar angegeben. Ein Beispiel fuer den
	 * Aufbau einer solchen HashMap findet sich in der Beschreibung dieser
	 * Klasse (\ref TestData "setJobSchedulerSSHJobOptions"). In dieser Routine
	 * werden die Schluessel analysiert und, falls gefunden, werden die
	 * dazugehoerigen Werte den Properties dieser Klasse zugewiesen.
	 *
	 * Nicht bekannte Schluessel werden ignoriert.
	 *
	 * \see JSOptionsClass::getOrderRecord
	 *
	 * @param pobjJSSettings
	 * @throws Exception
	 */
	public void setAllOptions(HashMap<String, String> pobjJSSettings) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setAllOptions";
		flgSetAllOptions = true;
		objSettings = pobjJSSettings;
		super.Settings(objSettings);
		super.setAllOptions(pobjJSSettings);
		flgSetAllOptions = false;
	} // public void setAllOptions (HashMap <String, String> JSSettings)

	/**
	 * \brief CheckMandatory - prueft alle Muss-Optionen auf Werte
	 *
	 * \details
	 * @throws Exception
	 *
	 * @throws Exception
	 * - wird ausgeloest, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	public void CheckMandatory() throws JSExceptionMandatoryOptionMissing {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 *
	 * \brief CommandLineArgs - uebernehmen der Options/Settings aus der Kommandozeile
	 *
	 * \details Die in der Kommandozeile beim Starten der Applikation
	 * angegebenen Parameter werden hier in die HashMap uebertragen und danach
	 * den Optionen als Wert zugewiesen.
	 *
	 * \return void
	 *
	 * @param pstrArgs
	 * @throws Exception
	 */
	@Override
	public void CommandLineArgs(String[] pstrArgs) {
		super.CommandLineArgs(pstrArgs);
		this.setAllOptions(super.objSettings);
	}
} // public class JobNetOptionsSuperClass