package com.sos.jobnet.utils;

import org.joda.time.DateTime;
import org.joda.time.DurationFieldType;
import org.joda.time.MutableDateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * This class is for simplified calculation of time keys.
 */
public class TimeKeyCalculator {

    public final static DateTimeFormatter FORMAT_YYYYMMDDHHMMSS = DateTimeFormat.forPattern("yyyyMMddHHmmss");
    public final static DateTimeFormatter FORMAT_YYYYMMDD = DateTimeFormat.forPattern("yyyyMMdd");
    public final static DateTimeFormatter FORMAT_YYYYMM = DateTimeFormat.forPattern("yyyyMM");
    public final static DurationFieldType TYPE_DAYS = DurationFieldType.days();
    public final static DurationFieldType TYPE_MONTHS = DurationFieldType.months();

    public final static Pattern FORMULA_ADD_MONTH = Pattern.compile("add_months\\s*\\(\\s*(\\d*)\\s*\\)");
    public final static Pattern FORMULA_SUB_MONTH = Pattern.compile("subtract_months\\s*\\(\\s*(\\d*)\\s*\\)");
    public final static Pattern FORMULA_ADD_MONTH_ABSOLUTE = Pattern.compile("add_months_absolute\\s*\\(\\s*(\\d*)\\s*\\)");
    public final static Pattern FORMULA_SUB_MONTH_ABSOLUTE = Pattern.compile("subtract_months_absolute\\s*\\(\\s*(\\d*)\\s*\\)");

    private final String timeKey;
    private final DateTimeFormatter format;
    private final DurationFieldType type;
    private final DateTime timeValue;

    private MutableDateTime calculatedTimeValue;

    public TimeKeyCalculator(String timeKey, DateTimeFormatter format, DurationFieldType type) {
        this.timeKey = timeKey;
        this.format = format;
        this.type = type;
        this.timeValue = format.parseDateTime(timeKey);
        this.calculatedTimeValue = timeValue.toMutableDateTime();
    }

    public TimeKeyCalculator(DateTime timeValue, DateTimeFormatter format, DurationFieldType type) {
        this.timeValue = timeValue;
        this.format = format;
        this.type = type;
        this.timeKey = timeValue.toString(format);
        this.calculatedTimeValue = timeValue.toMutableDateTime();
    }

    public String add(int operand) {
        calculatedTimeValue.add(type,operand);
        return format.print(calculatedTimeValue);
    }

    public String subtract(int operand) {
        return add(operand * -1);
    }

    public String increase() {
        return add(1);
    }

    public String decrease() {
        return subtract(1);
    }

    public String calculate(String calc) {
    	String result = timeKey;
    	if(!calc.isEmpty()) {
	    	String[] parts = calc.split("(?=[\\+|\\-])");
	    	for(String part : parts) {
	    		while (true) {
		    		Matcher m = FORMULA_ADD_MONTH.matcher(part);
		    		if(m.find()) {
			    		int operand = Integer.valueOf(m.group(1));
                        calculatedTimeValue.setDayOfMonth(1);
		    			calculatedTimeValue.addMonths(operand);
		    			break;
		    		}
		    		m = FORMULA_ADD_MONTH_ABSOLUTE.matcher(part);
		    		if(m.find()) {
			    		int operand = Integer.valueOf(m.group(1));
		    			calculatedTimeValue.addMonths(operand);
		    			break;
		    		}
		    		m = FORMULA_SUB_MONTH.matcher(part);
		    		if(m.find()) {
			    		int operand = Integer.valueOf(m.group(1)) * -1;
                        calculatedTimeValue.setDayOfMonth(1);
		    			calculatedTimeValue.addMonths(operand);
		    			break;
		    		}
		    		m = FORMULA_SUB_MONTH_ABSOLUTE.matcher(part);
		    		if(m.find()) {
			    		int operand = Integer.valueOf(m.group(1)) * -1;
		    			calculatedTimeValue.addMonths(operand);
		    			break;
		    		}
		    		// a positive sign will not be parsed by OpenJDK, therefore it has to be replaced
		    		int operand = Integer.valueOf(part.replaceAll("\\s","").replace("+", ""));
		    		calculatedTimeValue.add(type,operand);
		    		break;
	    		}
	    	}
	    	result = format.print(calculatedTimeValue);
    	}
    	return result;
    }

    public String convert(DateTimeFormatter toFormat) {
    	return toFormat.print(calculatedTimeValue);
    }

}
