/* DB2 */

CREATE PROCEDURE TMP2() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF EXISTS (SELECT * FROM SYSCAT.PROCEDURES WHERE "PROCSCHEMA"=CURRENT_SCHEMA AND "PROCNAME"='TMP') THEN SET SQL='
DROP PROCEDURE TMP
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP2
@
DROP PROCEDURE TMP2
@
COMMIT
@


CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET_PLAN') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET_PLAN
(
   "PLAN_ID"                 NUMERIC(10)   GENERATED AS IDENTITY, /* Id */
   "UUID"                    VARCHAR(42)                NOT NULL, /* Unique Id */
   "NODE_ID"                 NUMERIC(10)                NOT NULL, /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "SUBNET_ID"               NUMERIC(10)      DEFAULT 0 NOT NULL, /* Reference e.g. for the history */
   "CONNECTOR_ID"            NUMERIC(10)      DEFAULT 0 NOT NULL, /* Reference e.g. for the history */
   "TASK_ID"                 NUMERIC(10)      DEFAULT 0 NOT NULL, /* Reference e.g. for the history */
   "NODE_TYPE"               VARCHAR(10)                ,         /* Type of jobnet node: HEAD, CONNECTOR, JOB */
   "EXIT_CODE"               NUMERIC(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"            VARCHAR(255)               ,        
   "STATUS"                  NUMERIC(2)                 ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_WAITER"           NUMERIC(2)                 ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_RUNNER"           NUMERIC(2)                 ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_DISPATCHER"       NUMERIC(2)                 ,         /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "IS_WAITER_SKIPPED"       NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "IS_RUNNER_SKIPPED"       NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "IS_DISPATCHER_SKIPPED"   NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "IS_RUNNER_ON_DEMAND"     NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "PLANNED_START_TIME"      TIMESTAMP                  ,         /* */
   "START_TIME"              TIMESTAMP                  ,         /* */
   "END_TIME"                TIMESTAMP                  ,         /* */
   "ORDER_XML"               CLOB                       ,         /* xml definition of the order */
   "BOOTSTRAP"               NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "FREQUENCE_OVERWRITE"     NUMERIC(1)       DEFAULT 0 NOT NULL, /* */
   "MESSAGE"                 VARCHAR(255)               ,         /* */
   "CREATED"                 TIMESTAMP                  ,         /* */
   "CREATED_BY"              VARCHAR(255)               ,         /* */
   "MODIFIED"                TIMESTAMP                  ,         /* */
   "MODIFIED_BY"             VARCHAR(255)               ,         /* */
   CONSTRAINT SCHEDULER_JOB_NET_PLAN_UNIQUE UNIQUE( "UUID", "NODE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
)
'; EXECUTE IMMEDIATE SQL; SET SQL='
CREATE INDEX JOBNET_UUID          ON SCHEDULER_JOB_NET_PLAN ("UUID")
'; EXECUTE IMMEDIATE SQL; SET SQL='
CREATE INDEX JOBNET_NODE_TYPE     ON SCHEDULER_JOB_NET_PLAN ("NODE_TYPE")
'; EXECUTE IMMEDIATE SQL; SET SQL='
CREATE INDEX JOBNET_CONNECTOR_ID  ON SCHEDULER_JOB_NET_PLAN ("CONNECTOR_ID")
'; EXECUTE IMMEDIATE SQL; SET SQL='
CREATE INDEX JOBNET_SUBNET_ID     ON SCHEDULER_JOB_NET_PLAN ("SUBNET_ID")
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@



CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET_NODES') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET_NODES
(
   "NODE_ID"                NUMERIC(10)   GENERATED AS IDENTITY,  /* Id */
   "SCHEDULER_ID"           VARCHAR(255)               NOT NULL,  /* JobScheduler Id */
   "NODE"                   VARCHAR(255)               NOT NULL,  /* Name of the job net node */
   "JOB_NET"                VARCHAR(255)               NOT NULL,  /* Name of the job net */
   CONSTRAINT SCHEDULER_JOB_NET_NODES_UNIQUE UNIQUE( "SCHEDULER_ID", "NODE", "JOB_NET" ),
   PRIMARY KEY ( "NODE_ID" )
)
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@


CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET_EDGES') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET_EDGES
(
   "EDGE_ID"                NUMERIC(10)   GENERATED AS IDENTITY,  /* Id */
   "EDGE_TYPE"              CHAR(1)                    NOT NULL,  /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"         NUMERIC(10)                NOT NULL,  /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"          NUMERIC(10)                NOT NULL,  /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHEDULER_JOB_NET_EDGES_UNIQUE UNIQUE( "EDGE_TYPE", "PARENT_NODE_ID", "CHILD_NODE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
)
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@ 


CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET_PLAN_EDGES') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET_PLAN_EDGES
(
   "PLAN_ID"                NUMERIC(10)   GENERATED AS IDENTITY,  /* Id */
   "UUID"                   VARCHAR(42)                NOT NULL,  /* Unique Id */
   "EDGE_ID"                NUMERIC(10)                NOT NULL,  /* Id */
   "EDGE_TYPE"              CHAR(1)                    NOT NULL,  /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"         NUMERIC(10)                NOT NULL,  /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"          NUMERIC(10)                NOT NULL,  /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHED_JOBNET_PLAN_EDGES_UNIQUE UNIQUE( "UUID", "EDGE_ID"),
   PRIMARY KEY ( "PLAN_ID" )
)
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@


CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET_HISTORY') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET_HISTORY
(
   "HISTORY_ID"                NUMERIC(10)   GENERATED AS IDENTITY,         /* Id */
   "SCHEDULER_ID"              VARCHAR(255)               NOT NULL,         /* JobScheduler Id */
   "NET_ID"                    NUMERIC(10)                NOT NULL,         /* SCHEDULER_JOB_NET.NET_ID */
   "UUID"                      VARCHAR(42)                NOT NULL,         /* Unique Id */
   "START_TIME"                TIMESTAMP                          ,         /* */
   "END_TIME"                  TIMESTAMP                          ,         /* */
   "RUN_TIME"                  TIMESTAMP                          ,         /* */
   "STATUS"                    NUMERIC(10)                        ,         /* */
   "NUMBER_OF_NODES"           NUMERIC(10)      DEFAULT 0 NOT NULL,         /* */ 
   "NUMBER_OF_FINISHED_NODES"  NUMERIC(10)      DEFAULT 0 NOT NULL,         /* */
   "EXIT_CODE"                 NUMERIC(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"              VARCHAR(255)                       ,        
   "JOBNET_ID"                 VARCHAR(255)                       ,         /* external (custom) Jobnet id, e.g. Timekey. Need not be unique in the history */
   "JOBNET_ID_PARAMETER_NAME"  VARCHAR(255)                       ,        
   PRIMARY KEY ( "HISTORY_ID" )
)
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@


CREATE PROCEDURE TMP() LANGUAGE SQL BEGIN DECLARE SQL VARCHAR(4000); IF NOT EXISTS (SELECT * FROM SYSCAT.TABLES WHERE "TABSCHEMA"=CURRENT_SCHEMA AND "TABNAME"='SCHEDULER_JOB_NET') THEN SET SQL='
CREATE TABLE SCHEDULER_JOB_NET
(
   "NET_ID"                    NUMERIC(10)   GENERATED AS IDENTITY,         /* Id */
   "SCHEDULER_ID"              VARCHAR(255)               NOT NULL,         /* JobScheduler Id */
   "JOB_NET"                   VARCHAR(255)               NOT NULL,         /* */
   PRIMARY KEY ( "NET_ID" )
)
'; EXECUTE IMMEDIATE SQL; END IF; END
@
CALL TMP
@
DROP PROCEDURE TMP
@
COMMIT
@
