/* PostgreSQL */


CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET_PLAN'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET_PLAN
(
   "PLAN_ID"                 SERIAL                     ,                /* Id */
   "UUID"                    VARCHAR(42)        NOT NULL,                /* Unique Id */
   "NODE_ID"                 NUMERIC(10)        NOT NULL,                /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "SUBNET_ID"               NUMERIC(10)        DEFAULT 0 NOT NULL,      /* */
   "CONNECTOR_ID"            NUMERIC(10)        DEFAULT 0 NOT NULL,      /* */
   "TASK_ID"                 NUMERIC(10)        DEFAULT 0 NOT NULL,      /* Reference e.g. for the history */
   "NODE_TYPE"               VARCHAR(10)                ,                /* Type of jobnet node: HEAD, CONNECTOR, JOB */
   "EXIT_CODE"               NUMERIC(10)        DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"            VARCHAR(255)               ,        
   "STATUS"                  NUMERIC(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_WAITER"           NUMERIC(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_RUNNER"           NUMERIC(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_DISPATCHER"       NUMERIC(2)                 ,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "IS_WAITER_SKIPPED"       BOOLEAN,      								 /* */
   "IS_RUNNER_SKIPPED"       BOOLEAN,      								 /* */
   "IS_DISPATCHER_SKIPPED"   BOOLEAN,     								 /* */
   "IS_RUNNER_ON_DEMAND"     BOOLEAN,     								 /* */
   "PLANNED_START_TIME"      TIMESTAMP                  ,                /* */
   "START_TIME"              TIMESTAMP                  ,                /* */
   "END_TIME"                TIMESTAMP                  ,                /* */
   "ORDER_XML"               TEXT                       ,                /* xml definition of the order */
   "BOOTSTRAP"               BOOLEAN,     								 /* */
   "FREQUENCE_OVERWRITE"     BOOLEAN,     								 /* */
   "MESSAGE"                 VARCHAR(255)               ,                /* */
   "CREATED"                 TIMESTAMP                  ,                /* */
   "CREATED_BY"              VARCHAR(255)               ,                /* */
   "MODIFIED"                TIMESTAMP                  ,                /* */
   "MODIFIED_BY"             VARCHAR(255)               ,                /* */
   CONSTRAINT SCHEDULER_JOB_NET_PLAN_UNIQUE UNIQUE( "UUID", "NODE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
)
'; EXECUTE '
CREATE INDEX JOBNET_UUID         ON SCHEDULER_JOB_NET_PLAN ("UUID")
'; EXECUTE '
CREATE INDEX JOBNET_NODE_TYPE    ON SCHEDULER_JOB_NET_PLAN ("NODE_TYPE")
'; EXECUTE '
CREATE INDEX JOBNET_CONNECTOR_ID ON SCHEDULER_JOB_NET_PLAN ("CONNECTOR_ID")
'; EXECUTE '
CREATE INDEX JOBNET_SUBNET_ID    ON SCHEDULER_JOB_NET_PLAN ("SUBNET_ID")
';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;


CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET_NODES'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET_NODES
(
   "NODE_ID"             SERIAL                  ,                   /* Id */
   "SCHEDULER_ID"        VARCHAR(255)    NOT NULL,                   /* JobScheduler Id */
   "NODE"                VARCHAR(255)    NOT NULL,                   /* Name of the job net node */
   "JOB_NET"             VARCHAR(255)    NOT NULL,                   /* Name of the job net */
   CONSTRAINT SCHEDULER_JOB_NET_NODES_UNIQUE UNIQUE( "SCHEDULER_ID", "NODE", "JOB_NET" ),
   PRIMARY KEY ( "NODE_ID" )
)';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;


CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET_EDGES'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET_EDGES
(
   "EDGE_ID"             SERIAL                      ,                 /* Id */
   "EDGE_TYPE"           CHAR(1)             NOT NULL,                 /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHEDULER_JOB_NET_EDGES_UNIQUE UNIQUE( "EDGE_TYPE", "PARENT_NODE_ID", "CHILD_NODE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
)';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;


CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET_PLAN_EDGES'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET_PLAN_EDGES
(
   "PLAN_ID"             SERIAL                      ,                 /* Id */
   "UUID"                VARCHAR(42)         NOT NULL,                 /* Unique Id */
   "EDGE_ID"             NUMERIC(10)         NOT NULL,                 /* Id */
   "EDGE_TYPE"           CHAR(1)             NOT NULL,                 /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHED_JOBNET_PLAN_EDGES_UNIQUE UNIQUE( "UUID", "EDGE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
)';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;
         
         
CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET_HISTORY'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET_HISTORY
(
   "HISTORY_ID"                SERIAL                   ,           /* Id */
   "SCHEDULER_ID"              VARCHAR(255)     NOT NULL,           /* JobScheduler Id */
   "NET_ID"                    NUMERIC(10)      NOT NULL,           /* SCHEDULER_JOB_NET.NET_ID */
   "UUID"                      VARCHAR(42)      NOT NULL,           /* Unique Id */
   "START_TIME"                TIMESTAMP                ,           /* */
   "END_TIME"                  TIMESTAMP                ,           /* */
   "RUN_TIME"                  TIMESTAMP                ,           /* */
   "STATUS"                    NUMERIC(10)              ,           /* */
   "NUMBER_OF_NODES"           NUMERIC(10)      DEFAULT 0 NOT NULL, /* */ 
   "NUMBER_OF_FINISHED_NODES"  NUMERIC(10)      DEFAULT 0 NOT NULL, /* */
   "EXIT_CODE"                 NUMERIC(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"              VARCHAR(255)             ,        
   "JOBNET_ID"                 VARCHAR(255)             ,           /* external (custom) Jobnet id, e.g. Timekey. Need not be unique in the history */
   "JOBNET_ID_PARAMETER_NAME"  VARCHAR(255)             ,        
   PRIMARY KEY ( "HISTORY_ID" )
)';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;


CREATE OR REPLACE FUNCTION tmp() RETURNS VOID AS $$ DECLARE c integer; BEGIN SELECT COUNT(*) INTO c FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_CATALOG=CURRENT_DATABASE() AND TABLE_SCHEMA=CURRENT_SCHEMA() AND UPPER(TABLE_NAME)='SCHEDULER_JOB_NET'; IF c = 0 THEN EXECUTE '
CREATE TABLE SCHEDULER_JOB_NET
(
   "NET_ID"                    SERIAL                   ,         /* Id */
   "SCHEDULER_ID"              VARCHAR(255)     NOT NULL,         /* JobScheduler Id */
   "JOB_NET"                   VARCHAR(255)     NOT NULL,         /* */
   PRIMARY KEY ( "NET_ID" )
)';
END IF; RETURN; END; $$ LANGUAGE plpgsql;
SELECT tmp();
COMMIT;

DROP FUNCTION tmp();
