/* Sybase */


SET QUOTED_IDENTIFIER ON
go
SET CHAINED ON
go
SET CLOSE ON ENDTRAN ON
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET_PLAN'))
EXECUTE ('
CREATE SCHEDULER_JOB_NET_PLAN
(
   "PLAN_ID"                 NUMERIC(10)        IDENTITY,                /* Id */
   "UUID"                    VARCHAR(42)        NOT NULL,                /* Unique Id */
   "NODE_ID"                 NUMERIC(10)        NOT NULL,                /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "SUBNET_ID"               NUMERIC(10)        DEFAULT 0 NOT NULL,      /* */
   "CONNECTOR_ID"            NUMERIC(10)        DEFAULT 0 NOT NULL,      /* */
   "TASK_ID"                 NUMERIC(10)        DEFAULT 0 NOT NULL,      /* Reference e.g. for the history */
   "NODE_TYPE"               VARCHAR(10)            NULL,                /* Type of jobnet node: HEAD, CONNECTOR, JOB */
   "EXIT_CODE"               NUMERIC(10)        DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"            VARCHAR(255)           NULL,        
   "STATUS"                  NUMERIC(2)             NULL,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_WAITER"           NUMERIC(2)             NULL,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_RUNNER"           NUMERIC(2)             NULL,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "STATUS_DISPATCHER"       NUMERIC(2)             NULL,                /* 0 = not processed, 1 = running, 2 = finished, 3 = error, 4 = ignored, 5 = waiting, 6 = dispatching */
   "IS_WAITER_SKIPPED"       NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_RUNNER_SKIPPED"       NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_DISPATCHER_SKIPPED"   NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "IS_RUNNER_ON_DEMAND"     NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "PLANNED_START_TIME"      DATETIME               NULL,                /* */
   "START_TIME"              DATETIME               NULL,                /* */
   "END_TIME"                DATETIME               NULL,                /* */
   "ORDER_XML"               TEXT                   NULL,                /* xml definition of the order */
   "BOOTSTRAP"               NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "FREQUENCE_OVERWRITE"     NUMERIC(1)         DEFAULT 0 NOT NULL,      /* */
   "MESSAGE"                 VARCHAR(255)           NULL,                /* */
   "CREATED"                 DATETIME               NULL,                /* */
   "CREATED_BY"              VARCHAR(255)           NULL,                /* */
   "MODIFIED"                DATETIME               NULL,                /* */
   "MODIFIED_BY"             VARCHAR(255)           NULL,                /* */
   CONSTRAINT SCHEDULER_JOB_NET_PLAN_UNIQUE UNIQUE( "UUID", "NODE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
)
LOCK DATAROWS
')
go

IF NOT EXISTS (SELECT "id" FROM sysindexes WHERE "name" = 'JOBNET_UUID')
  CREATE INDEX JOBNET_UUID ON SCHEDULER_JOB_NET_PLAN ( "UUID" )
go

IF NOT EXISTS (SELECT "id" FROM sysindexes WHERE "name" = 'JOBNET_NODE_TYPE')
  CREATE INDEX JOBNET_NODE_TYPE ON SCHEDULER_JOB_NET_PLAN ( "NODE_TYPE" )
go

IF NOT EXISTS (SELECT "id" FROM sysindexes WHERE "name" = 'JOBNET_CONNECTOR_ID')
  CREATE INDEX JOBNET_CONNECTOR_ID ON SCHEDULER_JOB_NET_PLAN ( "CONNECTOR_ID" )
go
  
IF NOT EXISTS (SELECT "id" FROM sysindexes WHERE "name" = 'JOBNET_SUBNET_ID')
  CREATE INDEX JOBNET_SUBNET_ID ON SCHEDULER_JOB_NET_PLAN ( "SUBNET_ID" )
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET_NODES'))
EXECUTE ('
CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_NODES
(
   "NODE_ID"             NUMERIC(10)     IDENTITY,                   /* Id */
   "SCHEDULER_ID"        VARCHAR(255)    NOT NULL,                   /* JobScheduler Id */
   "NODE"                VARCHAR(255)    NOT NULL,                   /* Name of the job net node */
   "JOB_NET"             VARCHAR(255)    NOT NULL,                   /* Name of the job net */
   CONSTRAINT SCHEDULER_JOB_NET_NODES_UNIQUE UNIQUE( "SCHEDULER_ID", "NODE", "JOB_NET" ),
   PRIMARY KEY ( "NODE_ID" )
)
LOCK DATAROWS
')
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET_EDGES'))
EXECUTE ('
CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_EDGES
(
   "EDGE_ID"             NUMERIC(10)         IDENTITY,                 /* Id */
   "EDGE_TYPE"           CHAR(1)             NOT NULL,                 /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHEDULER_JOB_NET_EDGES_UNIQUE UNIQUE( "EDGE_TYPE", "PARENT_NODE_ID", "CHILD_NODE_ID" ),
   PRIMARY KEY ( "EDGE_ID" )
)
LOCK DATAROWS
')
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET_PLAN_EDGES'))
EXECUTE ('
CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_PLAN_EDGES
(
   "PLAN_ID"             NUMERIC(10)         IDENTITY,                 /* Id */
   "UUID"                VARCHAR(42)         NOT NULL,                 /* Unique Id */
   "EDGE_ID"             NUMERIC(10)         NOT NULL,                 /* Id */
   "EDGE_TYPE"           CHAR(1)             NOT NULL,                 /* (P)redecessor, (S)uccessor */
   "PARENT_NODE_ID"      NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID */
   "CHILD_NODE_ID"       NUMERIC(10)         NOT NULL,                 /* SCHEDULER_JOB_NET_NODES.NODE_ID  of predecessor or successor node */
   CONSTRAINT SCHED_JOBNET_PLAN_EDGES_UNIQUE UNIQUE( "UUID", "EDGE_ID" ),
   PRIMARY KEY ( "PLAN_ID" )
)
LOCK DATAROWS
')
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET_HISTORY'))
EXECUTE ('
CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET_HISTORY
(
   "HISTORY_ID"                NUMERIC(10)      IDENTITY,         /* Id */
   "SCHEDULER_ID"              VARCHAR(255)     NOT NULL,         /* JobScheduler Id */
   "NET_ID"                    NUMERIC(10)      NOT NULL,         /* SCHEDULER_JOB_NET.NET_ID */
   "UUID"                      VARCHAR(42)      NOT NULL,         /* Unique Id */
   "START_TIME"                DATETIME             NULL,         /* */
   "END_TIME"                  DATETIME             NULL,         /* */
   "RUN_TIME"                  DATETIME             NULL,         /* */
   "STATUS"                    NUMERIC(10)          NULL,         /* */
   "NUMBER_OF_NODES"           NUMERIC(10)      DEFAULT 0 NOT NULL, /* */ 
   "NUMBER_OF_FINISHED_NODES"  NUMERIC(10)      DEFAULT 0 NOT NULL, /* */
   "EXIT_CODE"                 NUMERIC(10)      DEFAULT 0 NOT NULL,  
   "EXIT_MESSAGE"              VARCHAR(255)         NULL,        
   "JOBNET_ID"                 VARCHAR(255)         NULL,         /* external (custom) Jobnet id, e.g. Timekey. Need not be unique in the history */
   "JOBNET_ID_PARAMETER_NAME"  VARCHAR(255)         NULL,        
   PRIMARY KEY ( "HISTORY_ID" )
)
LOCK DATAROWS
')
go


IF NOT EXISTS (SELECT "id" FROM sysobjects WHERE "id" = OBJECT_ID(N'SCHEDULER_JOB_NET'))
EXECUTE ('
CREATE TABLE IF NOT EXISTS SCHEDULER_JOB_NET
(
   "NET_ID"                    NUMERIC(10)      IDENTITY,         /* Id */
   "SCHEDULER_ID"              VARCHAR(255)     NOT NULL,         /* JobScheduler Id */
   "JOB_NET"                   VARCHAR(255)     NOT NULL,         /* */
   PRIMARY KEY ( "NET_ID" )
)
LOCK DATAROWS
')
go

SELECT COUNT(*) FROM SCHEDULER_JOB_NET_NODES
go
COMMIT
go
