package com.sos.scheduler.engine.common.xml

import com.sos.scheduler.engine.common.scalautil.Logger
import javax.xml.parsers.{DocumentBuilderFactory, ParserConfigurationException, SAXParserFactory}
import org.xml.sax.SAXNotRecognizedException

/**
 * @see https://www.owasp.org/index.php/XML_External_Entity_%28XXE%29_Processing
 */
object XxeVulnerability {
  private val logger = Logger(getClass)

  /*
    dbf.setValidating(false);
    dbf.setFeature("http://xml.org/sax/features/namespaces", false);
    dbf.setFeature("http://xml.org/sax/features/validation", false);
    dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
    dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
   */
  private val XmlOrgSettings = List(
    "http://xml.org/sax/features/external-general-entities" → false,
    "http://xml.org/sax/features/external-parameter-entities" → false,
    "http://apache.org/xml/features/disallow-doctype-decl" → true)
  private val OptionalSettings = List(
      "http://xerces.apache.org/xerces-j/features.html#external-general-entities" → false,
      "http://xerces.apache.org/xerces-j/features.html#external-parameter-entities" → false,
      "http://xerces.apache.org/xerces2-j/features.html#external-general-entities" → false,
      "http://xerces.apache.org/xerces2-j/features.html#external-parameter-entities" → false,
      "http://xerces.apache.org/xerces2-j/features.html#disallow-doctype-decl" → true)

  def inhibitFor(factory: DocumentBuilderFactory): Unit = {
    factory.setExpandEntityReferences(false)
    XmlOrgSettings.foreach { case (k, v) ⇒ factory.setFeature(k, v) }
    OptionalSettings foreach { case (k, v) ⇒
      try factory.setFeature(k, v)
      catch { case t: ParserConfigurationException ⇒ logger.trace(t.toString) }
    }
  }

  def inhibitFor(factory: SAXParserFactory): Unit = {
    XmlOrgSettings.foreach { case (k, v) ⇒ factory.setFeature(k, v) }
    OptionalSettings foreach { case (k, v) ⇒
      try factory.setFeature(k, v)
      catch { case t: SAXNotRecognizedException ⇒ logger.trace(t.toString) }
    }
  }
}
